@@ -26,7 +26,8 @@
 #include "gossip-utils.h"
 #include "gossip-contact.h"
 
-#define d(x)
+#define DEBUG_MSG(x) 
+/* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n"); */
 
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_CONTACT, GossipContactPriv))
 
@@ -38,32 +39,32 @@ typedef struct _GossipContactPriv GossipContactPriv;
 struct _GossipContactPriv {
 	GossipContactType   type;
 
-	gchar              *name;
-
 	gchar              *id;
+	gchar              *name;
 
 	GList              *presences;
 	GList              *groups;
 
 	GossipSubscription  subscription;
 
+	guchar             *avatar;
+	guint               avatar_size;
+
 	GossipAccount      *account;
 };
 
+static void contact_class_init   (GossipContactClass *class);
+static void contact_init         (GossipContact      *contact);
+static void contact_finalize     (GObject            *object);
+static void contact_get_property (GObject            *object,
+				  guint               param_id,
+				  GValue             *value,
+				  GParamSpec         *pspec);
+static void contact_set_property (GObject            *object,
+				  guint               param_id,
+				  const GValue       *value,
+				  GParamSpec         *pspec);
 
-static void contact_class_init        (GossipContactClass   *class);
-static void contact_init              (GossipContact        *contact);
-static void contact_finalize          (GObject              *object);
-static void contact_get_property      (GObject              *object,
-				       guint                 param_id,
-				       GValue               *value,
-				       GParamSpec           *pspec);
-static void contact_set_property      (GObject              *object,
-				       guint                 param_id,
-				       const GValue         *value,
-				       GParamSpec           *pspec);
-
-/* properties */
 enum {
 	PROP_0,
 	PROP_TYPE,
@@ -71,6 +72,8 @@ enum {
 	PROP_ID,
 	PROP_ACTIVE_PRESENCE,
 	PROP_SUBSCRIPTION,
+	PROP_AVATAR,
+	PROP_AVATAR_SIZE,
 	PROP_ACCOUNT
 };
 
@@ -147,6 +150,7 @@ contact_class_init (GossipContactClass *class)
 							      "Current presence of contact",
 							      GOSSIP_TYPE_PRESENCE,
 							      G_PARAM_READABLE));
+
 	g_object_class_install_property (object_class,
 					 PROP_SUBSCRIPTION,
 					 g_param_spec_int ("subscription",
@@ -157,6 +161,22 @@ contact_class_init (GossipContactClass *class)
 							   GOSSIP_SUBSCRIPTION_NONE,
 							   G_PARAM_READWRITE));
 
+	g_object_class_install_property (object_class,
+					 PROP_AVATAR,
+					 g_param_spec_pointer ("avatar",
+							       "Avatar image",
+							       "The avatar image",
+							       G_PARAM_READWRITE));						   
+	g_object_class_install_property (object_class,
+					 PROP_AVATAR_SIZE,
+					 g_param_spec_uint ("avatar_size",
+							    "Avatar size",
+							    "The size of the avatar image",
+							    0,
+							    G_MAXUINT,
+							    0,
+							    G_PARAM_READWRITE));
+
 	g_object_class_install_property (object_class,
 					 PROP_ACCOUNT,
 					 g_param_spec_object ("account",
@@ -175,14 +195,14 @@ contact_init (GossipContact *contact)
 
 	priv = GET_PRIV (contact);
 	
-	priv->type      = GOSSIP_CONTACT_TYPE_TEMPORARY;
-	priv->name      = NULL;
-	priv->id        = NULL;
-	priv->presences = NULL;
-
-	priv->account   = NULL;
-
-        priv->groups    = NULL;
+	priv->type        = GOSSIP_CONTACT_TYPE_TEMPORARY;
+	priv->name        = NULL;
+	priv->id          = NULL;
+	priv->presences   = NULL;
+	priv->account     = NULL;
+	priv->groups      = NULL;
+	priv->avatar      = NULL;
+	priv->avatar_size = 0;
 }
 
 static void
@@ -194,6 +214,7 @@ contact_finalize (GObject *object)
 	
 	g_free (priv->name);
 	g_free (priv->id);
+	g_free (priv->avatar);
 
 	if (priv->presences) {
 		GList *l;
@@ -217,7 +238,7 @@ contact_finalize (GObject *object)
 	
 	g_object_unref (priv->account);
 	
-	(* G_OBJECT_CLASS (parent_class)->finalize) (object);
+	(G_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
 static void
@@ -252,6 +273,12 @@ contact_get_property (GObject    *object,
 	case PROP_SUBSCRIPTION:
 		g_value_set_int (value, priv->subscription);
 		break;
+	case PROP_AVATAR:
+		g_value_set_pointer (value, priv->avatar);
+		break;		
+	case PROP_AVATAR_SIZE:
+		g_value_set_uint (value, priv->avatar_size);
+		break;
 	case PROP_ACCOUNT:
 		if (priv->account) {
 			g_value_set_object (value, G_OBJECT (priv->account));
@@ -290,6 +317,12 @@ contact_set_property (GObject      *object,
 	case PROP_SUBSCRIPTION:
 		priv->subscription = g_value_get_int (value);
 		break;
+	case PROP_AVATAR:
+ 		priv->avatar = g_value_get_pointer (value); 
+		break;		
+	case PROP_AVATAR_SIZE:
+		priv->avatar_size = g_value_get_uint (value);
+		break;
 	case PROP_ACCOUNT:
 		gossip_contact_set_account (GOSSIP_CONTACT (object), 
 					    GOSSIP_ACCOUNT (g_value_get_object (value)));
@@ -403,6 +436,24 @@ gossip_contact_get_name (GossipContact *contact)
 	return priv->name;
 }
 
+const guchar *
+gossip_contact_get_avatar (GossipContact *contact, 
+			   gsize         *avatar_size)
+{
+	GossipContactPriv *priv;
+	guchar            *avatar;
+
+	g_return_val_if_fail (GOSSIP_IS_CONTACT (contact), NULL);
+	g_return_val_if_fail (avatar_size != NULL, NULL);
+	
+	priv = GET_PRIV (contact);
+
+	avatar = (guchar*) priv->avatar;
+	*avatar_size = priv->avatar_size;
+	
+	return avatar;
+}
+
 GossipAccount *
 gossip_contact_get_account (GossipContact *contact)
 {
@@ -506,6 +557,8 @@ gossip_contact_set_id (GossipContact *contact,
 
 	g_free (priv->id);
 	priv->id = g_strdup (id);
+
+	g_object_notify (G_OBJECT (contact), "id");
 }
 
 void
@@ -521,6 +574,33 @@ gossip_contact_set_name (GossipContact *contact,
 
 	g_free (priv->name);
 	priv->name = g_strdup (name);
+
+	g_object_notify (G_OBJECT (contact), "name"); 
+}
+
+void 
+gossip_contact_set_avatar (GossipContact *contact, 
+			   const guchar  *avatar,
+			   gsize          avatar_size)
+{
+	GossipContactPriv *priv;
+
+	g_return_if_fail (GOSSIP_IS_CONTACT (contact));
+
+	priv = GET_PRIV (contact);
+	
+	g_free (priv->avatar);
+
+	if (avatar) {
+		priv->avatar = g_memdup (avatar, avatar_size);
+		priv->avatar_size = avatar_size;
+	} else {
+		priv->avatar = NULL;
+		priv->avatar_size = 0;
+	}
+
+	g_object_notify (G_OBJECT (contact), "avatar"); 
+	g_object_notify (G_OBJECT (contact), "avatar-size"); 
 }
 
 void
@@ -538,7 +618,13 @@ gossip_contact_set_account (GossipContact *contact,
 		g_object_unref (priv->account);
 	}
 	
-	priv->account = g_object_ref (account);
+	if (account) {
+		priv->account = g_object_ref (account);
+	} else {
+		priv->account = NULL;
+	}
+
+	g_object_notify (G_OBJECT (contact), "account"); 
 }
 
 void 
@@ -603,29 +689,31 @@ gossip_contact_remove_presence (GossipContact  *contact,
 
 	priv->presences = g_list_sort (priv->presences,
 				       gossip_presence_priority_sort_func);
-	/* highest priority first */
+	/* Highest priority first */
 	priv->presences = g_list_reverse (priv->presences);
 }
 
 gboolean
-gossip_contact_set_groups (GossipContact *contact, GList *groups)
+gossip_contact_set_groups (GossipContact *contact, 
+			   GList         *groups)
 {
 	GossipContactPriv *priv;
-	GList             *l;
+	GList             *old_groups, *l;
 
 	g_return_val_if_fail (GOSSIP_IS_CONTACT (contact), FALSE);
 	
 	priv = GET_PRIV (contact);
 
-	if (priv->groups) {
-		g_list_foreach (priv->groups, (GFunc)g_free, NULL);
-		priv->groups = NULL;
-	}
+	old_groups = priv->groups;
+	priv->groups = NULL;
 
 	for (l = groups; l; l = l->next) {
 		priv->groups = g_list_append (priv->groups, 
 					      g_strdup (l->data));
 	}
+	
+	g_list_foreach (old_groups, (GFunc) g_free, NULL);
+	g_list_free (old_groups);
 
 	return TRUE;
 }
@@ -641,10 +729,10 @@ gossip_contact_set_subscription (GossipContact      *contact,
 	priv = GET_PRIV (contact);
 
 	priv->subscription = subscription;
+
+	g_object_notify (G_OBJECT (contact), "subscription");
 }
 
-
-
 gint
 gossip_contact_compare (gconstpointer a,
 			gconstpointer b)
@@ -658,7 +746,7 @@ gossip_contact_compare (gconstpointer a,
 	id_a = gossip_contact_get_id (GOSSIP_CONTACT (a));
 	id_b = gossip_contact_get_id (GOSSIP_CONTACT (b));
 
-	/* FIXME: Maybe use utf8 strcmp? */
+	/* FIXME: Maybe use utf8 strcmp? At least jabber ids are UTF8. */
 	return strcmp (id_a, id_b);
 }
 
@@ -669,9 +757,9 @@ gossip_contact_name_compare (gconstpointer a,
 	g_return_val_if_fail (GOSSIP_IS_CONTACT (a), 0);
 	g_return_val_if_fail (GOSSIP_IS_CONTACT (b), 0);
 	
-	d(g_print ("COMPARE: %s vs. %s\n",
-		   gossip_contact_get_name (GOSSIP_CONTACT (a)),
-		   gossip_contact_get_name (GOSSIP_CONTACT (b))));
+	DEBUG_MSG (("Contact: Comparing %s vs. %s",
+		    gossip_contact_get_name (GOSSIP_CONTACT (a)),
+		    gossip_contact_get_name (GOSSIP_CONTACT (b))));
 
 	return strcmp (gossip_contact_get_name (GOSSIP_CONTACT (a)),
 		       gossip_contact_get_name (GOSSIP_CONTACT (b)));
@@ -700,7 +788,7 @@ gossip_contact_name_case_n_compare (gconstpointer a,
 	name_a = gossip_contact_get_name (GOSSIP_CONTACT (a));
 	name_b = gossip_contact_get_name (GOSSIP_CONTACT (b));
 					  
-	return gossip_utils_str_n_case_cmp (name_a, name_b, -1);
+	return gossip_strncasecmp (name_a, name_b, -1);
 }
 
 gboolean
@@ -722,7 +810,7 @@ gossip_contact_equal (gconstpointer v1,
 guint
 gossip_contact_hash (gconstpointer key)
 {
-	g_return_val_if_fail (GOSSIP_IS_CONTACT (key), 0);
+	g_return_val_if_fail (GOSSIP_IS_CONTACT (key), +1);
 
 	return g_str_hash (gossip_contact_get_id (GOSSIP_CONTACT (key)));
 }
