@@ -19,15 +19,13 @@
  */
 
 #include <config.h>
-
 #include <string.h>
 
 #include "libgossip-marshal.h"
-
+#include "gossip-debug.h"
 #include "gossip-session.h"
 
-#define DEBUG_MSG(x) 
-/* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n"); */
+#define DEBUG_DOMAIN "Session"
 
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_SESSION, GossipSessionPriv))
 
@@ -44,6 +42,7 @@ struct _GossipSessionPriv {
 	GList                *contacts;
 
 	guint                 connected_counter;
+	guint                 connecting_counter;
 
 	GHashTable           *timers; /* connected time */
 };
@@ -56,6 +55,7 @@ struct FindAccount {
 struct CountAccounts {
 	GossipSession *session;
 	guint          connected;
+	guint          connecting;
 	guint          disconnected;
 };
 
@@ -66,9 +66,16 @@ struct GetAccounts {
 
 struct ConnectData {
 	GossipSession *session;
+	gboolean       connect_all;
 	gboolean       startup;
 };
 
+struct ConnectFind {
+	gboolean       first;
+	gboolean       last;
+	gboolean       same;
+};
+
 static void            gossip_session_class_init                 (GossipSessionClass   *klass);
 static void            gossip_session_init                       (GossipSession        *session);
 static void            session_finalize                          (GObject              *object);
@@ -125,6 +132,9 @@ static void            session_account_removed_cb                (GossipAccountM
 								  GossipSession        *session);
 static void            session_connect                           (GossipSession        *session,
 								  GossipAccount        *account);
+static void            session_connect_match_foreach_cb          (GossipAccount        *account,
+								  GossipProtocol       *protocol,
+								  struct ConnectFind   *cf);
 static void            session_connect_foreach_cb                (GossipAccount        *account,
 								  GossipProtocol       *protocol,
 								  struct ConnectData   *cd);
@@ -212,9 +222,9 @@ gossip_session_class_init (GossipSessionClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__OBJECT,
+			      libgossip_marshal_VOID__OBJECT_OBJECT,
 			      G_TYPE_NONE, 
-			      1, GOSSIP_TYPE_ACCOUNT);
+			      2, GOSSIP_TYPE_ACCOUNT, GOSSIP_TYPE_PROTOCOL);
 
 	signals[PROTOCOL_CONNECTED] = 
 		g_signal_new ("protocol-connected",
@@ -242,9 +252,9 @@ gossip_session_class_init (GossipSessionClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      0,
 			      NULL, NULL,
-			      libgossip_marshal_VOID__OBJECT,
+			      libgossip_marshal_VOID__OBJECT_OBJECT,
 			      G_TYPE_NONE, 
-			      1, GOSSIP_TYPE_ACCOUNT);
+			      2, GOSSIP_TYPE_ACCOUNT, GOSSIP_TYPE_PROTOCOL);
 	
 	signals[PROTOCOL_ERROR] = 
 		g_signal_new ("protocol-error",
@@ -386,6 +396,10 @@ session_finalize (GObject *object)
 		g_list_free (priv->contacts);
 	}
 
+	if (priv->presence) {
+		g_object_unref (priv->presence);
+	}
+
 	if (priv->timers) {
 		g_hash_table_destroy (priv->timers);
 	}
@@ -449,11 +463,11 @@ session_protocol_logged_in (GossipProtocol *protocol,
 	GossipSessionPriv *priv;
 	GTimer            *timer;
 
-	DEBUG_MSG (("Session: Protocol logged in"));
+	gossip_debug (DEBUG_DOMAIN, "Protocol logged in");
 
 	priv = GET_PRIV (session);
 
-	/* setup timer */
+	/* Setup timer */
 	timer = g_timer_new ();
 	g_timer_start (timer);
 
@@ -461,9 +475,13 @@ session_protocol_logged_in (GossipProtocol *protocol,
 			     g_object_ref (account), 
 			     timer);
 	
-	/* update some status? */
+	/* Update some status? */
 	priv->connected_counter++;
 
+	if (priv->connecting_counter > 0) {
+		priv->connecting_counter--;
+	}
+
 	g_signal_emit (session, signals[PROTOCOL_CONNECTED], 0, account, protocol);
 
 	if (priv->connected_counter == 1) {
@@ -481,7 +499,8 @@ session_protocol_logged_out (GossipProtocol *protocol,
 	gdouble            seconds;
 
 	seconds = gossip_session_get_connected_time (session, account);
-	DEBUG_MSG (("Session: Protocol logged out (after %.2f seconds)", seconds));
+	gossip_debug (DEBUG_DOMAIN, "Protocol logged out (after %.2f seconds)", 
+		      seconds);
 
 	priv = GET_PRIV (session);
 
@@ -498,7 +517,11 @@ session_protocol_logged_out (GossipProtocol *protocol,
 	if (priv->connected_counter > 0) {
 		priv->connected_counter--;
 	}
-	
+
+	if (priv->connecting_counter > 0) {
+		priv->connecting_counter--;
+	}
+
 	g_signal_emit (session, signals[PROTOCOL_DISCONNECTED], 0, account, protocol);
 	
 	if (priv->connected_counter == 0) {
@@ -523,8 +546,8 @@ session_protocol_contact_added (GossipProtocol *protocol,
 {
 	GossipSessionPriv *priv;
 
-	DEBUG_MSG (("Session: Contact added '%s'",
-		   gossip_contact_get_name (contact)));
+	gossip_debug (DEBUG_DOMAIN, "Contact added '%s'",
+		      gossip_contact_get_name (contact));
 
 	priv = GET_PRIV (session);
 	
@@ -539,8 +562,8 @@ session_protocol_contact_updated (GossipProtocol *protocol,
 				  GossipContact  *contact,
 				  GossipSession  *session)
 {
-	DEBUG_MSG (("Session: Contact updated '%s'",
-		   gossip_contact_get_name (contact)));
+	gossip_debug (DEBUG_DOMAIN, "Contact updated '%s'",
+		      gossip_contact_get_name (contact));
 
 	g_signal_emit (session, signals[CONTACT_UPDATED], 0, contact);
 }
@@ -550,8 +573,8 @@ session_protocol_contact_presence_updated (GossipProtocol *protocol,
 					   GossipContact  *contact,
 					   GossipSession  *session)
 {
-	DEBUG_MSG (("Session: Contact presence updated '%s'",
-		   gossip_contact_get_name (contact)));
+	gossip_debug (DEBUG_DOMAIN, "Contact presence updated '%s'",
+		      gossip_contact_get_name (contact));
 	g_signal_emit (session, signals[CONTACT_PRESENCE_UPDATED], 0, contact);
 }
 
@@ -562,8 +585,8 @@ session_protocol_contact_removed (GossipProtocol *protocol,
 {
 	GossipSessionPriv *priv;
 	
-	DEBUG_MSG (("Session: Contact removed '%s'",
-		   gossip_contact_get_name (contact)));
+	gossip_debug (DEBUG_DOMAIN, "Contact removed '%s'",
+		   gossip_contact_get_name (contact));
 
 	priv = GET_PRIV (session);
 	
@@ -581,9 +604,9 @@ session_protocol_composing (GossipProtocol *protocol,
 {
 	GossipSessionPriv *priv;
 	
-	DEBUG_MSG (("Session: Contact %s composing:'%s'",
-		   composing ? "is" : "is not",
-		   gossip_contact_get_name (contact)));
+	gossip_debug (DEBUG_DOMAIN, "Contact %s composing:'%s'",
+		      composing ? "is" : "is not",
+		      gossip_contact_get_name (contact));
 
 	priv = GET_PRIV (session);
 	
@@ -597,7 +620,7 @@ session_protocol_get_password (GossipProtocol *protocol,
 {
 	gchar *password = NULL;
 
-	DEBUG_MSG (("Session: Get password"));
+	gossip_debug (DEBUG_DOMAIN, "Get password");
 
 	g_signal_emit (session, signals[GET_PASSWORD], 0, account, &password);
 	
@@ -610,9 +633,16 @@ session_protocol_error (GossipProtocol *protocol,
 			GError         *error,
 			GossipSession  *session)
 {
+	GossipSessionPriv *priv;
 
-	DEBUG_MSG (("Session: Error:%d->'%s'", 
-		   error->code, error->message));
+	gossip_debug (DEBUG_DOMAIN, "Error:%d->'%s'", 
+		      error->code, error->message);
+
+	priv = GET_PRIV (session);
+
+	if (priv->connecting_counter > 0) {
+		priv->connecting_counter--;
+	}
 
 	g_signal_emit (session, signals[PROTOCOL_ERROR], 0, protocol, account, error); 
 }
@@ -772,6 +802,7 @@ gossip_session_get_connected_time (GossipSession *session,
 void 
 gossip_session_count_accounts (GossipSession *session,
 			       guint         *connected,
+			       guint         *connecting,
 			       guint         *disconnected)
 {
 	GossipSessionPriv    *priv;
@@ -784,14 +815,25 @@ gossip_session_count_accounts (GossipSession *session,
 	ca.session = session;
 
 	ca.connected = 0;
+	ca.connecting = 0;
 	ca.disconnected = 0;
 
 	g_hash_table_foreach (priv->accounts, 
-			      (GHFunc)session_count_accounts_foreach_cb,
+			      (GHFunc) session_count_accounts_foreach_cb,
 			      &ca);
 
+	/* Here we update our own numbers, should we be doing this? I
+	 * am not sure but fuck it let's do it anyway :)
+	 */
+	priv->connected_counter = ca.connected;
+	priv->connecting_counter = ca.connecting;
+
 	if (connected) {
-		*connected = ca.connected;
+		*connected = priv->connected_counter;
+	}
+
+	if (connecting) {
+		*connecting = priv->connecting_counter;
 	}
 
 	if (disconnected) {
@@ -812,6 +854,10 @@ session_count_accounts_foreach_cb (GossipAccount        *account,
 		ca->connected++;
 	} else {
 		ca->disconnected++;
+
+		if (gossip_protocol_is_connecting (protocol)) {
+			ca->connecting++;
+		}
 	}
 }
 
@@ -934,12 +980,84 @@ session_account_removed_cb (GossipAccountManager *manager,
 	gossip_session_remove_account (session, account);
 }
 
+static void
+session_connect (GossipSession *session,
+		 GossipAccount *account) 
+{
+	GossipSessionPriv *priv;
+	GossipProtocol    *protocol;
+
+	priv = GET_PRIV (session);
+
+	protocol = g_hash_table_lookup (priv->accounts, account);
+	
+	if (gossip_protocol_is_connected (protocol)) {
+		return;
+	}
+	
+	g_signal_emit (session, signals[PROTOCOL_CONNECTING], 0, account, protocol);
+	priv->connecting_counter++;
+
+	/* Can we not just pass the GossipAccount on the GObject init? */
+	gossip_protocol_setup (protocol, account);
+	
+	/* Setup the network connection */
+	gossip_protocol_login (protocol);
+}
+
+static void
+session_connect_match_foreach_cb (GossipAccount      *account,
+				  GossipProtocol     *protocol,
+				  struct ConnectFind *cf)
+{
+	if (!cf->first && !cf->same) {
+		return;
+	}
+
+	if (cf->first) {
+		cf->first = FALSE;
+		cf->last = gossip_account_get_auto_connect (account);
+		return;
+	}
+
+	if (gossip_account_get_auto_connect (account) == cf->last) {
+		cf->same = TRUE;
+	} else {
+		cf->same = FALSE;
+	}
+}
+
+static void
+session_connect_foreach_cb (GossipAccount      *account,
+			    GossipProtocol     *protocol,
+			    struct ConnectData *cd)
+{
+	/* Connect it if this is a startup request and we have an auto
+	 * connecting account.
+	 */
+	if (cd->startup) {
+		if (gossip_account_get_auto_connect (account)) { 
+			session_connect (cd->session, account);
+		}
+		return; 
+	} 
+
+	/* Connect it if we should be connecting ALL accounts anyway. */
+	if (cd->connect_all) {
+		session_connect (cd->session, account);
+	} 
+	else if (gossip_account_get_auto_connect (account)) {
+		session_connect (cd->session, account);
+	}
+}
+
 void
 gossip_session_connect (GossipSession *session,
 			GossipAccount *account,
 			gboolean       startup)
 {
 	GossipSessionPriv  *priv;
+	struct ConnectFind  cf;
 	struct ConnectData  cd;
 	
 	g_return_if_fail (GOSSIP_IS_SESSION (session));
@@ -958,12 +1076,23 @@ gossip_session_connect (GossipSession *session,
 
 	/* Connect one account if provided */
 	if (account) {
-		g_return_if_fail (GOSSIP_IS_ACCOUNT (account));
-		
 		session_connect (session, account);
 		return;
 	}
 
+	/* We do some clever stuff here and check that ALL accounts
+	 * are either auto connecting or not, if they are all the same
+	 * then we connect ALL accounts, if not, we just connect the
+	 * auto connect accounts 
+	 */
+	cf.first = TRUE;
+	cf.same = TRUE;
+
+	g_hash_table_foreach (priv->accounts,
+			      (GHFunc) session_connect_match_foreach_cb,
+			      &cf);
+
+	cd.connect_all = cf.same;
 	cd.session = session;
 	cd.startup = startup;
 
@@ -974,40 +1103,33 @@ gossip_session_connect (GossipSession *session,
 }
 
 static void
-session_connect (GossipSession *session,
-		 GossipAccount *account) 
+session_disconnect (GossipSession *session,
+		    GossipAccount *account) 
 {
 	GossipSessionPriv *priv;
 	GossipProtocol    *protocol;
 
 	priv = GET_PRIV (session);
-
+	
+	g_return_if_fail (GOSSIP_IS_ACCOUNT (account));
+	
 	protocol = g_hash_table_lookup (priv->accounts, account);
 	
-	if (gossip_protocol_is_connected (protocol)) {
-		return;
-	}
+	g_signal_emit (session, signals[PROTOCOL_DISCONNECTING], 0, account, protocol);
 	
-	g_signal_emit (session, signals[PROTOCOL_CONNECTING], 0, account);
-
-	/* can we not just pass the GossipAccount on the GObject init? */
-	gossip_protocol_setup (protocol, account);
-	
-	/* setup the network connection */
-	gossip_protocol_login (protocol);
+	gossip_protocol_logout (protocol);
 }
 
 static void
-session_connect_foreach_cb (GossipAccount      *account,
-			    GossipProtocol     *protocol,
-			    struct ConnectData *cd)
+session_disconnect_foreach_cb (GossipAccount  *account,
+			       GossipProtocol *protocol,
+			       GossipSession  *session)
 {
- 	if (cd->startup && 
-	    !gossip_account_get_auto_connect (account)) { 
- 		return; 
- 	} 
+	GossipSessionPriv *priv;
 
-	session_connect (cd->session, account);
+	priv = GET_PRIV (session);
+	
+	session_disconnect (session, account);
 }
 
 void
@@ -1034,37 +1156,6 @@ gossip_session_disconnect (GossipSession *session,
 			      session);
 }
 
-
-static void
-session_disconnect (GossipSession *session,
-		    GossipAccount *account) 
-{
-	GossipSessionPriv *priv;
-	GossipProtocol    *protocol;
-
-	priv = GET_PRIV (session);
-	
-	g_return_if_fail (GOSSIP_IS_ACCOUNT (account));
-	
-	protocol = g_hash_table_lookup (priv->accounts, account);
-	
-	g_signal_emit (session, signals[PROTOCOL_DISCONNECTING], 0, account);
-	
-	gossip_protocol_logout (protocol);
-}
-
-static void
-session_disconnect_foreach_cb (GossipAccount  *account,
-			       GossipProtocol *protocol,
-			       GossipSession  *session)
-{
-	GossipSessionPriv *priv;
-
-	priv = GET_PRIV (session);
-	
-	session_disconnect (session, account);
-}
-
 void 
 gossip_session_send_message (GossipSession *session, 
 			     GossipMessage *message)
@@ -1195,9 +1286,34 @@ gossip_session_is_connected (GossipSession *session,
 	} 
 
 	/* Fall back to counter if no account is provided */
+	gossip_session_count_accounts (session, NULL, NULL, NULL);
+
 	return (priv->connected_counter > 0);
 }
 
+gboolean
+gossip_session_is_connecting (GossipSession *session,
+			      GossipAccount *account)
+{
+	GossipSessionPriv *priv;
+	GossipProtocol    *protocol;
+
+	g_return_val_if_fail (GOSSIP_IS_SESSION (session), FALSE);
+	
+	priv = GET_PRIV (session);
+
+	if (account) {
+		protocol = g_hash_table_lookup (priv->accounts, account);
+		
+		return gossip_protocol_is_connecting (protocol);
+	} 
+
+	/* Fall back to counter if no account is provided */
+	gossip_session_count_accounts (session, NULL, NULL, NULL);
+
+	return (priv->connecting_counter > 0);
+}
+
 const gchar *
 gossip_session_get_active_resource (GossipSession *session,
 				    GossipContact *contact)
@@ -1686,6 +1802,6 @@ gossip_session_get_version (GossipSession          *session,
 	}
 	
 	return gossip_protocol_get_version (protocol, contact, 
-						  callback, user_data,
-						  error);
+					    callback, user_data,
+					    error);
 }
