@@ -35,34 +35,23 @@
 #include "gossip-vcard.h"
 
 #define GOSSIP_TYPE_SESSION         (gossip_session_get_type ())
-#define GOSSIP_SESSION(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), \
- 			             GOSSIP_TYPE_SESSION, GossipSession))
-#define GOSSIP_SESSION_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST((k), \
-				     GOSSIP_TYPE_SESSION, \
-			             GossipSessionClass))
-#define GOSSIP_IS_SESSION(o)       (G_TYPE_CHECK_INSTANCE_TYPE ((o),\
-			             GOSSIP_TYPE_SESSION))
-#define GOSSIP_IS_SESSION_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), \
-				     GOSSIP_TYPE_SESSION))
-#define GOSSIP_SESSION_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o),\
-				     GOSSIP_TYPE_SESSION, \
-				     GossipSessionClass))
-
+#define GOSSIP_SESSION(o)           (G_TYPE_CHECK_INSTANCE_CAST ((o), GOSSIP_TYPE_SESSION, GossipSession))
+#define GOSSIP_SESSION_CLASS(k)     (G_TYPE_CHECK_CLASS_CAST((k), GOSSIP_TYPE_SESSION, GossipSessionClass))
+#define GOSSIP_IS_SESSION(o)       (G_TYPE_CHECK_INSTANCE_TYPE ((o), GOSSIP_TYPE_SESSION))
+#define GOSSIP_IS_SESSION_CLASS(k) (G_TYPE_CHECK_CLASS_TYPE ((k), GOSSIP_TYPE_SESSION))
+#define GOSSIP_SESSION_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS ((o), GOSSIP_TYPE_SESSION, GossipSessionClass))
 
 typedef struct _GossipSession      GossipSession;
 typedef struct _GossipSessionClass GossipSessionClass;
 
-
 struct _GossipSession {
 	GObject parent;
 };
 
-
 struct _GossipSessionClass {
 	GObjectClass parent_class;
 };
 
-
 GType           gossip_session_get_type                (void) G_GNUC_CONST;
 GossipSession * gossip_session_new                     (GossipAccountManager    *manager);
 
@@ -87,7 +76,7 @@ gdouble         gossip_session_get_connected_time      (GossipSession
 
 void            gossip_session_count_accounts          (GossipSession           *session,
 							guint                   *connected,
-							guint                   *connected_total,
+							guint                   *connecting,
 							guint                   *disconnected);
 
 gboolean        gossip_session_add_account             (GossipSession           *session,
@@ -103,6 +92,8 @@ void            gossip_session_disconnect              (GossipSession
 							GossipAccount           *account);
 gboolean        gossip_session_is_connected            (GossipSession           *session,
 							GossipAccount           *account);
+gboolean        gossip_session_is_connecting           (GossipSession           *session,
+							GossipAccount           *account);
 void            gossip_session_send_message            (GossipSession           *session,
 							GossipMessage           *message);
 void            gossip_session_send_composing          (GossipSession           *session,
