@@ -31,15 +31,12 @@
 #define DEBUG_MSG(x)   
 /* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n");   */
 
-static void utils_regex_init (void);
-static void utils_free_entry (GossipStatusEntry *entry);
+static void regex_init                  (void);
+static void status_message_free_foreach (GossipStatusEntry *entry);
 
-#define AVAILABLE_MESSAGE "Available"
-#define AWAY_MESSAGE      "Away"
-#define BUSY_MESSAGE      "Busy"
 
 GList *
-gossip_utils_get_status_messages (void)
+gossip_status_messages_get (void)
 {
 	GConfClient       *gconf_client;
 	GSList            *list, *l;
@@ -92,7 +89,7 @@ gossip_utils_get_status_messages (void)
 }
 
 void 
-gossip_utils_set_status_messages (GList *list)
+gossip_status_messages_set (GList *list)
 {
 	GConfClient       *gconf_client;
 	GList             *l;
@@ -136,21 +133,23 @@ gossip_utils_set_status_messages (GList *list)
 }
 
 static void
-utils_free_entry (GossipStatusEntry *entry)
+status_message_free_foreach (GossipStatusEntry *entry)
 {
 	g_free (entry->string);
 	g_free (entry);
 }
 
 void
-gossip_utils_free_status_messages (GList *list)
+gossip_status_messages_free (GList *list)
 {
-	g_list_foreach (list, (GFunc) utils_free_entry, NULL);
+	g_list_foreach (list, (GFunc) status_message_free_foreach, NULL);
 	g_list_free (list);
 }
 
 gchar *
-gossip_utils_substring (const gchar *str, gint start, gint end)
+gossip_substring (const gchar *str, 
+		  gint         start, 
+		  gint         end)
 {
 	return g_strndup (str + start, end - start);
 }
@@ -169,7 +168,7 @@ gossip_utils_substring (const gchar *str, gint start, gint end)
 static regex_t dingus[GOSSIP_REGEX_ALL];
 
 static void
-utils_regex_init (void)
+regex_init (void)
 {
 	static gboolean  inited = FALSE;
 	const gchar     *expression;
@@ -215,10 +214,10 @@ utils_regex_init (void)
 }
 
 gint
-gossip_utils_regex_match (GossipRegExType  type, 
-			  const gchar     *msg,
-			  GArray          *start,
-			  GArray          *end)
+gossip_regex_match (GossipRegExType  type, 
+		    const gchar     *msg,
+		    GArray          *start,
+		    GArray          *end)
 {
 	regmatch_t matches[1];
 	gint       ret = 0;
@@ -228,7 +227,7 @@ gossip_utils_regex_match (GossipRegExType  type,
 
 	g_return_val_if_fail (type >= 0 || type <= GOSSIP_REGEX_ALL, 0);
 
-	utils_regex_init ();
+	regex_init ();
 
 	while (!ret && type != GOSSIP_REGEX_ALL) {
 		ret = regexec (&dingus[type], msg + offset, 1, matches, 0);
@@ -273,37 +272,17 @@ gossip_utils_regex_match (GossipRegExType  type,
 	return num_matches;
 }
 
-GossipPresenceState
-gossip_utils_get_presence_state_from_show_string (const gchar *str)
-{
-	if (!str || !str[0]) {
-		return GOSSIP_PRESENCE_STATE_AVAILABLE;
-	}
-	else if (strcmp (str, "dnd") == 0) {
-		return GOSSIP_PRESENCE_STATE_BUSY;
-	}
-	else if (strcmp (str, "away") == 0) {
-		return GOSSIP_PRESENCE_STATE_AWAY;
-	}
-	else if (strcmp (str, "xa") == 0) {
-		return GOSSIP_PRESENCE_STATE_EXT_AWAY;
-	}
-	/* We don't support chat, so treat it like available. */
-	else if (strcmp (str, "chat") == 0) {
-		return GOSSIP_PRESENCE_STATE_AVAILABLE;
-	}
-
-	return GOSSIP_PRESENCE_STATE_AVAILABLE;
-}
-
 gint
-gossip_utils_str_case_cmp (const gchar *s1, const gchar *s2)
+gossip_strcasecmp (const gchar *s1, 
+		   const gchar *s2)
 {
-	return gossip_utils_str_n_case_cmp (s1, s2, -1);
+	return gossip_strncasecmp (s1, s2, -1);
 }	
 
 gint
-gossip_utils_str_n_case_cmp (const gchar *s1, const gchar *s2, gsize n)
+gossip_strncasecmp (const gchar *s1, 
+		    const gchar *s2, 
+		    gsize        n)
 {
 	gchar *u1, *u2;
 	gint   ret_val;
@@ -319,7 +298,8 @@ gossip_utils_str_n_case_cmp (const gchar *s1, const gchar *s2, gsize n)
 }
 
 gboolean 
-gossip_utils_xml_validate (xmlDoc *doc, const gchar *dtd_filename)
+gossip_xml_validate (xmlDoc      *doc, 
+		     const gchar *dtd_filename)
 {
 	gchar        *path;
 	xmlValidCtxt  cvp;
@@ -337,3 +317,142 @@ gossip_utils_xml_validate (xmlDoc *doc, const gchar *dtd_filename)
 	
         return ret;
 }
+
+guchar *
+gossip_base64_decode (const char *str, 
+		      gsize      *ret_len)
+{
+	guchar      *out = NULL;
+	gchar        tmp = 0;
+	const gchar *c;
+	gint32       tmp2 = 0;
+	gint         len = 0, n = 0;
+
+	g_return_val_if_fail (str != NULL, NULL);
+
+	c = str;
+
+	while (*c) {
+		if (*c >= 'A' && *c <= 'Z') {
+			tmp = *c - 'A';
+		} else if (*c >= 'a' && *c <= 'z') {
+			tmp = 26 + (*c - 'a');
+		} else if (*c >= '0' && *c <= 57) {
+			tmp = 52 + (*c - '0');
+		} else if (*c == '+') {
+			tmp = 62;
+		} else if (*c == '/') {
+			tmp = 63;
+		} else if (*c == '\r' || *c == '\n') {
+			c++;
+			continue;
+		} else if (*c == '=') {
+			if (n == 3) {
+				out = g_realloc (out, len + 2);
+				out[len] = (guchar)(tmp2 >> 10) & 0xff;
+				len++;
+				out[len] = (guchar)(tmp2 >> 2) & 0xff;
+				len++;
+			} else if (n == 2) {
+				out = g_realloc (out, len + 1);
+				out[len] = (guchar)(tmp2 >> 4) & 0xff;
+				len++;
+			}
+			break;
+		}
+
+		tmp2 = ((tmp2 << 6) | (tmp & 0xff));
+		n++;
+
+		if (n == 4) {
+			out = g_realloc (out, len + 3);
+			out[len] = (guchar)((tmp2 >> 16) & 0xff);
+			len++;
+			out[len] = (guchar)((tmp2 >> 8) & 0xff);
+			len++;
+			out[len] = (guchar)(tmp2 & 0xff);
+			len++;
+			tmp2 = 0;
+			n = 0;
+		}
+
+		c++;
+	}
+
+	out = g_realloc (out, len + 1);
+	out[len] = 0;
+
+	if (ret_len != NULL) {
+		*ret_len = len;
+	}
+
+	return out;
+}
+
+gchar *
+gossip_base64_encode (const guchar *data, 
+		      gsize         len)
+{
+	gchar              *out;
+	gchar              *rv;
+	static const gchar  alphabet[] =
+		"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"
+		"0123456789+/";
+
+	if (data == NULL || len < 1) {
+		return NULL;
+	}
+
+	rv = out = g_malloc (((len / 3) + 1) * 4 + 1);
+
+	for (; len >= 3; len -= 3) {
+		*out++ = alphabet[data[0] >> 2];
+		*out++ = alphabet[((data[0] << 4) & 0x30) | (data[1] >> 4)];
+		*out++ = alphabet[((data[1] << 2) & 0x3c) | (data[2] >> 6)];
+		*out++ = alphabet[data[2] & 0x3f];
+		data += 3;
+	}
+
+	if (len > 0) {
+		unsigned char fragment;
+
+		*out++ = alphabet[data[0] >> 2];
+		fragment = (data[0] << 4) & 0x30;
+
+		if (len > 1) {
+			fragment |= data[1] >> 4;
+		}
+
+		*out++ = alphabet[fragment];
+		*out++ = (len < 2) ? '=' : alphabet[(data[1] << 2) & 0x3c];
+		*out++ = '=';
+	}
+
+	*out = '\0';
+
+	return rv;
+}
+
+
+gchar *
+gossip_sha1_string (const guchar *data,
+		    gsize         len) {
+	gchar  *hash_string;
+	gchar  *p;
+	guchar  hash[20];
+	gint    i;
+	
+	if (data == NULL || len < 1) {
+		return g_strdup("");
+	}
+	
+	gcry_md_hash_buffer (GCRY_MD_SHA1, hash, data, (size_t) len);
+	hash_string = g_malloc (sizeof (gchar) * 41);
+	p = hash_string;
+
+	for (i = 0; i < 20; i++, p += 2) {
+		snprintf (p, 3, "%02x", hash[i]);
+	}
+
+	return hash_string;
+}
