@@ -270,6 +270,12 @@ jabber_ft_handle_error (GossipJabber *jabber,
 
 	id_str = lm_message_node_get_attribute (m->node, "id");
 
+	ft = g_hash_table_lookup (fts->ft_ids, id_str);
+	if (!ft) {
+		DEBUG_MSG (("ProtocolFT: Could not find GossipFT* id:'%s'", id_str));
+		return;
+	}
+
 	from_str = lm_message_node_get_attribute (m->node, "from");
 	from = gossip_jabber_get_contact_from_jid (jabber, from_str, NULL, TRUE);	
 
@@ -282,14 +288,6 @@ jabber_ft_handle_error (GossipJabber *jabber,
 		  error_code_str,
 		  error_reason));
 	
-
-	ft = g_hash_table_lookup (fts->ft_ids, id_str);
-	if (!ft) {
-		g_warning ("Could not find GossipFT* from id:'%s'", 
-			   id_str);
-		return;
-	}
-
 	switch (atoi (error_code_str)) {
 	case 400:
 		error_code = GOSSIP_FT_ERROR_UNSUPPORTED;
@@ -317,19 +315,14 @@ jabber_ft_error (GossipJabber  *jabber,
 		 GossipFTError  code,
 		 const gchar   *reason)
 {
-	GError        *error;
-	static GQuark  quark = 0;
+	GError *error;
 
 	g_return_if_fail (GOSSIP_IS_JABBER (jabber));
 	g_return_if_fail (GOSSIP_IS_FT (ft));
 	g_return_if_fail (signal != NULL);
 	g_return_if_fail (reason != NULL);
 
-	if (!quark) {
-		quark = g_quark_from_static_string ("gossip-jabber-ft");
-	}
-
-	error = g_error_new_literal (quark, code, reason);
+	error = g_error_new_literal (gossip_jabber_error_quark(), code, reason);
 	g_signal_emit_by_name (jabber, signal, ft, error);
  	g_error_free (error); 
 }
