@@ -27,6 +27,7 @@
 
 #include <libgossip/gossip-account.h>
 #include <libgossip/gossip-contact.h>
+#include <libgossip/gossip-debug.h>
 #include <libgossip/gossip-vcard.h>
 #include <libgossip/gossip-utils.h>
 #include <libgossip/gossip-chatroom-provider.h>
@@ -41,26 +42,27 @@
 #include "gossip-jabber.h"
 #include "gossip-jabber-private.h"
 
-#define DEBUG_MSG(x)
-/* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n"); */
+#define DEBUG_DOMAIN "Jabber"
 
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_JABBER, GossipJabberPriv))
 
+#define GOSSIP_JABBER_ERROR_DOMAIN "GossipJabber"
+
 /* Common XMPP XML namespaces we use. */
-#define XMPP_VERSION_XMLNS  "jabber:iq:version"
-#define XMPP_ROSTER_XMLNS   "jabber:iq:roster"
-#define XMPP_REGISTER_XMLNS "jabber:iq:register"
+#define XMPP_VERSION_XMLNS         "jabber:iq:version"
+#define XMPP_ROSTER_XMLNS          "jabber:iq:roster"
+#define XMPP_REGISTER_XMLNS        "jabber:iq:register"
 
 /* We use 3.5 minutes because if the port is just wrong then it
  * will timeout before then with that error.
  */
-#define CONNECT_TIMEOUT     210 
+#define CONNECT_TIMEOUT            210 
 
 /* This is the timeout we will accept the user to be composing for
  * before we assume it is stuck and the server has failed to tell us
  * the user has stopped composing.
  */
-#define COMPOSING_TIMEOUT   45
+#define COMPOSING_TIMEOUT          45
   
 struct _GossipJabberPriv {
 	LmConnection          *connection;
@@ -117,7 +119,7 @@ typedef struct {
 
 static void             gossip_jabber_class_init            (GossipJabberClass       *klass);
 static void             gossip_jabber_init                  (GossipJabber            *jabber);
-static void             jabber_finalize                     (GObject                 *obj);
+static void             jabber_finalize                     (GObject                 *object);
 static void             jabber_setup                        (GossipProtocol          *protocol,
 							     GossipAccount           *account);
 static void             jabber_setup_connection             (GossipJabber            *jabber);
@@ -162,6 +164,7 @@ static LmHandlerResult  jabber_register_message_handler     (LmMessageHandler
 							     LmMessage               *m,
 							     RegisterData            *ra);
 static gboolean         jabber_is_connected                 (GossipProtocol          *protocol);
+static gboolean         jabber_is_connecting                (GossipProtocol          *protocol);
 static gboolean         jabber_is_valid_username            (GossipProtocol          *protocol,
 							     const gchar             *username);
 static gboolean         jabber_is_ssl_supported             (GossipProtocol          *protocol);
@@ -332,6 +335,7 @@ gossip_jabber_class_init (GossipJabberClass *klass)
 	protocol_class->login                 = jabber_login;
 	protocol_class->logout                = jabber_logout;
 	protocol_class->is_connected          = jabber_is_connected;
+	protocol_class->is_connecting         = jabber_is_connecting;
 	protocol_class->is_valid_username     = jabber_is_valid_username;
 	protocol_class->is_ssl_supported      = jabber_is_ssl_supported;
 	protocol_class->get_example_username  = jabber_get_example_username;
@@ -393,12 +397,12 @@ gossip_jabber_init (GossipJabber *jabber)
 }
 	
 static void
-jabber_finalize (GObject *obj)
+jabber_finalize (GObject *object)
 {
  	GossipJabber     *jabber;
  	GossipJabberPriv *priv;
 
- 	jabber = GOSSIP_JABBER (obj);
+ 	jabber = GOSSIP_JABBER (object);
  	priv = GET_PRIV (jabber);
 
 	if (priv->account) {
@@ -426,7 +430,9 @@ jabber_finalize (GObject *obj)
 	
 #ifdef USE_TRANSPORTS
  	gossip_transport_account_list_free (priv->account_list);
-#endif	
+#endif
+
+	(G_OBJECT_CLASS (gossip_jabber_parent_class)->finalize) (object);
 }
 
 static void
@@ -436,11 +442,10 @@ jabber_setup (GossipProtocol *protocol,
  	GossipJabber     *jabber;
  	GossipJabberPriv *priv;
  	LmMessageHandler *handler;
-	gchar            *id;
 	GossipJID        *jid;
 	
- 	g_return_if_fail (GOSSIP_IS_JABBER (protocol));
-  	g_return_if_fail (GOSSIP_IS_ACCOUNT (account));
+        g_return_if_fail (GOSSIP_IS_JABBER (protocol));
+        g_return_if_fail (GOSSIP_IS_ACCOUNT (account));
 	
  	jabber = GOSSIP_JABBER (protocol);
  	priv = GET_PRIV (jabber);
@@ -450,16 +455,10 @@ jabber_setup (GossipProtocol *protocol,
  	priv->contact = gossip_contact_new (GOSSIP_CONTACT_TYPE_USER,
 					    priv->account);
 	
-	id = g_strdup_printf ("%s/%s", 
-			      gossip_account_get_id (account),
-			      gossip_account_get_resource (account));
-	jid = gossip_jid_new (id);
-
 	g_object_set (priv->contact,
-		      "id", id,          /* we need FULL JID here */
-		      "name", id,
+		      "id", gossip_account_get_id (account),
+		      "name", gossip_account_get_id (account),
 		      NULL);
-	g_free (id);
 
 	priv->connection = lm_connection_new (gossip_account_get_server (account));
 
@@ -471,8 +470,9 @@ jabber_setup (GossipProtocol *protocol,
 					       jabber, NULL);
 
 	lm_connection_set_port (priv->connection, gossip_account_get_port (account));
-	lm_connection_set_jid (priv->connection, gossip_jid_get_without_resource (jid));
 
+	jid = gossip_jid_new (gossip_account_get_id (account));
+	lm_connection_set_jid (priv->connection, gossip_jid_get_without_resource (jid));
 	gossip_jid_unref (jid);
 
 	handler = lm_message_handler_new ((LmHandleMessageFunction) jabber_message_handler,
@@ -522,7 +522,7 @@ jabber_setup_connection (GossipJabber *jabber)
 	if (gossip_account_get_use_ssl (priv->account)) {
 		LmSSL *ssl;
 
-		DEBUG_MSG (("Protocol: Using SSL"));
+		gossip_debug (DEBUG_DOMAIN, "Using SSL");
 
 		ssl = lm_ssl_new (NULL,
 				  (LmSSLFunction) jabber_ssl_status_cb,
@@ -536,7 +536,7 @@ jabber_setup_connection (GossipJabber *jabber)
 	}
 
 	if (gossip_account_get_use_proxy (priv->account)) {
-		DEBUG_MSG (("Protocol: Using proxy"));
+		gossip_debug (DEBUG_DOMAIN, "Using proxy");
 
 		jabber_set_proxy (priv->connection);
 	} else {
@@ -562,8 +562,12 @@ jabber_login (GossipProtocol *protocol)
 	jabber = GOSSIP_JABBER (protocol);
 	priv = GET_PRIV (jabber);
 
-	DEBUG_MSG (("Protocol: Logging in Jabber"));
+	gossip_debug (DEBUG_DOMAIN, "Logging in Jabber");
 
+	if (priv->presence) {
+		g_object_unref (priv->presence);
+	}
+	
 	priv->presence = gossip_presence_new ();
 	gossip_presence_set_state (priv->presence, 
 				   GOSSIP_PRESENCE_STATE_AVAILABLE);
@@ -580,7 +584,6 @@ jabber_login (GossipProtocol *protocol)
 				     (LmResultFunction) jabber_connection_open_cb,
 				     jabber, NULL, &error);
 
-
 	if (result && !error) {
 		/* FIXME: add timeout incase we get nothing back from
 		   Loudmouth, this happens with current CVS loudmouth
@@ -635,8 +638,7 @@ jabber_logout (GossipProtocol *protocol)
 
 	priv->disconnect_request = TRUE;
 
-	if (priv->connection && 
-	    lm_connection_is_open (priv->connection)) {
+	if (priv->connection) {
 		lm_connection_close (priv->connection, NULL);
 	}
 }
@@ -653,7 +655,11 @@ jabber_logout_contact_foreach (gpointer       key,
 	/* Set each contact to be offline, since they effectively are
 	 * now we don't know.
 	 */
-	presences = gossip_contact_get_presence_list (contact);
+	presences = g_list_copy (gossip_contact_get_presence_list (contact));
+
+	/* Copy the list since it will be modified during traversal
+	 * otherwise.
+	 */
 	for (l = presences; l; l = l->next) {
 		presence = l->data;
 
@@ -664,6 +670,8 @@ jabber_logout_contact_foreach (gpointer       key,
 		gossip_contact_remove_presence (contact, presence);
 	}
 
+	g_list_free (presences);
+	
 	g_signal_emit_by_name (jabber, "contact-removed", contact);
 	return TRUE;
 }
@@ -698,7 +706,7 @@ jabber_error (GossipProtocol      *protocol,
 	priv = GET_PRIV (jabber);
 
 	message = gossip_protocol_error_to_string (code);
-	DEBUG_MSG (("Protocol: Error:%d->'%s'", code, message));
+	gossip_debug (DEBUG_DOMAIN, "Error:%d->'%s'", code, message);
 
 	error = jabber_error_create (code, message);
 	g_signal_emit_by_name (protocol, "error", priv->account, error);
@@ -734,7 +742,7 @@ jabber_connection_open_cb (LmConnection *connection,
 		return;
 	}
 
-	DEBUG_MSG (("Protocol: Connection open!"));
+	gossip_debug (DEBUG_DOMAIN, "Connection open!");
 
 	if (priv->connection_timeout_id != 0) {
 		g_source_remove (priv->connection_timeout_id);
@@ -745,15 +753,15 @@ jabber_connection_open_cb (LmConnection *connection,
 	account_password = gossip_account_get_password (account);
 
 	if (!account_password || strlen (account_password) < 1) {
-		DEBUG_MSG (("Protocol: Requesting password for:'%s'", 
-			    gossip_account_get_id (account)));
+		gossip_debug (DEBUG_DOMAIN, "Requesting password for:'%s'", 
+			      gossip_account_get_id (account));
 
 		g_signal_emit_by_name (jabber, "get-password", 
 				       account, &password);
 
 		if (!password) {
-			DEBUG_MSG (("Protocol: Cancelled password request for:'%s'", 
-				    gossip_account_get_id (account)));
+			gossip_debug (DEBUG_DOMAIN, "Cancelled password request for:'%s'", 
+				      gossip_account_get_id (account));
 
 			jabber_logout (GOSSIP_PROTOCOL (jabber));
 			return;
@@ -764,13 +772,13 @@ jabber_connection_open_cb (LmConnection *connection,
 	 
 	/* FIXME: Decide on Resource */
 	jid_str = gossip_account_get_id (account);
-	DEBUG_MSG (("Protocol: Attempting to use JabberID:'%s'", jid_str));
+	gossip_debug (DEBUG_DOMAIN, "Attempting to use JabberID:'%s'", jid_str);
 
 	id = gossip_jid_string_get_part_name (jid_str);
 	resource = gossip_account_get_resource (account);
 
 	if (!resource) {
-		DEBUG_MSG (("Protocol: JabberID:'%s' is invalid, there is no resource.", jid_str));
+		gossip_debug (DEBUG_DOMAIN, "JabberID:'%s' is invalid, there is no resource.", jid_str);
 
 		jabber_logout (GOSSIP_PROTOCOL (jabber));
 		jabber_error (GOSSIP_PROTOCOL (jabber), GOSSIP_PROTOCOL_INVALID_USER);
@@ -805,7 +813,7 @@ jabber_connection_auth_cb (LmConnection *connection,
 		return;
 	}
 
-	DEBUG_MSG (("Protocol: Connection logged in!"));
+	gossip_debug (DEBUG_DOMAIN, "Connection logged in!");
 
 	/* Request roster */
 	m = lm_message_new_with_sub_type (NULL,
@@ -917,7 +925,7 @@ jabber_ssl_status_cb (LmConnection *connection,
 		break;
 	}
 
-	DEBUG_MSG (("Protocol: %s", str));
+	gossip_debug (DEBUG_DOMAIN, "%s", str);
 
 	return LM_SSL_RESPONSE_CONTINUE;
 }
@@ -975,7 +983,7 @@ jabber_register_account (GossipProtocol          *protocol,
 	g_return_if_fail (GOSSIP_IS_JABBER (protocol));
 	g_return_if_fail (callback != NULL);
 
-	DEBUG_MSG (("Protocol: Registering with Jabber server..."));
+	gossip_debug (DEBUG_DOMAIN, "Registering with Jabber server...");
 
 	jabber = GOSSIP_JABBER (protocol);
 	priv = GET_PRIV (jabber);
@@ -1005,7 +1013,7 @@ error:
 	error_message = gossip_protocol_error_to_string (error_code);
 	error = jabber_error_create (error_code, error_message);
 
-	DEBUG_MSG (("Protocol: %s", error_message));
+	gossip_debug (DEBUG_DOMAIN, "%s", error_message);
 	
 	if (callback) {
 		(callback) (GOSSIP_RESULT_ERROR_REGISTRATION, 
@@ -1025,7 +1033,7 @@ jabber_register_cancel (GossipProtocol *protocol)
 
 	g_return_if_fail (GOSSIP_IS_JABBER (protocol));
 
-	DEBUG_MSG (("Protocol: Canceling registration with Jabber server..."));
+	gossip_debug (DEBUG_DOMAIN, "Canceling registration with Jabber server...");
 
 	jabber = GOSSIP_JABBER (protocol);
 	priv = GET_PRIV (jabber);
@@ -1062,7 +1070,7 @@ jabber_register_connection_open_cb (LmConnection *connection,
 		error = jabber_error_create (GOSSIP_PROTOCOL_NO_CONNECTION,
 					     error_message);
 
-		DEBUG_MSG (("Protocol: %s", error_message));
+		gossip_debug (DEBUG_DOMAIN, "%s", error_message);
 
 		if (ra->callback) {
 			(ra->callback) (GOSSIP_RESULT_ERROR_REGISTRATION, 
@@ -1075,7 +1083,7 @@ jabber_register_connection_open_cb (LmConnection *connection,
 		return;
 	}
  
-	DEBUG_MSG (("Protocol: Connection open!"));
+	gossip_debug (DEBUG_DOMAIN, "Connection open!");
 
 	ra->message_handler = lm_message_handler_new ((LmHandleMessageFunction) 
 						      jabber_register_message_handler,
@@ -1110,7 +1118,7 @@ jabber_register_connection_open_cb (LmConnection *connection,
 		error = jabber_error_create (GOSSIP_PROTOCOL_SPECIFIC_ERROR,
 					     error_message);
 
-		DEBUG_MSG (("Protocol: %s", error_message));
+		gossip_debug (DEBUG_DOMAIN, "%s", error_message);
 
 		if (ra->callback) {
 			(ra->callback) (GOSSIP_RESULT_ERROR_REGISTRATION, 
@@ -1123,7 +1131,7 @@ jabber_register_connection_open_cb (LmConnection *connection,
 		return;
 	}
 
-	DEBUG_MSG (("Protocol: Sent registration details"));
+	gossip_debug (DEBUG_DOMAIN, "Sent registration details");
 }
 
 static LmHandlerResult
@@ -1192,10 +1200,10 @@ jabber_register_message_handler (LmMessageHandler *handler,
 		error = jabber_error_create (error_code,
 					     error_message);
 
-		DEBUG_MSG (("Protocol: Registration failed with error:%s->'%s'",
-			   error_code_str, error_message));	
+		gossip_debug (DEBUG_DOMAIN, "Registration failed with error:%s->'%s'",
+			      error_code_str, error_message);	
 	} else {
-		DEBUG_MSG (("Protocol: Registration success"));
+		gossip_debug (DEBUG_DOMAIN, "Registration success");
 	}
 
 	if (ra->callback) {
@@ -1232,6 +1240,30 @@ jabber_is_connected (GossipProtocol *protocol)
 	return lm_connection_is_authenticated (priv->connection);
 }
 
+static gboolean
+jabber_is_connecting (GossipProtocol *protocol)
+{
+	GossipJabber      *jabber;
+	GossipJabberPriv  *priv;
+	LmConnectionState  state;
+
+	g_return_val_if_fail (GOSSIP_IS_JABBER (protocol), FALSE);
+
+	jabber = GOSSIP_JABBER (protocol);
+	priv = GET_PRIV (jabber);
+
+	if (priv->connection == NULL) {
+		return FALSE;
+	}
+
+	state = lm_connection_get_state (priv->connection);
+	if (state == LM_CONNECTION_STATE_OPENING) {
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
 static gboolean
 jabber_is_valid_username (GossipProtocol *protocol, 
 			  const gchar    *username)
@@ -1335,7 +1367,7 @@ jabber_send_message (GossipProtocol *protocol,
 		jid_str = g_strdup (recipient_id);
 	}
 
-	DEBUG_MSG (("Protocol: Sending message to:'%s'", jid_str));
+	gossip_debug (DEBUG_DOMAIN, "Sending message to:'%s'", jid_str);
 	
 	m = lm_message_new_with_sub_type (jid_str,
 					  LM_MESSAGE_TYPE_MESSAGE,
@@ -1386,9 +1418,9 @@ jabber_send_composing (GossipProtocol *protocol,
 
 	contact_id = gossip_contact_get_id (contact);
 
-	DEBUG_MSG (("Protocol: Sending %s to contact:'%s'", 
-		   typing ? "composing" : "not composing",
-		   contact_id));
+	gossip_debug (DEBUG_DOMAIN, "Sending %s to contact:'%s'", 
+		      typing ? "composing" : "not composing",
+		      contact_id);
 	
 	jid_str = g_strdup (contact_id);
 
@@ -1476,12 +1508,12 @@ jabber_set_presence (GossipProtocol *protocol,
 	avatar = gossip_contact_get_avatar (contact, &avatar_size);
 	sha1 = gossip_sha1_string (avatar, avatar_size);
 
-	DEBUG_MSG (("Protocol: Setting presence to:'%s', status:'%s', "
-		    "priority:'%s' sha1:'%s' avatar_size:%" G_GSIZE_FORMAT " avatar:%p", 
-		    show ? show : "available", 
-		    status ? status : "",
-		    priority, sha1, 
-		    avatar_size, avatar));
+	gossip_debug (DEBUG_DOMAIN, "Setting presence to:'%s', status:'%s', "
+		      "priority:'%s' sha1:'%s' avatar_size:%" G_GSIZE_FORMAT " avatar:%p", 
+		      show ? show : "available", 
+		      status ? status : "",
+		      priority, sha1, 
+		      avatar_size, avatar);
 
 	if (show) {
 		lm_message_node_add_child (m->node, "show", show);
@@ -1510,6 +1542,9 @@ jabber_set_presence (GossipProtocol *protocol,
 	
 	lm_connection_send (priv->connection, m, NULL);
 	lm_message_unref (m);
+
+	/* Don't forget to set any chatroom status too */
+	gossip_jabber_chatrooms_set_presence (priv->chatrooms, priv->presence);
 }
 
 static void
@@ -1524,9 +1559,9 @@ jabber_set_subscription (GossipProtocol *protocol,
 
 	jabber = GOSSIP_JABBER (protocol);
 
-	DEBUG_MSG (("Protocol: Setting subscription for contact:'%s' as %s", 
-		   gossip_contact_get_id (contact),
-		   subscribed ? "subscribed" : "unsubscribed"));
+	gossip_debug (DEBUG_DOMAIN, "Setting subscription for contact:'%s' as %s", 
+		      gossip_contact_get_id (contact),
+		      subscribed ? "subscribed" : "unsubscribed");
 
 	if (subscribed) {
 		gossip_jabber_send_subscribed (jabber, contact);
@@ -1546,9 +1581,9 @@ jabber_set_vcard (GossipProtocol        *protocol,
 
 	jabber = GOSSIP_JABBER (protocol);
 	
-	DEBUG_MSG (("Protocol: Setting vcard for '%s'", 
-		    gossip_vcard_get_name (vcard)))
-
+	gossip_debug (DEBUG_DOMAIN, "Setting vcard for '%s'", 
+		      gossip_vcard_get_name (vcard));
+	
 	return gossip_jabber_vcard_set (jabber,
 					vcard,
 					callback, user_data,
@@ -1559,12 +1594,12 @@ static GossipContact *
 jabber_contact_find (GossipProtocol *protocol,
 		     const gchar    *id)
 {
-        GossipJabber     *jabber;
+	GossipJabber     *jabber;
 	GossipJabberPriv *priv;
 	GossipJID        *jid;
 	GossipContact    *contact;
 
-        jabber = GOSSIP_JABBER (protocol);
+	jabber = GOSSIP_JABBER (protocol);
 	priv = GET_PRIV (jabber);
 
 	jid = gossip_jid_new (id);
@@ -1578,114 +1613,106 @@ jabber_contact_find (GossipProtocol *protocol,
 
 static void
 jabber_contact_add (GossipProtocol *protocol,
-                    const gchar    *id,
-                    const gchar    *name,
-                    const gchar    *group,
-                    const gchar    *message)
+		    const gchar    *id,
+		    const gchar    *name,
+		    const gchar    *group,
+		    const gchar    *message)
 {
 	LmMessage          *m;
-        LmMessageNode      *node;
-        GossipJabber       *jabber;
-        GossipJabberPriv   *priv;
+	LmMessageNode      *node;
+	GossipJabber       *jabber;
+	GossipJabberPriv   *priv;
 	GossipJID          *jid;
 	GossipContact      *contact;
-	GossipContactType   type;
 	GossipSubscription  subscription;
-	gboolean            ask = TRUE;
 	gchar              *escaped;
+	gboolean            add_to_roster;
 
-	DEBUG_MSG (("Protocol: Adding contact:'%s' with name:'%s' and group:'%s'", 
-		    id, name, group));
+	gossip_debug (DEBUG_DOMAIN, "Adding contact:'%s' with name:'%s' and group:'%s'", 
+		      id, name, group);
 
-        jabber = GOSSIP_JABBER (protocol);
-        priv = GET_PRIV (jabber);
+	jabber = GOSSIP_JABBER (protocol);
+	priv = GET_PRIV (jabber);
 
 	jid = gossip_jid_new (id);
 	contact = g_hash_table_lookup (priv->contacts, gossip_jid_get_without_resource (jid));
 
-	escaped = g_markup_escape_text (message, -1);
-
-        /* Request subscription */
-	DEBUG_MSG (("Protocol: Sending subscribe request with message:'%s'...", message));
-
-        m = lm_message_new_with_sub_type (id, LM_MESSAGE_TYPE_PRESENCE,
-                                          LM_MESSAGE_SUB_TYPE_SUBSCRIBE);
-        lm_message_node_add_child (m->node, "status", escaped);
-	g_free (escaped);
-
-        lm_connection_send (priv->connection, m, NULL);
-        lm_message_unref (m);
-
-	/* Add to roster */
-	DEBUG_MSG (("Protocol: Adding contact:'%s' to roster...", id));
-
-	m = lm_message_new_with_sub_type (NULL, 
-					  LM_MESSAGE_TYPE_IQ,
-					  LM_MESSAGE_SUB_TYPE_SET);
-
-	node = lm_message_node_add_child (m->node, "query", NULL);
-	lm_message_node_set_attributes (node,
-					"xmlns", XMPP_ROSTER_XMLNS, NULL);
-
-	node = lm_message_node_add_child (node, "item", NULL);
-
-	/* Only ask for subscription if we already have NONE */
 	if (contact) {
-		type = gossip_contact_get_type (contact);
 		subscription = gossip_contact_get_subscription (contact);
+	} else {
+		subscription = GOSSIP_SUBSCRIPTION_NONE;
+	}
+
+	/* We would normally only add to roster IF not on it but we
+	 * always do this because the server will inforce it otherwise
+	 * and it makes sense to use our provided name/group, etc 
+	 */
+	add_to_roster = TRUE;
+/* 	add_to_roster = gossip_contact_get_type (contact) == GOSSIP_CONTACT_TYPE_TEMPORARY; */
+
+	if (add_to_roster) {
+		gossip_debug (DEBUG_DOMAIN, "Adding contact:'%s' to roster...", id);
 		
-		ask &= type == GOSSIP_CONTACT_TYPE_TEMPORARY;
-		ask &= subscription != GOSSIP_SUBSCRIPTION_BOTH;
-
-		DEBUG_MSG (("Protocol: %s for subscription (type is %s and subscription is %s)...", 
-			    ask ? "Asking" : "Not asking",
-			    type == GOSSIP_CONTACT_TYPE_TEMPORARY ? "temporary" : "not temporary",
-			    subscription == GOSSIP_SUBSCRIPTION_BOTH ? "both" : "not both"));
-	} else {
-		DEBUG_MSG (("Protocol: Not asking for subscription (no contact)..."));
-		ask = FALSE;
-	}
-
-	escaped = g_markup_escape_text (name, -1);
-
-	if (ask) {
+		m = lm_message_new_with_sub_type (NULL, 
+						  LM_MESSAGE_TYPE_IQ,
+						  LM_MESSAGE_SUB_TYPE_SET);
+		
+		node = lm_message_node_add_child (m->node, "query", NULL);
 		lm_message_node_set_attributes (node,
-						"jid", gossip_jid_get_without_resource (jid),
-						"subscription", "none",
-						"ask", "subscribe",
+						"xmlns", XMPP_ROSTER_XMLNS, NULL);
+		
+		node = lm_message_node_add_child (node, "item", NULL);
+		
+		escaped = g_markup_escape_text (name, -1);
+		lm_message_node_set_attributes (node, 
+						"jid", gossip_jid_get_without_resource (jid), 
 						"name", escaped,
 						NULL);
-	} else {
-		/* Make sure we set the name incase we updated it */
-		lm_message_node_set_attributes (node,
-						"jid", gossip_jid_get_without_resource (jid),
-						"name", escaped,
-						NULL);
-	}
-
-	g_free (escaped);
-
-        if (group && g_utf8_strlen (group, -1) > 0) {
-		escaped = g_markup_escape_text (group, -1);
-                lm_message_node_add_child (node, "group", escaped);
 		g_free (escaped);
-        }
-	
-	lm_connection_send (priv->connection, m, NULL);
-	lm_message_unref (m);
+		
+		if (group && g_utf8_strlen (group, -1) > 0) {
+			escaped = g_markup_escape_text (group, -1);
+			lm_message_node_add_child (node, "group", escaped);
+			g_free (escaped);
+		}
+		
+		lm_connection_send (priv->connection, m, NULL);
+		lm_message_unref (m);
+	}
+
+	/* Request subscription */
+	if (subscription == GOSSIP_SUBSCRIPTION_NONE ||
+	    subscription == GOSSIP_SUBSCRIPTION_FROM) {
+		gossip_debug (DEBUG_DOMAIN, "Sending subscribe request with message:'%s'...", 
+			      message);
+		
+		m = lm_message_new_with_sub_type (gossip_jid_get_without_resource (jid), 
+						  LM_MESSAGE_TYPE_PRESENCE,
+						  LM_MESSAGE_SUB_TYPE_SUBSCRIBE);
+		
+		escaped = g_markup_escape_text (message, -1);
+		lm_message_node_add_child (m->node, "status", escaped);
+		g_free (escaped);
+		
+		lm_connection_send (priv->connection, m, NULL);
+		lm_message_unref (m);
+	} else {
+		gossip_debug (DEBUG_DOMAIN, "NOT Sending subscribe request, "
+			      "subscription is either TO or BOTH");
+	}
 
 	gossip_jid_unref (jid);
 }
 
 static void
 jabber_contact_rename (GossipProtocol *protocol,
-                       GossipContact  *contact,
-                       const gchar    *new_name)
+		       GossipContact  *contact,
+		       const gchar    *new_name)
 {
 	GossipJabber     *jabber;
 	GossipJabberPriv *priv;
 	LmMessage        *m;
-        LmMessageNode    *node;
+	LmMessageNode    *node;
 	gchar            *escaped;
 	GList            *l; 
 	
@@ -1707,12 +1734,12 @@ jabber_contact_rename (GossipProtocol *protocol,
 					"jid", gossip_contact_get_id (contact),
 					"name", escaped,
 					NULL);
-        g_free (escaped);
+	g_free (escaped);
 	
-        for (l = gossip_contact_get_groups (contact); l; l = l->next) {
-                escaped = g_markup_escape_text (l->data, -1);
-                lm_message_node_add_child (node, "group", escaped);
-                g_free (escaped);
+	for (l = gossip_contact_get_groups (contact); l; l = l->next) {
+		escaped = g_markup_escape_text (l->data, -1);
+		lm_message_node_add_child (node, "group", escaped);
+		g_free (escaped);
 	}	
 
 	lm_connection_send (priv->connection, m, NULL);
@@ -1726,32 +1753,27 @@ jabber_contact_remove (GossipProtocol *protocol,
 	GossipJabber     *jabber;
 	GossipJabberPriv *priv;
 	LmMessage        *m;
-        LmMessageNode    *node;
+	LmMessageNode    *node;
 	
 	jabber = GOSSIP_JABBER (protocol);
 	priv = GET_PRIV (jabber);
 
+	/* Next remove the contact from the roster */
 	m = lm_message_new_with_sub_type (NULL,
- 					  LM_MESSAGE_TYPE_IQ,
- 					  LM_MESSAGE_SUB_TYPE_SET);
+					  LM_MESSAGE_TYPE_IQ,
+					  LM_MESSAGE_SUB_TYPE_SET);
 	
- 	node = lm_message_node_add_child (m->node, "query", NULL);
- 	lm_message_node_set_attribute (node,
+	node = lm_message_node_add_child (m->node, "query", NULL);
+	lm_message_node_set_attribute (node,
 				       "xmlns",
 				       XMPP_ROSTER_XMLNS);
 	
- 	node = lm_message_node_add_child (node, "item", NULL);
- 	lm_message_node_set_attributes (node,
+	node = lm_message_node_add_child (node, "item", NULL);
+	lm_message_node_set_attributes (node,
 					"jid", gossip_contact_get_id (contact),
 					"subscription", "remove",
 					NULL);
  	
- 	lm_connection_send (priv->connection, m, NULL);
- 	lm_message_unref (m);
-
-	m = lm_message_new_with_sub_type (gossip_contact_get_id (contact),
-					  LM_MESSAGE_TYPE_PRESENCE,
-					  LM_MESSAGE_SUB_TYPE_UNSUBSCRIBE);
 	lm_connection_send (priv->connection, m, NULL);
 	lm_message_unref (m);
 }
@@ -1919,14 +1941,14 @@ jabber_group_rename_foreach_cb (const gchar     *jid,
 {
 	GossipJabberPriv *priv;
 	LmMessage        *m;
-        LmMessageNode    *node;
+	LmMessageNode    *node;
 	gchar            *escaped;
 	GList            *l; 
 	gboolean          found = FALSE;
 
 	priv = GET_PRIV (rg->jabber);
 
-        for (l = gossip_contact_get_groups (contact); l && !found; l = l->next) {
+	for (l = gossip_contact_get_groups (contact); l && !found; l = l->next) {
 		gchar *group = (gchar*)l->data;
 
 		if (group && strcmp (group, rg->group) == 0) {
@@ -2206,8 +2228,8 @@ jabber_composing_timeout_cb (JabberData *data)
 
 	priv = GET_PRIV (data->jabber);
 
-	DEBUG_MSG (("Protocol: Contact:'%s' is NOT composing (timed out)",
-		    gossip_contact_get_id (data->contact)));
+	gossip_debug (DEBUG_DOMAIN, "Contact:'%s' is NOT composing (timed out)",
+		      gossip_contact_get_id (data->contact));
 
 	g_signal_emit_by_name (data->jabber, "composing", data->contact, FALSE);
 
@@ -2236,8 +2258,8 @@ jabber_message_handler (LmMessageHandler *handler,
 
 	priv = GET_PRIV (jabber);
 	
-	DEBUG_MSG (("Protocol: New message from:'%s'", 
-		   lm_message_node_get_attribute (m->node, "from")));
+	gossip_debug (DEBUG_DOMAIN, "New message from:'%s'", 
+		      lm_message_node_get_attribute (m->node, "from"));
 
 	sub_type = lm_message_get_sub_type (m);
 	if (sub_type != LM_MESSAGE_SUB_TYPE_NOT_SET && 
@@ -2258,9 +2280,9 @@ jabber_message_handler (LmMessageHandler *handler,
 		
 		composing = gossip_jabber_get_message_is_composing (m);
 		
-		DEBUG_MSG (("Protocol: Contact:'%s' %s",
-			    gossip_contact_get_id (from),
-			    composing ? "is composing" : "is NOT composing"));
+		gossip_debug (DEBUG_DOMAIN, "Contact:'%s' %s",
+			      gossip_contact_get_id (from),
+			      composing ? "is composing" : "is NOT composing");
 		
 		g_signal_emit_by_name (jabber, "composing", from, composing);
 		
@@ -2302,13 +2324,12 @@ jabber_message_handler (LmMessageHandler *handler,
 					     g_object_ref (from),
 					     GINT_TO_POINTER (TRUE));
 		} else {
-			g_hash_table_remove (priv->composing_requests, 
-					     g_object_ref (from));
+			g_hash_table_remove (priv->composing_requests, from);
 		}
 		
-		DEBUG_MSG (("Protocol: Contact:'%s' %s composing info...",
-			    gossip_contact_get_id (from),
-			    wants_composing ? "wants" : "does NOT want"));
+		gossip_debug (DEBUG_DOMAIN, "Contact:'%s' %s composing info...",
+			      gossip_contact_get_id (from),
+			      wants_composing ? "wants" : "does NOT want");
 	}
 	
 	node = lm_message_node_get_child (m->node, "subject");
@@ -2322,10 +2343,10 @@ jabber_message_handler (LmMessageHandler *handler,
 		
 		invitor = gossip_chatroom_invite_get_invitor (invite);
 		
-		DEBUG_MSG (("Protocol: Chat room invitiation from:'%s' for room:'%s', reason:'%s'", 
-			    gossip_contact_get_id (invitor), 
-			    gossip_chatroom_invite_get_id (invite), 
-			    gossip_chatroom_invite_get_reason (invite)));
+		gossip_debug (DEBUG_DOMAIN, "Chat room invitiation from:'%s' for room:'%s', reason:'%s'", 
+			      gossip_contact_get_id (invitor), 
+			      gossip_chatroom_invite_get_id (invite), 
+			      gossip_chatroom_invite_get_reason (invite));
 		
 		/* We set the from to be the person that invited you,
 		 * not the chatroom that sent the request on their
@@ -2350,10 +2371,10 @@ jabber_message_handler (LmMessageHandler *handler,
 		 * has no purpose now (see fixes #309912). 
 		 */
 
-		DEBUG_MSG (("Protocol: Dropping new message, no <body> element"));
+		gossip_debug (DEBUG_DOMAIN, "Dropping new message, no <body> element");
 		return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
 	}
-	
+
 	node = lm_message_node_get_child (m->node, "thread");
 	if (node) {
 		thread = node->value;
@@ -2362,6 +2383,26 @@ jabber_message_handler (LmMessageHandler *handler,
 	message = gossip_message_new (GOSSIP_MESSAGE_TYPE_NORMAL,
 				      priv->contact);
 
+	/* To make the sender right in private chat messages sent from
+	 * groupchats, we take the name from the resource, which carries the
+	 * nick for those messages.
+	 */
+	if (gossip_jabber_chatrooms_get_jid_is_chatroom (priv->chatrooms,
+							 from_str)) {
+		GossipJID   *jid;
+		const gchar *resource;
+
+		jid = gossip_jid_new (from_str);
+		resource = gossip_jid_get_resource (jid);
+		if (!resource) {
+			resource = "";
+		}
+		
+		gossip_contact_set_name (from, resource);
+
+		gossip_jid_unref (jid);
+	}
+		
 	gossip_message_set_sender (message, from);
 	gossip_message_set_body (message, body);
 
@@ -2407,8 +2448,8 @@ jabber_presence_handler (LmMessageHandler *handler,
 	priv = GET_PRIV (jabber);
 
 	from = lm_message_node_get_attribute (m->node, "from");
-        DEBUG_MSG (("Protocol: New presence from:'%s'", 
-		   lm_message_node_get_attribute (m->node, "from")));
+	gossip_debug (DEBUG_DOMAIN, "New presence from:'%s'", 
+		      lm_message_node_get_attribute (m->node, "from"));
 
 	if (gossip_jabber_chatrooms_get_jid_is_chatroom (priv->chatrooms,
 							 from)) {
@@ -2547,7 +2588,7 @@ jabber_subscription_message_handler (LmMessageHandler  *handler,
 	gchar         *to;
 
 	if (lm_message_get_sub_type (m) != LM_MESSAGE_SUB_TYPE_SUBSCRIBE) {
-                return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
+		return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
 	}
 
 	from = lm_message_node_get_attribute (m->node, "from");
@@ -2557,7 +2598,7 @@ jabber_subscription_message_handler (LmMessageHandler  *handler,
 	lm_message_handler_invalidate (handler);
 	
 	/* Send subscribed to them */
-	DEBUG_MSG (("Protocol: Sending subscribed message to new service:'%s'", to));
+	gossip_debug (DEBUG_DOMAIN, "Sending subscribed message to new service:'%s'", to);
 	new_message = lm_message_new_with_sub_type (to,
 						    LM_MESSAGE_TYPE_PRESENCE,
 						    LM_MESSAGE_SUB_TYPE_SUBSCRIBED);
@@ -2581,7 +2622,7 @@ jabber_subscription_message_handler (LmMessageHandler  *handler,
 
 	g_free (to);
 
- 	return LM_HANDLER_RESULT_REMOVE_MESSAGE;
+	return LM_HANDLER_RESULT_REMOVE_MESSAGE;
 }
 
 
@@ -2604,7 +2645,7 @@ jabber_request_version (GossipJabber *jabber,
 	from = lm_message_node_get_attribute (m->node, "from");
 	id = lm_message_node_get_attribute (m->node, "id");
 
-	DEBUG_MSG (("Protocol: Version request from:'%s'", from));
+	gossip_debug (DEBUG_DOMAIN, "Version request from:'%s'", from);
 
 	r = lm_message_new_with_sub_type (from,
 					  LM_MESSAGE_TYPE_IQ,
@@ -2662,10 +2703,9 @@ jabber_request_roster (GossipJabber *jabber,
 		gboolean       updated;
 		LmMessageNode *subnode;
 		LmMessageNode *child;
-		GList         *groups = NULL;
-
+		GList         *groups;
 		const gchar   *name;
-		GList         *new_groups = NULL;
+		GList         *new_groups;
 		gboolean       name_updated, groups_updated;
 		
 
@@ -2686,6 +2726,7 @@ jabber_request_roster (GossipJabber *jabber,
 		g_object_set (contact, "type", GOSSIP_CONTACT_TYPE_CONTACTLIST, NULL);
 
 		/* Groups */
+		groups = NULL;
 		for (subnode = node->children; subnode; subnode = subnode->next) {
 			if (strcmp (subnode->name, "group") != 0) {
 				continue;
@@ -2701,6 +2742,8 @@ jabber_request_roster (GossipJabber *jabber,
 		if (groups) {
 			gossip_contact_set_groups (contact, groups);
 		}
+		
+		g_list_free (groups);
 
 		/* Subscription */
 		subscription = lm_message_node_get_attribute (node, "subscription");
@@ -2735,6 +2778,7 @@ jabber_request_roster (GossipJabber *jabber,
 			gossip_contact_set_name (contact, name);
 		}
 
+		new_groups = NULL;
 		for (child = node->children; child; child = child->next) {
 			if (strcmp (child->name, "group") == 0 && child->value) {
 				new_groups = g_list_append (new_groups, child->value);
@@ -2742,12 +2786,12 @@ jabber_request_roster (GossipJabber *jabber,
 		}
 
 		if (new_groups) {
-			/* Does not get free'd */
 			groups_updated = gossip_contact_set_groups (contact, new_groups);
 		}
 
+		g_list_free (new_groups);
+		
 		updated = (name_updated || groups_updated);
-
 		
 		if (new_item) {
 			g_signal_emit_by_name (jabber, 
@@ -3130,12 +3174,18 @@ jabber_data_free (JabberData *data)
  * External functions 
  */ 
 
+GQuark
+gossip_jabber_error_quark (void)
+{
+	return g_quark_from_static_string (GOSSIP_JABBER_ERROR_DOMAIN);
+}
+
 GossipAccount *
 gossip_jabber_get_account (GossipJabber *jabber)
 {
 	GossipJabberPriv *priv;
 	
-        g_return_val_if_fail (GOSSIP_IS_JABBER (jabber), NULL);
+	g_return_val_if_fail (GOSSIP_IS_JABBER (jabber), NULL);
 	
 	priv = GET_PRIV (jabber);
 	
@@ -3173,7 +3223,7 @@ gossip_jabber_get_contact_from_jid (GossipJabber *jabber,
 				       gossip_jid_get_without_resource (jid));
 
 	if (!contact) {
-		DEBUG_MSG (("Protocol: New contact:'%s'", gossip_jid_get_full (jid)));
+		gossip_debug (DEBUG_DOMAIN, "New contact:'%s'", gossip_jid_get_full (jid));
 		contact = gossip_contact_new (GOSSIP_CONTACT_TYPE_TEMPORARY,
 					      priv->account);
 		gossip_contact_set_id (contact, 
@@ -3194,7 +3244,7 @@ gossip_jabber_get_contact_from_jid (GossipJabber *jabber,
 			jabber_contact_vcard (jabber, contact);
 		}
 	} else {
-		DEBUG_MSG (("Protocol: Get contact:'%s'", gossip_jid_get_full (jid)));
+		gossip_debug (DEBUG_DOMAIN, "Get contact:'%s'", gossip_jid_get_full (jid));
 	}
 
 	gossip_jid_unref (jid);
@@ -3323,7 +3373,7 @@ gossip_jabber_get_connection (GossipJabber *jabber)
 {
 	GossipJabberPriv *priv;
 	
-        g_return_val_if_fail (GOSSIP_IS_JABBER (jabber), NULL);
+	g_return_val_if_fail (GOSSIP_IS_JABBER (jabber), NULL);
 	
 	priv = GET_PRIV (jabber);
 	
