@@ -31,31 +31,28 @@
 
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_ACCOUNT_CHOOSER, GossipAccountChooserPriv))
 
-
 typedef struct {
 	GossipSession        *session;
 	GossipAccountManager *account_manager;
 
-	gboolean              active_item_set;
+	gboolean              set_active_item;
+	gboolean              can_select_all;
 } GossipAccountChooserPriv;
 
-
 struct SetAccountData {
 	GossipAccountChooser *account_chooser;
 	GossipAccount        *account;
 	gboolean              set;
 };
 
-
 enum {
 	COL_ACCOUNT_IMAGE,
 	COL_ACCOUNT_TEXT,
-	COL_ACCOUNT_CONNECTED,
+	COL_ACCOUNT_ENABLED, /* usually tied to connected state */
 	COL_ACCOUNT_POINTER,
 	COL_ACCOUNT_COUNT
 };
 
-
 static void     account_chooser_finalize                 (GObject               *object);
 static void     account_chooser_setup                    (GossipAccountChooser  *account_chooser);
 static void     account_chooser_account_added_cb         (GossipAccountManager  *manager,
@@ -68,13 +65,13 @@ static void     account_chooser_account_removed_cb       (GossipAccountManager
 							  GossipAccountChooser  *account_chooser);
 static void     account_chooser_account_remove_foreach   (GossipAccount         *account,
 							  GossipAccountChooser  *account_chooser);
-static void     account_chooser_account_name_changed_cb  (GossipAccount         *account,
-							  GParamSpec            *param,
-							  GossipAccountChooser  *account_chooser);
 static gboolean account_chooser_account_name_foreach     (GtkTreeModel          *model,
 							  GtkTreePath           *path,
 							  GtkTreeIter           *iter,
 							  struct SetAccountData *data);
+static void     account_chooser_account_name_changed_cb  (GossipAccount         *account,
+							  GParamSpec            *param,
+							  GossipAccountChooser  *account_chooser);
 static gboolean account_chooser_account_update_foreach   (GtkTreeModel          *model,
 							  GtkTreePath           *path,
 							  GtkTreeIter           *iter,
@@ -91,11 +88,13 @@ static gboolean account_chooser_set_account_foreach      (GtkTreeModel
 							  GtkTreePath           *path,
 							  GtkTreeIter           *iter,
 							  struct SetAccountData *data);
-
+static gboolean account_chooser_set_enabled_foreach      (GtkTreeModel         *model,
+							  GtkTreePath          *path,
+							  GtkTreeIter          *iter,
+							  GossipAccountChooser *account_chooser);
 
 G_DEFINE_TYPE (GossipAccountChooser, gossip_account_chooser, GTK_TYPE_COMBO_BOX);
 
-
 static void
 gossip_account_chooser_class_init (GossipAccountChooserClass *klass)
 {
@@ -152,14 +151,14 @@ account_chooser_setup (GossipAccountChooser *account_chooser)
 
 	GtkListStore             *store;
 	GtkCellRenderer          *renderer;
-	GtkComboBox              *combo_box;
+	GtkComboBox              *combobox;
  
 	priv = GET_PRIV (account_chooser);
 
 	/* set up combo box with new store */
-	combo_box = GTK_COMBO_BOX (account_chooser);
+	combobox = GTK_COMBO_BOX (account_chooser);
 
-  	gtk_cell_layout_clear (GTK_CELL_LAYOUT (combo_box));  
+  	gtk_cell_layout_clear (GTK_CELL_LAYOUT (combobox));  
 
 	store = gtk_list_store_new (COL_ACCOUNT_COUNT,
 				    GDK_TYPE_PIXBUF,
@@ -167,20 +166,20 @@ account_chooser_setup (GossipAccountChooser *account_chooser)
 				    G_TYPE_BOOLEAN,
 				    GOSSIP_TYPE_ACCOUNT);    
 
-	gtk_combo_box_set_model (combo_box, GTK_TREE_MODEL (store));
+	gtk_combo_box_set_model (combobox, GTK_TREE_MODEL (store));
 
 	renderer = gtk_cell_renderer_pixbuf_new ();
-	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo_box), renderer, FALSE);
-	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo_box), renderer,
+	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combobox), renderer, FALSE);
+	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combobox), renderer,
 					"pixbuf", COL_ACCOUNT_IMAGE,
-					"sensitive", COL_ACCOUNT_CONNECTED,
+					"sensitive", COL_ACCOUNT_ENABLED,
 					NULL);
 
 	renderer = gtk_cell_renderer_text_new ();
-	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo_box), renderer, TRUE);
-	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo_box), renderer,
+	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combobox), renderer, TRUE);
+	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combobox), renderer,
 					"text", COL_ACCOUNT_TEXT,
-					"sensitive", COL_ACCOUNT_CONNECTED,
+					"sensitive", COL_ACCOUNT_ENABLED,
 					NULL);
 
 	/* populate accounts */
@@ -207,36 +206,45 @@ account_chooser_account_add_foreach (GossipAccount        *account,
 	GossipAccountChooserPriv *priv;
 
 	GtkListStore             *store;
-	GtkComboBox              *combo_box;
+	GtkComboBox              *combobox;
  	GtkTreeIter               iter;
 
 	GdkPixbuf                *pixbuf;
 	gboolean                  is_connected;
+	gboolean                  is_enabled;
 	
 	priv = GET_PRIV (account_chooser);
 
-	combo_box = GTK_COMBO_BOX (account_chooser);
-	store = GTK_LIST_STORE (gtk_combo_box_get_model (combo_box));
+	combobox = GTK_COMBO_BOX (account_chooser);
+	store = GTK_LIST_STORE (gtk_combo_box_get_model (combobox));
 
 	is_connected = gossip_session_is_connected (priv->session, account);
 	pixbuf = gossip_pixbuf_from_account_status (account, 
 						    GTK_ICON_SIZE_MENU,
 						    is_connected);
+
+	if (!priv->can_select_all && !is_connected) {
+		is_enabled = FALSE;
+	} else {
+		is_enabled = TRUE;
+	}
 	
 	gtk_list_store_append (store, &iter);
 	gtk_list_store_set (store, &iter, 
 			    COL_ACCOUNT_IMAGE, pixbuf, 
 			    COL_ACCOUNT_TEXT, gossip_account_get_name (account), 
-			    COL_ACCOUNT_CONNECTED, is_connected,
+			    COL_ACCOUNT_ENABLED, is_enabled,
 			    COL_ACCOUNT_POINTER, account,
 			    -1);
 	
 	g_object_unref (pixbuf);
 	
 	/* set first connected account as active account */
-	if (!priv->active_item_set && is_connected) {
-		priv->active_item_set = TRUE;
-		gtk_combo_box_set_active_iter (combo_box, &iter); 
+	if (priv->set_active_item == FALSE && 
+	    ((priv->can_select_all == TRUE) || 
+	     (priv->can_select_all == FALSE && is_connected == TRUE))) {
+		priv->set_active_item = TRUE;
+		gtk_combo_box_set_active_iter (combobox, &iter); 
 	}	
 	
 	g_signal_connect (account, "notify::name", 
@@ -261,32 +269,6 @@ account_chooser_account_remove_foreach (GossipAccount        *account,
 					      account_chooser);
 }
 
-static void
-account_chooser_account_name_changed_cb (GossipAccount        *account,
-					 GParamSpec           *param,
-					 GossipAccountChooser *account_chooser)
-{
-	GtkComboBox           *combobox;
-	GtkTreeModel          *model;
-	GtkTreeIter            iter;
-
-	struct SetAccountData  data;
-
-	g_return_if_fail (GOSSIP_IS_ACCOUNT_CHOOSER (account_chooser));
-	g_return_if_fail (GOSSIP_IS_ACCOUNT (account));
-	
-	combobox = GTK_COMBO_BOX (account_chooser);
-	model = gtk_combo_box_get_model (combobox);
-	gtk_combo_box_get_active_iter (combobox, &iter);
-
-	data.account_chooser = account_chooser;
-	data.account = account;
-
-	gtk_tree_model_foreach (model, 
-				(GtkTreeModelForeachFunc) account_chooser_account_name_foreach, 
-				&data);
-}
-
 static gboolean
 account_chooser_account_name_foreach (GtkTreeModel          *model,
 				      GtkTreePath           *path,
@@ -316,6 +298,32 @@ account_chooser_account_name_foreach (GtkTreeModel          *model,
 	return equal;
 }
 
+static void
+account_chooser_account_name_changed_cb (GossipAccount        *account,
+					 GParamSpec           *param,
+					 GossipAccountChooser *account_chooser)
+{
+	GtkComboBox           *combobox;
+	GtkTreeModel          *model;
+	GtkTreeIter            iter;
+
+	struct SetAccountData  data;
+
+	g_return_if_fail (GOSSIP_IS_ACCOUNT_CHOOSER (account_chooser));
+	g_return_if_fail (GOSSIP_IS_ACCOUNT (account));
+	
+	combobox = GTK_COMBO_BOX (account_chooser);
+	model = gtk_combo_box_get_model (combobox);
+	gtk_combo_box_get_active_iter (combobox, &iter);
+
+	data.account_chooser = account_chooser;
+	data.account = account;
+
+	gtk_tree_model_foreach (model, 
+				(GtkTreeModelForeachFunc) account_chooser_account_name_foreach, 
+				&data);
+}
+
 static gboolean
 account_chooser_account_update_foreach (GtkTreeModel          *model,
 					GtkTreePath           *path,
@@ -327,7 +335,7 @@ account_chooser_account_update_foreach (GtkTreeModel          *model,
 
 	account1 = GOSSIP_ACCOUNT (data->account);
 	gtk_tree_model_get (model, iter, COL_ACCOUNT_POINTER, &account2, -1);
-
+	
 	equal = gossip_account_equal (account1, account2);
 	g_object_unref (account2);
 
@@ -336,7 +344,8 @@ account_chooser_account_update_foreach (GtkTreeModel          *model,
 		GtkListStore             *store;
 		GdkPixbuf                *pixbuf;
 		gboolean                  is_connected;
-		
+		gboolean                  is_enabled;
+
 		priv = GET_PRIV (data->account_chooser);
 
 		store = GTK_LIST_STORE (model);
@@ -347,9 +356,15 @@ account_chooser_account_update_foreach (GtkTreeModel          *model,
 							    GTK_ICON_SIZE_MENU,
 							    is_connected);
 		
+		if (!priv->can_select_all && !is_connected) {
+			is_enabled = FALSE;
+		} else {
+			is_enabled = TRUE;
+		}
+
 		gtk_list_store_set (store, iter, 
 				    COL_ACCOUNT_IMAGE, pixbuf, 
-				    COL_ACCOUNT_CONNECTED, is_connected,
+				    COL_ACCOUNT_ENABLED, is_enabled,
 				    -1);
 		
 		g_object_unref (pixbuf);
@@ -474,6 +489,33 @@ gossip_account_chooser_get_account (GossipAccountChooser *account_chooser)
 	return account;
 }
 
+static gboolean
+account_chooser_set_account_foreach (GtkTreeModel          *model,
+				     GtkTreePath           *path,
+				     GtkTreeIter           *iter,
+				     struct SetAccountData *data)
+{
+	GossipAccount *account1, *account2;
+	gboolean       equal;
+
+	account1 = GOSSIP_ACCOUNT (data->account);
+	gtk_tree_model_get (model, iter, COL_ACCOUNT_POINTER, &account2, -1);
+
+	equal = gossip_account_equal (account1, account2);
+	g_object_unref (account2);
+
+	if (equal) {
+		GtkComboBox *combobox;
+
+		combobox = GTK_COMBO_BOX (data->account_chooser);
+		gtk_combo_box_set_active_iter (combobox, iter);
+
+		data->set = TRUE;
+	}
+		
+	return equal;
+}
+
 gboolean
 gossip_account_chooser_set_account (GossipAccountChooser *account_chooser,
 				    GossipAccount        *account) 
@@ -502,28 +544,66 @@ gossip_account_chooser_set_account (GossipAccountChooser *account_chooser,
 }
 
 static gboolean
-account_chooser_set_account_foreach (GtkTreeModel          *model,
-				     GtkTreePath           *path,
-				     GtkTreeIter           *iter,
-				     struct SetAccountData *data)
+account_chooser_set_enabled_foreach (GtkTreeModel         *model,
+				     GtkTreePath          *path,
+				     GtkTreeIter          *iter,
+				     GossipAccountChooser *account_chooser)
 {
-	GossipAccount *account1, *account2;
-	gboolean       equal;
+	GossipAccountChooserPriv *priv;
+	GossipAccount            *account;
+	GtkListStore             *store;
+	gboolean                  is_connected;
+	gboolean                  is_enabled;
 
-	account1 = GOSSIP_ACCOUNT (data->account);
-	gtk_tree_model_get (model, iter, COL_ACCOUNT_POINTER, &account2, -1);
+	priv = GET_PRIV (account_chooser);
+	
+	store = GTK_LIST_STORE (model);
+	
+	gtk_tree_model_get (model, iter, COL_ACCOUNT_POINTER, &account, -1);
 
-	equal = gossip_account_equal (account1, account2);
-	g_object_unref (account2);
+	is_connected = gossip_session_is_connected (priv->session, account);
+	g_object_unref (account);
+		
+	if (!priv->can_select_all && !is_connected) {
+		is_enabled = FALSE;
+	} else {
+		is_enabled = TRUE;
+	}
 
-	if (equal) {
+	gtk_list_store_set (store, iter, COL_ACCOUNT_ENABLED, is_enabled, -1);
+
+	if (priv->set_active_item == FALSE && 
+	    ((priv->can_select_all == TRUE) || 
+	     (priv->can_select_all == FALSE && is_connected == TRUE))) {
 		GtkComboBox *combobox;
 
-		combobox = GTK_COMBO_BOX (data->account_chooser);
-		gtk_combo_box_set_active_iter (combobox, iter);
+		combobox = GTK_COMBO_BOX (account_chooser);
 
-		data->set = TRUE;
+		priv->set_active_item = TRUE;
+		gtk_combo_box_set_active_iter (combobox, iter); 
 	}
-		
-	return equal;
+
+	return FALSE;
+}
+
+void
+gossip_account_chooser_set_can_select_all (GossipAccountChooser *account_chooser,
+					   gboolean              can_select_all)
+{
+	GossipAccountChooserPriv *priv;
+	GtkComboBox              *combobox;
+	GtkTreeModel             *model;
+
+	g_return_if_fail (GOSSIP_IS_ACCOUNT_CHOOSER (account_chooser));
+
+	priv = GET_PRIV (account_chooser);
+
+	combobox = GTK_COMBO_BOX (account_chooser);
+	model = gtk_combo_box_get_model (combobox);
+
+	priv->can_select_all = can_select_all;
+
+	gtk_tree_model_foreach (model, 
+				(GtkTreeModelForeachFunc) account_chooser_set_enabled_foreach, 
+				account_chooser);
 }
