@@ -39,11 +39,14 @@
 
 #define STRING_EMPTY(x) ((x) == NULL || (x)[0] == '\0')
 
+/* Flashing delay for icons (milliseconds). */
+#define FLASH_TIMEOUT 500
+
 typedef struct {
 	GtkWidget *window;
 
 	GtkWidget *treeview;
-	GtkWidget *notebook;
+	GtkWidget *vbox_details;
 
 	GtkWidget *button_remove;
 	GtkWidget *button_connect;
@@ -61,7 +64,9 @@ typedef struct {
 	GtkWidget *checkbutton_connect;
 
 	gboolean   account_changed;
-
+	
+	gboolean   connecting_show;
+	guint      connecting_id;
 } GossipAccountsDialog;
 
 typedef struct {
@@ -72,7 +77,7 @@ typedef struct {
 static void           accounts_dialog_setup                     (GossipAccountsDialog  *dialog);
 static void           accounts_dialog_update_connect_button     (GossipAccountsDialog  *dialog);
 static void           accounts_dialog_update_account            (GossipAccountsDialog  *dialog,
-								 GossipAccount         *account);
+								 GossipAccount         *account);   
 static void           accounts_dialog_block_widgets             (GossipAccountsDialog  *dialog,
 								 gboolean               block);
 static void           accounts_dialog_save                      (GossipAccountsDialog  *dialog,
@@ -105,6 +110,15 @@ static void           accounts_dialog_account_removed_cb        (GossipAccountMa
 static void           accounts_dialog_account_name_changed_cb   (GossipAccount         *account,
 								 GParamSpec            *param,
 								 GossipAccountsDialog  *dialog);
+static gboolean       accounts_dialog_flash_connecting_foreach  (GtkTreeModel          *model,
+								 GtkTreePath           *path,
+								 GtkTreeIter           *iter,
+								 gpointer               user_data);
+static gboolean       accounts_dialog_flash_connecting_cb       (GossipAccountsDialog  *dialog);
+static void           accounts_dialog_protocol_connecting_cb    (GossipSession         *session,
+								 GossipAccount         *account,
+								 GossipProtocol        *protocol,
+								 GossipAccountsDialog  *dialog);
 static void           accounts_dialog_protocol_connected_cb     (GossipSession         *session,
 								 GossipAccount         *account,
 								 GossipProtocol        *protocol,
@@ -125,26 +139,34 @@ static void           accounts_dialog_entry_port_insert_text_cb (GtkEditable
 								 gint                   len,
 								 gint                  *position,
 								 GossipAccountsDialog  *dialog);
-static void           accounts_dialog_button_add_clicked_cb     (GtkWidget             *button,
-								 GossipAccountsDialog  *dialog);
-static void           accounts_dialog_button_remove_clicked_cb  (GtkWidget             *button,
-								 GossipAccountsDialog  *dialog);
 static void           accounts_dialog_button_forget_clicked_cb  (GtkWidget             *button,
 								 GossipAccountsDialog  *dialog);
-static void           accounts_dialog_button_close_clicked_cb   (GtkWidget             *button,
+static void           accounts_dialog_button_add_clicked_cb     (GtkWidget             *button,
 								 GossipAccountsDialog  *dialog);
+static void           accounts_dialog_remove_response_cb        (GtkWidget             *dialog,
+								 gint                   response,
+								 GossipAccount         *account);
+static void           accounts_dialog_button_remove_clicked_cb  (GtkWidget             *button,
+								 GossipAccountsDialog  *dialog);
+static void           accounts_dialog_treeview_row_activated_cb (GtkTreeView           *tree_view,
+								 GtkTreePath           *path,
+								 GtkTreeViewColumn     *column,
+								 gpointer              *dialog);
 static gboolean       accounts_dialog_foreach                   (GtkTreeModel          *model,
 								 GtkTreePath           *path,
 								 GtkTreeIter           *iter,
 								 GossipAccountsDialog  *dialog);
+static void           accounts_dialog_response_cb               (GtkWidget             *widget,
+								 gint                   response,
+								 GossipAccountsDialog  *dialog);
 static void           accounts_dialog_destroy_cb                (GtkWidget             *widget,
 								 GossipAccountsDialog  *dialog);
-
 enum {
 	COL_NAME,
 	COL_EDITABLE,
 	COL_DEFAULT,
 	COL_CONNECTED, 
+	COL_CONNECTING, 
 	COL_AUTO_CONNECT,
 	COL_ACCOUNT_POINTER,
 	COL_COUNT
@@ -220,6 +242,7 @@ accounts_dialog_update_connect_button (GossipAccountsDialog *dialog)
 	const gchar      *stock_id;
 	const gchar      *label;
 	gboolean          is_connected;
+	gboolean          is_connecting;
 
 	view = GTK_TREE_VIEW (dialog->treeview);
 	selection = gtk_tree_view_get_selection (view);
@@ -228,17 +251,25 @@ accounts_dialog_update_connect_button (GossipAccountsDialog *dialog)
 		return;
 	}
 
-	gtk_tree_model_get (model, &iter, COL_CONNECTED, &is_connected, -1);
-	
-	/* The stock items are not defined correctly in GTK+ so we do it
-	 * ourselves (#318939). 
-	 */
-	if (is_connected) {
-		label = _("Disconnect");
-		stock_id = "gtk-disconnect";
+	gtk_tree_model_get (model, &iter, 
+			    COL_CONNECTED, &is_connected, 
+			    COL_CONNECTING, &is_connecting, 
+			    -1);
+
+	if (is_connecting) {
+		label = _("Stop");
+		stock_id = "gtk-stop";
 	} else {
-		label = _("Connect");
-		stock_id = "gtk-connect";
+		/* The stock items are not defined correctly in GTK+ so we do it
+		 * ourselves (#318939). 
+		 */
+		if (is_connected) {
+			label = _("Disconnect");
+			stock_id = "gtk-disconnect";
+		} else {
+			label = _("Connect");
+			stock_id = "gtk-connect";
+		}
 	}
 
 	image = gtk_image_new_from_stock (stock_id, GTK_ICON_SIZE_BUTTON);
@@ -441,7 +472,8 @@ accounts_dialog_model_setup (GossipAccountsDialog *dialog)
 				    G_TYPE_STRING,        /* name */
 				    G_TYPE_BOOLEAN,       /* editable */
 				    G_TYPE_BOOLEAN,       /* default */
-				    G_TYPE_BOOLEAN,       /* connected */
+ 				    G_TYPE_BOOLEAN,       /* connected */
+ 				    G_TYPE_BOOLEAN,       /* connecting */
 				    G_TYPE_BOOLEAN,       /* auto start */
 				    GOSSIP_TYPE_ACCOUNT); /* account */ 
 	
@@ -512,9 +544,9 @@ accounts_dialog_model_select_first (GossipAccountsDialog *dialog)
 	model = gtk_tree_view_get_model (view);
 	
 	if (!gtk_tree_model_get_iter_first (model, &iter)) {
-		gtk_widget_set_sensitive (dialog->notebook, FALSE);
+		gtk_widget_set_sensitive (dialog->vbox_details, FALSE);
 	} else {
-		gtk_widget_set_sensitive (dialog->notebook, TRUE);
+		gtk_widget_set_sensitive (dialog->vbox_details, TRUE);
 		
 		selection = gtk_tree_view_get_selection (view);
 		gtk_tree_selection_select_iter (selection, &iter);
@@ -531,29 +563,34 @@ accounts_dialog_model_pixbuf_data_func (GtkTreeViewColumn    *tree_column,
 	GossipAccount *account;
 	GdkPixbuf     *pixbuf;
 	gboolean       is_connected;
+	gboolean       is_connecting;
 
 	gtk_tree_model_get (model, iter, 
 			    COL_CONNECTED, &is_connected,
+			    COL_CONNECTING, &is_connecting,
 			    COL_ACCOUNT_POINTER, &account, 
 			    -1);
 
 	pixbuf = gossip_pixbuf_from_account (account, GTK_ICON_SIZE_BUTTON);
 
-	if (pixbuf && !is_connected) {
-		GdkPixbuf *modded_pixbuf;
-
-		modded_pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
-						TRUE,
-						8,
-						gdk_pixbuf_get_width (pixbuf), 
-						gdk_pixbuf_get_height (pixbuf));
-
-		gdk_pixbuf_saturate_and_pixelate (pixbuf,
-						  modded_pixbuf,
-						  1.0,
-						  TRUE);
-		g_object_unref (pixbuf);
-		pixbuf = modded_pixbuf;
+	if (pixbuf) {
+		if ((!is_connecting && !is_connected) || 
+		    (is_connecting && !dialog->connecting_show)) {
+			GdkPixbuf *modded_pixbuf;
+			
+			modded_pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
+							TRUE,
+							8,
+							gdk_pixbuf_get_width (pixbuf), 
+							gdk_pixbuf_get_height (pixbuf));
+			
+			gdk_pixbuf_saturate_and_pixelate (pixbuf,
+							  modded_pixbuf,
+							  1.0,
+							  TRUE);
+			g_object_unref (pixbuf);
+			pixbuf = modded_pixbuf;
+		}
 	}
 
 	g_object_set (cell, 
@@ -653,7 +690,7 @@ accounts_dialog_model_selection_changed (GtkTreeSelection     *selection,
 
 	is_selection = gtk_tree_selection_get_selected (selection, &model, &iter);
 
-	gtk_widget_set_sensitive (dialog->notebook, is_selection);
+	gtk_widget_set_sensitive (dialog->vbox_details, is_selection);
 	gtk_widget_set_sensitive (dialog->button_remove, is_selection);
 	gtk_widget_set_sensitive (dialog->button_connect, is_selection);
 
@@ -1002,6 +1039,102 @@ accounts_dialog_entry_port_insert_text_cb (GtkEditable          *editable,
 	}
 }
 
+static gboolean
+accounts_dialog_flash_connecting_foreach (GtkTreeModel *model,
+					  GtkTreePath  *path,
+					  GtkTreeIter  *iter,
+					  gpointer      user_data)
+{
+	gboolean is_connecting;
+
+	gtk_tree_model_get (model, iter, COL_CONNECTING, &is_connecting, -1);
+
+/* 	if (is_connecting) { */
+		gtk_tree_model_row_changed (model, path, iter);
+/* 	} */
+
+	return FALSE;
+}
+
+static gboolean
+accounts_dialog_flash_connecting_cb (GossipAccountsDialog *dialog)
+{
+	GtkTreeView  *view;
+	GtkTreeModel *model;
+
+	dialog->connecting_show = !dialog->connecting_show;
+
+	view = GTK_TREE_VIEW (dialog->treeview);
+	model = gtk_tree_view_get_model (view);
+
+	gtk_tree_model_foreach (model, accounts_dialog_flash_connecting_foreach, NULL);
+
+	return TRUE;
+}
+
+static void
+accounts_dialog_protocol_connecting_cb (GossipSession        *session,
+					GossipAccount        *account,
+					GossipProtocol       *protocol,
+					GossipAccountsDialog *dialog)
+{
+	GtkTreeView      *view;
+	GtkTreeSelection *selection;
+	GtkTreeModel     *model;
+	GtkTreeIter       iter;
+	gboolean          ok;
+	GossipAccount    *selected_account;
+
+	if (!dialog->connecting_id) {
+		dialog->connecting_id = g_timeout_add (FLASH_TIMEOUT, 
+						       (GSourceFunc) accounts_dialog_flash_connecting_cb, 
+						       dialog);
+	}
+
+	selected_account = accounts_dialog_model_get_selected (dialog);
+	if (selected_account) {
+		if (gossip_account_equal (selected_account, account)) {
+			gtk_widget_set_sensitive (dialog->button_remove, FALSE);
+		}
+		
+		g_object_unref (selected_account);
+	}
+
+	view = GTK_TREE_VIEW (dialog->treeview);
+	selection = gtk_tree_view_get_selection (view);
+	model = gtk_tree_view_get_model (view);
+
+	for (ok = gtk_tree_model_get_iter_first (model, &iter);
+	     ok;
+	     ok = gtk_tree_model_iter_next (model, &iter)) {
+		GossipAccount *this_account;
+		gboolean       equal;
+		
+		gtk_tree_model_get (model, &iter, 
+				    COL_ACCOUNT_POINTER, &this_account, 
+				    -1);
+		
+		equal = gossip_account_equal (this_account, account);
+		g_object_unref (this_account);
+
+		if (equal) {
+			GtkTreePath *path;
+
+			gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+					    COL_CONNECTING, TRUE,
+					    -1);
+
+			path = gtk_tree_model_get_path (model, &iter);
+			gtk_tree_model_row_changed (model, path, &iter);
+			gtk_tree_path_free (path);
+
+			break;
+		}
+	}
+
+	accounts_dialog_update_connect_button (dialog);
+}
+
 static void
 accounts_dialog_protocol_connected_cb (GossipSession        *session,
 				       GossipAccount        *account,
@@ -1013,7 +1146,15 @@ accounts_dialog_protocol_connected_cb (GossipSession        *session,
 	GtkTreeModel     *model;
 	GtkTreeIter       iter;
 	gboolean          ok;
+
 	GossipAccount    *selected_account;
+	guint             connecting;
+
+	gossip_session_count_accounts (session, NULL, &connecting, NULL); 
+	if (connecting < 1 && dialog->connecting_id) {
+		g_source_remove (dialog->connecting_id);
+		dialog->connecting_id = 0;
+	}
 
 	selected_account = accounts_dialog_model_get_selected (dialog);
 	if (selected_account) {
@@ -1046,6 +1187,7 @@ accounts_dialog_protocol_connected_cb (GossipSession        *session,
 
 			gtk_list_store_set (GTK_LIST_STORE (model), &iter,
 					    COL_CONNECTED, TRUE,
+					    COL_CONNECTING, FALSE,
 					    -1);
 
 			path = gtk_tree_model_get_path (model, &iter);
@@ -1072,6 +1214,13 @@ accounts_dialog_protocol_disconnected_cb (GossipSession        *session,
 	gboolean          ok;
 	
 	GossipAccount    *selected_account;
+	guint             connecting;
+
+	gossip_session_count_accounts (session, NULL, &connecting, NULL); 
+	if (connecting < 1 && dialog->connecting_id) {
+		g_source_remove (dialog->connecting_id);
+		dialog->connecting_id = 0;
+	}
 
 	selected_account = accounts_dialog_model_get_selected (dialog);
 	if (selected_account) {
@@ -1104,6 +1253,7 @@ accounts_dialog_protocol_disconnected_cb (GossipSession        *session,
 
 			gtk_list_store_set (GTK_LIST_STORE (model), &iter,
 					    COL_CONNECTED, FALSE,
+					    COL_CONNECTING, FALSE,
 					    -1);
 
 			path = gtk_tree_model_get_path (model, &iter);
@@ -1117,32 +1267,6 @@ accounts_dialog_protocol_disconnected_cb (GossipSession        *session,
 	accounts_dialog_update_connect_button (dialog);
 }
 
-static void
-accounts_dialog_button_add_clicked_cb (GtkWidget            *button,
-				       GossipAccountsDialog *dialog)
-{
-	gossip_new_account_window_show (GTK_WINDOW (dialog->window));
-}
-
-static void
-accounts_dialog_button_remove_clicked_cb (GtkWidget            *button,
-					  GossipAccountsDialog *dialog)
-{
-	GossipSession        *session;
-	GossipAccountManager *manager;
-	GossipAccount        *account;
-
-	session = gossip_app_get_session ();
- 	manager = gossip_session_get_account_manager (session);
-
-	account = accounts_dialog_model_get_selected (dialog);
-
-	gossip_account_manager_remove (manager, account);
-	gossip_account_manager_store (manager);
-
-	g_object_unref (account);
-}
-
 static void
 accounts_dialog_button_forget_clicked_cb (GtkWidget            *button,
 					  GossipAccountsDialog *dialog)
@@ -1168,29 +1292,129 @@ static void
 accounts_dialog_button_connect_clicked_cb (GtkWidget            *button,
 					   GossipAccountsDialog *dialog)
 {
+	GtkTreeView          *view;
+	GtkTreeModel         *model;
+	GtkTreeSelection     *selection;
+	GtkTreeIter           iter;
 	GossipSession        *session;
 	GossipAccountManager *manager;
 	GossipAccount        *account;
+	gboolean              is_connected = FALSE;
+	gboolean              is_connecting = FALSE;
+	gboolean              should_connect;
+	
+	view = GTK_TREE_VIEW (dialog->treeview);
+	selection = gtk_tree_view_get_selection (view);
+	
+	if (!gtk_tree_selection_get_selected (selection, &model, &iter)) {
+		g_warning ("No selection but connect button clicked?");
+		return;
+	}
 
 	session = gossip_app_get_session ();
  	manager = gossip_session_get_account_manager (session);
 
 	account = accounts_dialog_model_get_selected (dialog);
 
-	if (gossip_session_is_connected (session, account)) {
-		gossip_session_disconnect (session, account);
+	is_connected = gossip_session_is_connected (session, account);
+	is_connecting = gossip_session_is_connecting (session, account);
+
+	/* Make sure we update our information */
+	gtk_list_store_set (GTK_LIST_STORE (model), &iter, 
+			    COL_CONNECTED, is_connected,
+			    COL_CONNECTING, is_connecting,
+			    -1);
+
+	if (!is_connected) {
+		if (is_connecting) {
+			should_connect = FALSE;
+		} else {
+			should_connect = TRUE;
+		}
 	} else {
+		should_connect = FALSE;
+	}
+
+	if (should_connect) {
 		gossip_session_connect (session, account, FALSE);
+	} else {
+		gossip_session_disconnect (session, account);
+	}
+	
+	g_object_unref (account);
+}
+
+static void
+accounts_dialog_button_add_clicked_cb (GtkWidget            *button,
+				       GossipAccountsDialog *dialog)
+{
+	gossip_new_account_window_show (GTK_WINDOW (dialog->window));
+}
+
+static void
+accounts_dialog_remove_response_cb (GtkWidget     *dialog,
+				    gint           response,
+				    GossipAccount *account)
+{
+	gtk_widget_destroy (dialog);
+
+	if (response == GTK_RESPONSE_YES) {
+		GossipSession        *session;
+		GossipAccountManager *manager;
+
+		session = gossip_app_get_session ();
+		manager = gossip_session_get_account_manager (session);
+		
+		gossip_account_manager_remove (manager, account);
+		gossip_account_manager_store (manager);
 	}
 
 	g_object_unref (account);
 }
 
 static void
-accounts_dialog_button_close_clicked_cb (GtkWidget            *button,
-					 GossipAccountsDialog *dialog)
+accounts_dialog_button_remove_clicked_cb (GtkWidget            *button,
+					  GossipAccountsDialog *dialog)
 {
-	gtk_widget_destroy (dialog->window);
+	GossipAccount *account;
+	GtkWidget     *message_dialog;
+
+	account = accounts_dialog_model_get_selected (dialog);
+	
+	message_dialog = gtk_message_dialog_new 
+		(GTK_WINDOW (dialog->window), 
+		 GTK_DIALOG_DESTROY_WITH_PARENT,
+		 GTK_MESSAGE_QUESTION,
+		 GTK_BUTTONS_YES_NO,
+		 _("You are about to remove your %s account!\n"
+		   "Are you sure you want to proceed?"),
+		 gossip_account_get_name (account));
+
+	gtk_message_dialog_format_secondary_text 
+		(GTK_MESSAGE_DIALOG (message_dialog),
+		 _("Any associated conversations and chat rooms will NOT be "
+		   "removed if you decide to proceed.\n"
+		   "\n"
+		   "Should you decide to add the account back at a later time, "
+		   "they will still be available."));
+
+	g_signal_connect (message_dialog, "response",
+			  G_CALLBACK (accounts_dialog_remove_response_cb),
+			  account);
+
+	gtk_widget_show (message_dialog);
+}
+
+static void
+accounts_dialog_treeview_row_activated_cb (GtkTreeView           *tree_view,
+					   GtkTreePath           *path,
+					   GtkTreeViewColumn     *column,
+					   gpointer              *data)
+{
+	GossipAccountsDialog *dialog = (GossipAccountsDialog *) data;
+
+	accounts_dialog_button_connect_clicked_cb (dialog->button_connect,
+						   dialog);
 }
 
 static gboolean
@@ -1212,6 +1436,14 @@ accounts_dialog_foreach (GtkTreeModel         *model,
 	return FALSE;
 }
 
+static void
+accounts_dialog_response_cb (GtkWidget            *widget,
+			     gint                  response,
+			     GossipAccountsDialog *dialog)
+{
+	gtk_widget_destroy (widget);
+}
+
 static void
 accounts_dialog_destroy_cb (GtkWidget            *widget,
 			    GossipAccountsDialog *dialog)
@@ -1231,6 +1463,10 @@ accounts_dialog_destroy_cb (GtkWidget            *widget,
 					      accounts_dialog_account_removed_cb, 
 					      dialog);
 
+	g_signal_handlers_disconnect_by_func (session,
+					      accounts_dialog_protocol_connecting_cb, 
+					      dialog);
+
 	g_signal_handlers_disconnect_by_func (session,
 					      accounts_dialog_protocol_connected_cb, 
 					      dialog);
@@ -1253,6 +1489,10 @@ accounts_dialog_destroy_cb (GtkWidget            *widget,
 		g_object_unref (account);
 	}
 
+	if (dialog->connecting_id) {
+		g_source_remove (dialog->connecting_id);
+	}
+
 	g_free (dialog);
 }
 
@@ -1275,13 +1515,13 @@ gossip_accounts_dialog_show (GossipAccount *account)
 	
 	dialog = g_new0 (GossipAccountsDialog, 1);
 
-	glade = gossip_glade_get_file (GLADEDIR "/connect.glade",
+	glade = gossip_glade_get_file (GLADEDIR "/main.glade",
 				       "accounts_dialog",
 				       NULL,
 				       "accounts_dialog", &dialog->window,
 				       "dialog-action_area", &bbox,
 				       "treeview", &dialog->treeview,
-				       "notebook", &dialog->notebook,
+				       "vbox_details", &dialog->vbox_details,
 				       "label_name", &label_name,
 				       "label_id", &label_id,
 				       "label_password", &label_password,
@@ -1306,6 +1546,7 @@ gossip_accounts_dialog_show (GossipAccount *account)
 	gossip_glade_connect (glade, 
 			      dialog,
 			      "accounts_dialog", "destroy", accounts_dialog_destroy_cb,
+			      "accounts_dialog", "response", accounts_dialog_response_cb,
 			      "entry_name", "changed", accounts_dialog_entry_changed_cb,
 			      "entry_id", "changed", accounts_dialog_entry_changed_cb,
 			      "entry_password", "changed", accounts_dialog_entry_changed_cb,
@@ -1322,11 +1563,11 @@ gossip_accounts_dialog_show (GossipAccount *account)
 			      "checkbutton_proxy", "toggled", accounts_dialog_checkbutton_toggled_cb,
 			      "checkbutton_ssl", "toggled", accounts_dialog_checkbutton_toggled_cb,
 			      "checkbutton_connect", "toggled", accounts_dialog_checkbutton_toggled_cb,
-			      "button_add", "clicked", accounts_dialog_button_add_clicked_cb,
-			      "button_remove", "clicked", accounts_dialog_button_remove_clicked_cb,
 			      "button_forget", "clicked", accounts_dialog_button_forget_clicked_cb,
 			      "button_connect", "clicked", accounts_dialog_button_connect_clicked_cb,
-			      "button_close", "clicked", accounts_dialog_button_close_clicked_cb,
+			      "button_add", "clicked", accounts_dialog_button_add_clicked_cb,
+			      "button_remove", "clicked", accounts_dialog_button_remove_clicked_cb,
+			      "treeview", "row-activated", accounts_dialog_treeview_row_activated_cb,
 			      NULL);
 
 	g_object_add_weak_pointer (G_OBJECT (dialog->window), (gpointer) &dialog);
@@ -1344,6 +1585,10 @@ gossip_accounts_dialog_show (GossipAccount *account)
 			  G_CALLBACK (accounts_dialog_account_removed_cb), 
 			  dialog);
 
+	g_signal_connect (session, "protocol-connecting",
+			  G_CALLBACK (accounts_dialog_protocol_connecting_cb), 
+			  dialog);
+
 	g_signal_connect (session, "protocol-connected",
 			  G_CALLBACK (accounts_dialog_protocol_connected_cb), 
 			  dialog);
@@ -1372,14 +1617,14 @@ gossip_accounts_dialog_show (GossipAccount *account)
 
 	g_object_unref (size_group);
 
-	gtk_button_box_set_child_secondary (GTK_BUTTON_BOX (bbox), 
-					    button_close, TRUE);
+/* 	gtk_button_box_set_child_secondary (GTK_BUTTON_BOX (bbox),  */
+/* 					    button_close, TRUE); */
 	
 	gtk_window_set_transient_for (GTK_WINDOW (dialog->window), 
 				      GTK_WINDOW (gossip_app_get_window ()));
 	gtk_widget_show (dialog->window);
 
-	gtk_widget_set_sensitive (dialog->notebook, FALSE);
+	gtk_widget_set_sensitive (dialog->vbox_details, FALSE);
 
 	if (GOSSIP_IS_ACCOUNT (account)) {
 		/* if account was specified then we select it */
