@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2003-2005 Imendio AB
+ * Copyright (C) 2003-2006 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,11 +19,9 @@
  */
 
 #include <config.h>
-
 #include <sys/types.h>
 #include <string.h>
 #include <time.h>
-
 #include <glib/gi18n.h>
 #include <gtk/gtkimage.h>
 #include <gtk/gtkmenu.h>
@@ -38,6 +36,7 @@
 #include <libgnome/gnome-url.h>
 #endif
 
+#include <libgossip/gossip-debug.h>
 #include <libgossip/gossip-time.h>
 #include <libgossip/gossip-utils.h>
 #include <libgossip/gossip-session.h>
@@ -48,16 +47,15 @@
 #include "gossip-preferences.h"
 #include "gossip-theme-manager.h"
 
-#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_CHAT_VIEW, GossipChatViewPriv))
+#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj),	\
+		       GOSSIP_TYPE_CHAT_VIEW, GossipChatViewPriv))
 
-#define DEBUG_MSG(x)
-/* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n"); */
+#define DEBUG_DOMAIN "ChatView"
 
 /* Number of seconds between timestamps when using normal mode, 5 minutes. */
 #define TIMESTAMP_INTERVAL 300
 
-/* Maximum lines in any chat buffer, see bug #141292. */
-#define MAX_LINES 1000
+#define MAX_LINES 800
 
 typedef enum {
 	BLOCK_TYPE_NONE,
@@ -82,13 +80,13 @@ struct _GossipChatViewPriv {
 	 */
 	GossipContact *last_contact;
 
-	guint          gconf_system_fonts_id;
-	guint          gconf_use_system_fonts_id;
+	guint          notify_system_fonts_id;
+	guint          notify_show_avatars_id;
 };
 
 typedef struct {
 	GossipSmiley  smiley;
-	gchar        *pattern;
+	const gchar  *pattern;
 	gint          index;
 } GossipSmileyPattern;
 
@@ -182,8 +180,6 @@ static GossipSmileyPattern smileys[] = {
 	{ GOSSIP_SMILEY_SICK,         ")o+", 0 }
 };
 
-static gint num_smileys = G_N_ELEMENTS (smileys);
-
 static void     gossip_chat_view_class_init          (GossipChatViewClass      *klass);
 static void     gossip_chat_view_init                (GossipChatView           *view);
 static void     chat_view_finalize                   (GObject                  *object);
@@ -197,10 +193,14 @@ static void     chat_view_size_allocate              (GtkWidget                *
 static void     chat_view_setup_tags                 (GossipChatView           *view);
 static void     chat_view_system_font_update         (GossipChatView           *view,
 						      GConfClient              *gconf_client);
-static void     chat_view_system_font_changed_cb     (GConfClient              *gconf_client,
+static void     chat_view_system_font_notify_cb      (GConfClient              *gconf_client,
 						      guint                     id,
 						      GConfEntry               *entry,
-						      GossipChatView           *view);
+						      gpointer                  user_data);
+static void     chat_view_show_avatars_notify_cb     (GConfClient              *gconf_client,
+						      guint                     id,
+						      GConfEntry               *entry,
+						      gpointer                  user_data);
 static void     chat_view_populate_popup             (GossipChatView           *view,
 						      GtkMenu                  *menu,
 						      gpointer                  user_data);
@@ -231,7 +231,7 @@ static void     chat_view_invite_join_cb             (GossipChatroomProvider   *
 						      GossipChatroomJoinResult  result,
 						      gint                      id,
 						      gpointer                  user_data);
-static void     theme_manager_theme_changed_cb       (GossipThemeManager       *manager,
+static void     chat_view_theme_changed_cb           (GossipThemeManager       *manager,
 						      GossipChatView           *view);
 static void     chat_view_maybe_append_date_and_time (GossipChatView           *view,
 						      GossipMessage            *msg);
@@ -242,7 +242,8 @@ static void     chat_view_append_text                (GossipChatView           *
 static void     chat_view_maybe_append_fancy_header  (GossipChatView           *view,
 						      GossipMessage            *msg,
 						      GossipContact            *my_contact,
-						      gboolean                  from_self);
+						      gboolean                  from_self,
+						      GdkPixbuf                *avatar);
 static void     chat_view_append_irc_action          (GossipChatView           *view,
 						      GossipMessage            *msg,
 						      GossipContact            *my_contact,
@@ -298,21 +299,29 @@ gossip_chat_view_init (GossipChatView *view)
 
 	gconf_client = gossip_app_get_gconf_client ();
 	
-	/* Watch system font changes and preferences to turn it on/off too */
-	priv->gconf_system_fonts_id = 
+	priv->notify_system_fonts_id = 
 		gconf_client_notify_add (gconf_client,
 					 "/desktop/gnome/interface/document_font_name",
-					 (GConfClientNotifyFunc) chat_view_system_font_changed_cb,
+					 chat_view_system_font_notify_cb,
+					 view, NULL, NULL);
+	chat_view_system_font_update (view, gconf_client);
+	
+	priv->notify_show_avatars_id = 
+		gconf_client_notify_add (gconf_client,
+					 GCONF_UI_SHOW_AVATARS,
+					 chat_view_show_avatars_notify_cb,
 					 view, NULL, NULL);
 
-	/* Should we use GNOME's document font */
-	chat_view_system_font_update (view, gconf_client);
-
-	/* Set up formatting tags */
 	chat_view_setup_tags (view);
 
 	gossip_theme_manager_apply (gossip_theme_manager_get (), view);
-	
+	gossip_theme_manager_update_show_avatars (gossip_theme_manager_get (),
+						  view,
+						  gconf_client_get_bool (
+							  gconf_client,
+							  GCONF_UI_SHOW_AVATARS,
+							  NULL));
+
 	g_signal_connect (view,
 			  "populate-popup",
 			  G_CALLBACK (chat_view_populate_popup),
@@ -320,7 +329,7 @@ gossip_chat_view_init (GossipChatView *view)
 
 	g_signal_connect_object (gossip_theme_manager_get (),
 				 "theme-changed",
-				 G_CALLBACK (theme_manager_theme_changed_cb),
+				 G_CALLBACK (chat_view_theme_changed_cb),
 				 view,
 				 0);
 }
@@ -336,16 +345,8 @@ chat_view_finalize (GObject *object)
 	priv = GET_PRIV (view);
 
 	gconf_client = gossip_app_get_gconf_client ();
-
-	if (priv->gconf_system_fonts_id) {
-		gconf_client_notify_remove (gconf_client, priv->gconf_system_fonts_id);
-		priv->gconf_system_fonts_id = 0;
-	}
-
-	if (priv->gconf_use_system_fonts_id) {
-		gconf_client_notify_remove (gconf_client, priv->gconf_use_system_fonts_id);
-		priv->gconf_use_system_fonts_id = 0;
-	}
+	gconf_client_notify_remove (gconf_client, priv->notify_system_fonts_id);
+	gconf_client_notify_remove (gconf_client, priv->notify_show_avatars_id);
 
 	G_OBJECT_CLASS (gossip_chat_view_parent_class)->finalize (object);
 }
@@ -391,6 +392,9 @@ chat_view_setup_tags (GossipChatView *view)
 				    "cut",
 				    NULL);
 
+	/* FIXME: Move to the theme and come up with something that looks a bit
+	 * nicer.
+	 */
 	gtk_text_buffer_create_tag (priv->buffer,
 				    "highlight",
 				    "background", "yellow",
@@ -437,12 +441,46 @@ chat_view_system_font_update (GossipChatView *view,
 }
 
 static void
-chat_view_system_font_changed_cb (GConfClient    *gconf_client,
+chat_view_system_font_notify_cb (GConfClient    *gconf_client,
+				 guint           id,
+				 GConfEntry     *entry,
+				 gpointer        user_data)
+{
+	GossipChatView *view;
+
+	view = user_data;
+	chat_view_system_font_update (view, gconf_client);
+	
+	/* Ugly, again, to adjust the vertical position of the nick... Will fix
+	 * this when reworking the theme manager so that view register
+	 * themselves with it instead of the other way around.
+	 */
+	gossip_theme_manager_update_show_avatars (gossip_theme_manager_get (),
+						  view,
+						  gconf_client_get_bool (
+							  gconf_client,
+							  GCONF_UI_SHOW_AVATARS,
+							  NULL));
+}
+
+static void
+chat_view_show_avatars_notify_cb (GConfClient    *gconf_client,
 				  guint           id,
 				  GConfEntry     *entry,
-				  GossipChatView *view)
+				  gpointer        user_data)
 {
-	chat_view_system_font_update (view, gconf_client);	
+	GossipChatView     *view;
+	GossipChatViewPriv *priv;
+	GConfValue         *value;
+
+	view = user_data;
+	priv = GET_PRIV (view);
+
+	value = gconf_entry_get_value (entry);
+	
+	gossip_theme_manager_update_show_avatars (gossip_theme_manager_get (),
+						  view,
+						  gconf_value_get_bool (value));
 }
 
 static void
@@ -534,7 +572,6 @@ chat_view_event_cb (GossipChatView *view,
 		    GtkTextTag     *tag)
 {
 	static GdkCursor  *hand = NULL;
-	static GdkCursor  *beam = NULL;
 	GtkTextWindowType  type;
 	GtkTextIter        iter;
 	GdkWindow         *win;
@@ -549,6 +586,10 @@ chat_view_event_cb (GossipChatView *view,
 
 	/* Get where the pointer really is. */
 	win = gtk_text_view_get_window (GTK_TEXT_VIEW (view), type);
+	if (!win) {
+		return FALSE;
+	}
+
 	gdk_window_get_pointer (win, &x, &y, NULL);
 
 	/* Get the iter where the cursor is at */
@@ -560,15 +601,13 @@ chat_view_event_cb (GossipChatView *view,
 					    &iter,
 					    buf_x, buf_y);
 
-	if (!hand) {
-		hand = gdk_cursor_new (GDK_HAND2);
-		beam = gdk_cursor_new (GDK_XTERM);
-	}
-
 	if (gtk_text_iter_has_tag (&iter, tag)) {
+		if (!hand) {
+			hand = gdk_cursor_new (GDK_HAND2);
+		}
 		gdk_window_set_cursor (win, hand);
 	} else {
-		gdk_window_set_cursor (win, beam);
+		gdk_window_set_cursor (win, NULL);
 	}
 
 	return FALSE;
@@ -611,16 +650,43 @@ chat_view_url_event_cb (GtkTextTag    *tag,
 static void
 chat_view_open_address (const gchar *url)
 {
-#ifdef USE_GNOMEVFS_FOR_URL
-	GnomeVFSResult result;
+	gchar *real_url;
 
-	result = gnome_vfs_url_show (url);
-	if (result == GNOME_VFS_OK) {
-		g_warning ("Couldn't show URL:'%s'", url);
+	/* The URL opening code can't handle schemeless strings, so we try to be
+	 * smart and add http if there is no scheme or doesn't look like a mail
+	 * address. This should work in most cases, and let us click on strings
+	 * like "www.gnome.org".
+	 */
+	if (!g_str_has_prefix (url, "http://") &&
+	    !strstr (url, ":/") &&
+	    !strstr (url, "@")) {
+		real_url = g_strdup_printf ("http://%s", url);
+	} else {
+		real_url = g_strdup (url);
+	}
+
+#ifdef USE_GNOMEVFS_FOR_URL
+	{
+		GnomeVFSResult result;
+		
+		result = gnome_vfs_url_show (real_url);
+		if (result != GNOME_VFS_OK) {
+			g_warning ("Couldn't show URL:'%s'", real_url);
+		}
 	}
 #else
-	gnome_url_show (url, NULL);
+	{
+		GError *error = NULL;
+		
+		gnome_url_show (real_url, &error);
+		if (error) {
+			g_warning ("Couldn't show URL:'%s'", real_url);
+			g_error_free (error);
+		}
+	}
 #endif
+
+	g_free (real_url);
 }
 
 static void
@@ -674,7 +740,7 @@ chat_view_insert_text_with_emoticons (GtkTextBuffer *buf,
 		gint         len;
 		const gchar *start;
 
-		for (i = 0; i < num_smileys; i++) {
+		for (i = 0; i < G_N_ELEMENTS (smileys); i++) {
 			smileys[i].index = 0;
 		}
 
@@ -695,7 +761,7 @@ chat_view_insert_text_with_emoticons (GtkTextBuffer *buf,
 			if (submatch != -1 || prev_c == 0 || g_unichar_isspace (prev_c)) {
 				submatch = -1;
 
-				for (i = 0; i < num_smileys; i++) {
+				for (i = 0; i < G_N_ELEMENTS (smileys); i++) {
 					if (smileys[i].pattern[smileys[i].index] == c) {
 						submatch = i;
 
@@ -1020,7 +1086,8 @@ static void
 chat_view_maybe_append_fancy_header (GossipChatView *view,
 				     GossipMessage  *msg,
 				     GossipContact  *my_contact,
-				     gboolean        from_self)
+				     gboolean        from_self,
+				     GdkPixbuf      *avatar)
 {
 	GossipChatViewPriv *priv;
 	GossipContact      *contact;
@@ -1029,24 +1096,28 @@ chat_view_maybe_append_fancy_header (GossipChatView *view,
 	GtkTextIter         iter;
 	gchar              *tmp;
 	const gchar        *tag;
-	const gchar        *line_tag;
+	const gchar        *avatar_tag;
+	const gchar        *line_top_tag;
+	const gchar        *line_bottom_tag;
 
 	priv = GET_PRIV (view);
 
 	contact = gossip_message_get_sender (msg);
 	
-	DEBUG_MSG (("ChatView: Maybe add fancy header"));
+	gossip_debug (DEBUG_DOMAIN, "Maybe add fancy header");
 
 	if (from_self) {
 		name = gossip_contact_get_name (my_contact);
 		
 		tag = "fancy-header-self";
-		line_tag = "fancy-line-self";
+		line_top_tag = "fancy-line-top-self";
+		line_bottom_tag = "fancy-line-bottom-self";
 	} else {
 		name = gossip_contact_get_name (contact);
 		
 		tag = "fancy-header-other";
-		line_tag = "fancy-line-other";
+		line_top_tag = "fancy-line-top-other";
+		line_bottom_tag = "fancy-line-bottom-other";
 	}
 
 	header = FALSE;
@@ -1081,25 +1152,53 @@ chat_view_maybe_append_fancy_header (GossipChatView *view,
 						  &iter,
 						  "\n",
 						  -1,
-						  line_tag,
+						  line_top_tag,
 						  NULL);
 
+	if (avatar) {
+		GtkTextIter start;
+		
+		gtk_text_buffer_get_end_iter (priv->buffer, &iter);
+ 		gtk_text_buffer_insert_pixbuf (priv->buffer, &iter, avatar);
+
+		gtk_text_buffer_get_end_iter (priv->buffer, &iter);
+		start = iter;
+		gtk_text_iter_backward_char (&start);
+
+		if (from_self) {
+			gtk_text_buffer_apply_tag_by_name (priv->buffer,
+							   "fancy-avatar-self",
+							   &start, &iter);
+			avatar_tag = "fancy-header-self-avatar";
+		} else {
+			gtk_text_buffer_apply_tag_by_name (priv->buffer,
+							   "fancy-avatar-other",
+							   &start, &iter);
+			avatar_tag = "fancy-header-other-avatar";
+		}			
+		
+	} else {
+		avatar_tag = NULL;
+	}
+
 	tmp = g_strdup_printf ("%s\n", name);
+
 	gtk_text_buffer_get_end_iter (priv->buffer, &iter);
 	gtk_text_buffer_insert_with_tags_by_name (priv->buffer,
 						  &iter,
 						  tmp,
 						  -1,
 						  tag,
+						  avatar_tag,
 						  NULL);
 	g_free (tmp);
-	
+
 	gtk_text_buffer_get_end_iter (priv->buffer, &iter);
 	gtk_text_buffer_insert_with_tags_by_name (priv->buffer,
 						  &iter,
 						  "\n",
 						  -1,
-						  line_tag,
+						  line_bottom_tag,
 						  NULL);
 }
 
@@ -1118,7 +1217,7 @@ chat_view_append_irc_action (GossipChatView *view,
 
 	priv = GET_PRIV (view);
 
-	DEBUG_MSG (("ChatView: Add IRC action"));
+	gossip_debug (DEBUG_DOMAIN, "Add IRC action");
 
 	/* Skip the "/me ". */
 	if (from_self) {
@@ -1172,7 +1271,7 @@ chat_view_append_fancy_action (GossipChatView *view,
 
 	priv = GET_PRIV (view);
 
-	DEBUG_MSG (("ChatView: Add fancy action"));
+	gossip_debug (DEBUG_DOMAIN, "Add fancy action");
 
 	contact = gossip_message_get_sender (msg);
 
@@ -1221,7 +1320,7 @@ chat_view_append_irc_message (GossipChatView *view,
 
 	priv = GET_PRIV (view);
 
-	DEBUG_MSG (("ChatView: Add IRC message"));
+	gossip_debug (DEBUG_DOMAIN, "Add IRC message");
 
 	body = gossip_message_get_body (msg);
 
@@ -1362,8 +1461,8 @@ chat_view_invite_join_cb (GossipChatroomProvider   *provider,
 }
 
 static void
-theme_manager_theme_changed_cb (GossipThemeManager *manager,
-				GossipChatView     *view)
+chat_view_theme_changed_cb (GossipThemeManager *manager,
+			    GossipChatView     *view)
 {
 	GossipChatViewPriv *priv;
 	
@@ -1372,6 +1471,16 @@ theme_manager_theme_changed_cb (GossipThemeManager *manager,
 	priv->last_block_type = BLOCK_TYPE_NONE;
 	
 	gossip_theme_manager_apply (manager, view);
+
+	/* Needed for now to update the "rise" property of the names to get it
+	 * vertically centered.
+	 */
+	gossip_theme_manager_update_show_avatars (manager,
+						  view,
+						  gconf_client_get_bool (
+							  gossip_app_get_gconf_client (),
+							  GCONF_UI_SHOW_AVATARS,
+							  NULL));
 }
 
 GossipChatView *
@@ -1384,7 +1493,8 @@ gossip_chat_view_new (void)
 void
 gossip_chat_view_append_message_from_self (GossipChatView *view,
 					   GossipMessage  *msg,
-					   GossipContact  *my_contact)
+					   GossipContact  *my_contact,
+					   GdkPixbuf      *avatar)
 {
 	GossipChatViewPriv *priv;
 	const gchar        *body;
@@ -1403,7 +1513,9 @@ gossip_chat_view_append_message_from_self (GossipChatView *view,
 	chat_view_maybe_append_date_and_time (view, msg);
 
 	if (!priv->irc_style) {
-		chat_view_maybe_append_fancy_header (view, msg, my_contact, TRUE);
+		chat_view_maybe_append_fancy_header (view, msg,
+						     my_contact,
+						     TRUE, avatar);
 	}
 	
 	/* Handle action messages (/me) and normal messages, in combination with
@@ -1440,7 +1552,8 @@ gossip_chat_view_append_message_from_self (GossipChatView *view,
 void
 gossip_chat_view_append_message_from_other (GossipChatView *view,
 					    GossipMessage  *msg,
-					    GossipContact  *contact)
+					    GossipContact  *contact,
+					    GdkPixbuf      *avatar)
 {
 	GossipChatViewPriv *priv;
 	const gchar        *body;
@@ -1459,7 +1572,9 @@ gossip_chat_view_append_message_from_other (GossipChatView *view,
 	chat_view_maybe_append_date_and_time (view, msg);
 
 	if (!priv->irc_style) {
-		chat_view_maybe_append_fancy_header (view, msg, contact, FALSE);
+		chat_view_maybe_append_fancy_header (view, msg,
+						     contact, FALSE,
+						     avatar);
 	}
 	
 	/* Handle action messages (/me) and normal messages, in combination with
@@ -1517,7 +1632,8 @@ gossip_chat_view_append_event (GossipChatView *view,
 	}
 	
 	if (priv->last_block_type != BLOCK_TYPE_EVENT) {
-		chat_view_append_spacing (view);
+		/* Comment out for now. */
+		/*chat_view_append_spacing (view);*/
 	}
 
 	chat_view_maybe_append_date_and_time (view, NULL);
@@ -1682,14 +1798,14 @@ gossip_chat_view_scroll (GossipChatView *view,
 
 	priv->allow_scrolling = allow_scrolling;
 
-	DEBUG_MSG (("ChatView: Scrolling %s", allow_scrolling ? "enabled" : "disabled"));
+	gossip_debug (DEBUG_DOMAIN, "Scrolling %s",
+		      allow_scrolling ? "enabled" : "disabled");
 }
 
 void
 gossip_chat_view_scroll_down (GossipChatView *view)
 {
 	GossipChatViewPriv *priv;
-
 	GtkTextBuffer      *buffer;
 	GtkTextIter         iter;
 	GtkTextMark        *mark;
@@ -1700,7 +1816,7 @@ gossip_chat_view_scroll_down (GossipChatView *view)
 		return;
 	}
 
-	DEBUG_MSG (("ChatView: Scrolling down"));
+	gossip_debug (DEBUG_DOMAIN, "Scrolling down");
 
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
 
@@ -1762,15 +1878,11 @@ gossip_chat_view_find (GossipChatView *view,
 	GtkTextIter         iter_at_mark;
 	GtkTextIter         iter_match_start;
 	GtkTextIter         iter_match_end;
-
 	static GtkTextMark *find_mark = NULL;
 	static gboolean     wrapped = FALSE;
-        
 	gboolean            found;
 	gboolean            from_start = FALSE;
 
-	gchar              *str;
-
 	g_return_if_fail (GOSSIP_IS_CHAT_VIEW (view));
 	g_return_if_fail (search_criteria != NULL);
 	g_return_if_fail (strlen (search_criteria) > 0);
@@ -1791,14 +1903,11 @@ gossip_chat_view_find (GossipChatView *view,
 		from_start = TRUE;
 	}
 
-	str = g_utf8_casefold (search_criteria, -1);
-	found = gtk_text_iter_forward_search (&iter_at_mark, 
-					      str, 
-					      GTK_TEXT_SEARCH_TEXT_ONLY, 
-					      &iter_match_start, 
-					      &iter_match_end,
-					      NULL);
-	g_free (str);
+	found = gossip_text_iter_forward_search (&iter_at_mark,
+						 search_criteria,
+						 &iter_match_start, 
+						 &iter_match_end,
+						 NULL);
     
 	if (!found) {
 		if (from_start) {
@@ -1838,11 +1947,8 @@ gossip_chat_view_highlight (GossipChatView *view,
 	GtkTextIter    iter_end;
 	GtkTextIter    iter_match_start;
 	GtkTextIter    iter_match_end;
-	        
 	gboolean       found;
 
-	gchar         *str;
-
 	g_return_if_fail (GOSSIP_IS_CHAT_VIEW (view));
 
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (view));
@@ -1858,15 +1964,12 @@ gossip_chat_view_highlight (GossipChatView *view,
 		return;
 	}
 
-	str = g_utf8_casefold (text, -1);
-
 	while (TRUE) {
-		found = gtk_text_iter_forward_search (&iter, 
-						      str, 
-						      GTK_TEXT_SEARCH_TEXT_ONLY, 
-						      &iter_match_start, 
-						      &iter_match_end,
-						      NULL);
+		found = gossip_text_iter_forward_search (&iter, 
+							 text, 
+							 &iter_match_start, 
+							 &iter_match_end,
+							 NULL);
 
 		if (!found) {
 			break;
@@ -1879,8 +1982,6 @@ gossip_chat_view_highlight (GossipChatView *view,
 		iter = iter_match_end;
 		gtk_text_iter_forward_char (&iter);
 	}
-
-	g_free (str);
 }
 
 void
@@ -1914,20 +2015,24 @@ gossip_chat_view_set_irc_style (GossipChatView *view,
 				gboolean        irc_style)
 {
 	GossipChatViewPriv *priv;
-	gint                margin;
 	
 	g_return_if_fail (GOSSIP_IS_CHAT_VIEW (view));
 
 	priv = GET_PRIV (view);
 
 	priv->irc_style = irc_style;
+}
 
-	if (priv->irc_style) {
-		margin = 3;
-	} else {
-		margin = 0;
-	}
+void
+gossip_chat_view_set_margin (GossipChatView *view,
+			     gint            margin)
+{
+	GossipChatViewPriv *priv;
 	
+	g_return_if_fail (GOSSIP_IS_CHAT_VIEW (view));
+
+	priv = GET_PRIV (view);
+
 	g_object_set (view,
 		      "left-margin", margin,
 		      "right-margin", margin,
@@ -1970,22 +2075,21 @@ gossip_chat_view_get_smiley_text (GossipSmiley smiley)
 }
 
 GtkWidget *
-gossip_chat_view_get_smiley_menu (GCallback callback, 
-				  gpointer  user_data)
+gossip_chat_view_get_smiley_menu (GCallback    callback, 
+				  gpointer     user_data,
+				  GtkTooltips *tooltips)
 {
 	GtkWidget *menu;
 	gint       x;
 	gint       y;
 	gint       i;
 
-	/* Set up smiley menu */
 	menu = gtk_menu_new ();
 
 	for (i = 0, x = 0, y = 0; i < GOSSIP_SMILEY_COUNT; i++) {
 		GtkWidget   *item;
 		GtkWidget   *image;
 		GdkPixbuf   *pixbuf;
-		GtkTooltips *tooltips;
 		const gchar *smiley_text;
 
 		pixbuf = gossip_chat_view_get_smiley_image (i);
@@ -2003,7 +2107,6 @@ gossip_chat_view_get_smiley_menu (GCallback callback,
 
 		smiley_text = gossip_chat_view_get_smiley_text (i);
 
-		tooltips = gtk_tooltips_new ();
 		gtk_tooltips_set_tip (tooltips, 
 				      item, 
 				      smiley_text, 
@@ -2024,3 +2127,4 @@ gossip_chat_view_get_smiley_menu (GCallback callback,
 
 	return menu;
 }
+
