@@ -225,7 +225,7 @@ static GtkTargetEntry drop_types[] = {
 
 G_DEFINE_TYPE (GossipChatWindow, gossip_chat_window, G_TYPE_OBJECT);
 
-#define URGENCY_TIMEOUT 15*1000
+#define URGENCY_TIMEOUT 60*1000
 
 static void
 gossip_chat_window_class_init (GossipChatWindowClass *klass)
@@ -261,6 +261,9 @@ gossip_chat_window_init (GossipChatWindow *window)
 
 	priv = GET_PRIV (window);
 
+	priv->tooltips = g_object_ref (gtk_tooltips_new ());
+	gtk_object_sink (GTK_OBJECT (priv->tooltips));
+	
 	glade = gossip_glade_get_file (GLADEDIR "/chat.glade",
 				       "chat_window",
 				       NULL,
@@ -335,7 +338,8 @@ gossip_chat_window_init (GossipChatWindow *window)
 	/* Set up smiley menu */
 	menu = gossip_chat_view_get_smiley_menu (
 		G_CALLBACK (chat_window_insert_smiley_activate_cb), 
-		window);
+		window,
+		priv->tooltips);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (priv->menu_edit_insert_smiley), menu);
 
 	/* Set up signals we can't do with glade since we may need to
@@ -409,8 +413,6 @@ gossip_chat_window_init (GossipChatWindow *window)
 	priv->chats_new_msg = NULL;
 	priv->chats_composing = NULL;
 	priv->current_chat = NULL;
-
-	priv->tooltips = gtk_tooltips_new ();
 }
 
 /* Returns the window to open a new tab in if there is only one window
@@ -484,6 +486,8 @@ gossip_chat_window_finalize (GObject *object)
 	chat_windows = g_list_remove (chat_windows, window);
 	gtk_widget_destroy (priv->dialog);
 
+	g_object_unref (priv->tooltips);
+
 	G_OBJECT_CLASS (gossip_chat_window_parent_class)->finalize (object);
 }
 
@@ -756,6 +760,8 @@ chat_window_update_status (GossipChatWindow *window,
 	pixbuf = chat_window_get_status_pixbuf (window, chat);
 	image = g_object_get_data (G_OBJECT (chat), "chat-window-tab-image");
 	gtk_image_set_from_pixbuf (image, pixbuf);
+
+	g_object_unref (pixbuf);
 	
 	chat_window_update_tooltip (window, chat);
 }
@@ -786,6 +792,7 @@ chat_window_update_title (GossipChatWindow *window,
 		pixbuf = gossip_pixbuf_from_stock (GOSSIP_STOCK_MESSAGE,
 						   GTK_ICON_SIZE_MENU);
 		gtk_window_set_icon (GTK_WINDOW (priv->dialog), pixbuf);
+		g_object_unref (pixbuf);
         } else {
 		gtk_window_set_icon (GTK_WINDOW (priv->dialog), NULL);
 		chat_window_set_urgency_hint (window, FALSE);
@@ -923,8 +930,7 @@ chat_window_insert_smiley_activate_cb (GtkWidget        *menuitem,
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (chat->input_text_view));
 	gtk_text_buffer_get_end_iter (buffer, &iter);
 	gtk_text_buffer_insert (buffer, &iter, 
-				smiley, 
-				g_utf8_strlen (smiley, -1));
+				smiley, -1);
 }
 
 static void
@@ -1394,11 +1400,9 @@ chat_window_new_message_cb (GossipChat       *chat,
 			DEBUG_MSG (("ChatWindow: Should we highlight this nick?"));
 			if (gossip_chat_should_highlight_nick (message, own_contact)) {
 				chat_window_set_urgency_hint (window, TRUE);
-				DEBUG_MSG (("ChatWindow: Turning on urgency hint"));
 			}
 		} else {
 			chat_window_set_urgency_hint (window, TRUE);
-			DEBUG_MSG (("ChatWindow: Turning on urgency hint"));
 		}
 	}
 
@@ -1683,6 +1687,7 @@ chat_window_urgency_timeout_func (GossipChatWindow *window)
 
 	priv = GET_PRIV (window);
 
+	DEBUG_MSG (("ChatWindow: Turning off urgency hint"));
 	gtk_window_set_urgency_hint (GTK_WINDOW (priv->dialog), FALSE);
 
 	priv->urgency_timeout_id = 0;
@@ -1701,8 +1706,10 @@ chat_window_set_urgency_hint (GossipChatWindow *window,
 	if (!urgent) {
 		/* Remove any existing hint and timeout. */
 		if (priv->urgency_timeout_id) {
+			DEBUG_MSG (("ChatWindow: Turning off urgency hint"));
 			gtk_window_set_urgency_hint (GTK_WINDOW (priv->dialog), FALSE);
 			g_source_remove (priv->urgency_timeout_id);
+			priv->urgency_timeout_id = 0;
 		}
 		return;
 	}	
@@ -1711,6 +1718,7 @@ chat_window_set_urgency_hint (GossipChatWindow *window,
 	if (priv->urgency_timeout_id) {
 		g_source_remove (priv->urgency_timeout_id);
 	} else {
+		DEBUG_MSG (("ChatWindow: Turning on urgency hint"));
 		gtk_window_set_urgency_hint (GTK_WINDOW (priv->dialog), TRUE);
 	}
 
