@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2002-2005 Imendio AB
+ * Copyright (C) 2002-2006 Imendio AB
  * Copyright (C) 2003-2004 Geert-Jan Van den Bogaerde <geertjan@gnome.org>
  * Copyright (C) 2004      Martyn Russell <mr@gnome.org>
  *
@@ -21,10 +21,8 @@
  */
 
 #include <config.h>
-
 #include <string.h>
 #include <stdlib.h>
-
 #include <gtk/gtk.h>
 #include <gconf/gconf-client.h>
 #include <glib/gi18n.h>
@@ -47,18 +45,14 @@
 #define CHAT_DIR_CREATE_MODE    (S_IRUSR | S_IWUSR | S_IXUSR)
 #define CHAT_FILE_CREATE_MODE   (S_IRUSR | S_IWUSR)
 
-#define CHAT_KEY_FILENAME       "geometry.ini"
-#define CHAT_KEY_GROUP_CONTACTS "Chat"
-
-#define CHAT_GEOMETRY_FORMAT    "%d,%d,%d,%d"
-
 struct _GossipChatPriv {
 	GossipChatWindow *window;
 
-	GossipSpell      *spell;
-
-	/* Used to automatically shrink a window that has temporarily
-	   grown due to long input */
+	GtkTooltips      *tooltips;
+	
+	/* Used to automatically shrink a window that has temporarily grown due
+	 * to long input.
+	 */
 	gint              padding_height;
 	gint              default_window_height;
 	gint              last_input_height;
@@ -85,7 +79,7 @@ static void      chat_text_populate_popup_cb       (GtkTextView     *view,
 						    GossipChat      *chat);
 static void      chat_text_check_word_spelling_cb  (GtkMenuItem     *menuitem,
 						    GossipChatSpell *chat_spell);
-GossipChatSpell *chat_spell_new                    (GossipChat      *chat,
+static GossipChatSpell *chat_spell_new             (GossipChat      *chat,
 						    const gchar     *word,
 						    GtkTextIter      start,
 						    GtkTextIter      end);
@@ -161,9 +155,6 @@ gossip_chat_init (GossipChat *chat)
 	GossipChatPriv *priv;
 	GtkTextBuffer  *buffer;
 
-	gchar          *value;
-	GList          *languages = NULL;
-
 	chat->view = gossip_chat_view_new ();
 	chat->input_text_view = gtk_text_view_new ();
 	chat->is_first_char = TRUE;
@@ -179,6 +170,8 @@ gossip_chat_init (GossipChat *chat)
 
 	priv = GET_PRIV (chat);
 
+	priv->tooltips = gtk_tooltips_new ();
+	
 	priv->default_window_height = -1;
 
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (chat->input_text_view));
@@ -202,34 +195,6 @@ gossip_chat_init (GossipChat *chat)
 				    "misspelled",
 				    "underline", PANGO_UNDERLINE_ERROR,
 				    NULL);
-
-	/* get spelling languages */
-	value = gconf_client_get_string (gossip_app_get_gconf_client (),
-					 GCONF_CHAT_SPELL_CHECKER_LANGUAGES,
-					 NULL);
-	
-	if (value) {
-		gchar **vlanguages;
-		gchar  *lang;
-		gint    i;
-
-		vlanguages = g_strsplit (value, ",", -1);
-		
-		for (i = 0, lang = vlanguages[i]; lang; lang = vlanguages[++i]) {
-			languages = g_list_append (languages, g_strdup (lang));
-		}
-
-		g_strfreev (vlanguages);
-	}
-	
-	g_free (value);
-	
-	priv->spell = gossip_spell_new (languages);
-	
-	if (languages) {
-		g_list_foreach (languages, (GFunc)g_free, NULL);
-		g_list_free (languages);
-	}
 }
 
 static void
@@ -238,15 +203,9 @@ chat_finalize (GObject *object)
 	GossipChat     *chat;
 	GossipChatPriv *priv;
 
-	g_return_if_fail (GOSSIP_IS_CHAT (object));
-
 	chat = GOSSIP_CHAT (object);
 	priv = GET_PRIV (chat);
 
-	if (priv->spell) {
-		gossip_spell_unref (priv->spell);
-	}
-
 	G_OBJECT_CLASS (gossip_chat_parent_class)->finalize (object);
 }
 
@@ -254,7 +213,6 @@ static void
 chat_input_text_buffer_changed_cb (GtkTextBuffer *buffer, GossipChat *chat)
 {
 	GossipChatPriv *priv;
-
 	GtkTextIter     start, end;
 	gchar          *str;
 	gboolean        spell_checker;
@@ -295,7 +253,7 @@ chat_input_text_buffer_changed_cb (GtkTextBuffer *buffer, GossipChat *chat)
 		return;
 	}
 
-	if (!gossip_spell_supported () || !gossip_spell_has_backend (priv->spell)) {
+	if (!gossip_spell_supported ()) {
 		return;
 	}
 	
@@ -327,7 +285,7 @@ chat_input_text_buffer_changed_cb (GtkTextBuffer *buffer, GossipChat *chat)
 
 		/* spell check string */
   		if (!gossip_chat_get_is_command (str)) { 
-			correct = gossip_spell_check (priv->spell, str);
+			correct = gossip_spell_check (str);
  		} else { 
  			correct = TRUE; 
  		} 
@@ -432,8 +390,7 @@ chat_insert_smiley_activate_cb (GtkWidget  *menuitem,
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (chat->input_text_view));
 	gtk_text_buffer_get_end_iter (buffer, &iter);
 	gtk_text_buffer_insert (buffer, &iter, 
-				smiley, 
-				g_utf8_strlen (smiley, -1));
+				smiley, -1);
 }
 
 static void
@@ -441,6 +398,7 @@ chat_text_populate_popup_cb (GtkTextView *view,
 			     GtkMenu     *menu,
 			     GossipChat  *chat)
 {
+	GossipChatPriv  *priv;
 	GtkTextBuffer   *buffer;
 	GtkTextTagTable *table;
 	GtkTextTag      *tag;
@@ -451,10 +409,9 @@ chat_text_populate_popup_cb (GtkTextView *view,
 	GossipChatSpell *chat_spell;
 	GtkWidget       *smiley_menu;
 
-	/*
-	 * Add the emoticon menu.
-	 */
-
+	priv = GET_PRIV (chat);
+	
+	/* Add the emoticon menu. */
 	item = gtk_separator_menu_item_new ();
 	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), item);
 	gtk_widget_show (item);
@@ -465,13 +422,11 @@ chat_text_populate_popup_cb (GtkTextView *view,
 
 	smiley_menu = gossip_chat_view_get_smiley_menu (
 		G_CALLBACK (chat_insert_smiley_activate_cb), 
-		chat);
+		chat,
+		priv->tooltips);
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), smiley_menu);
 
-	/*
-	 * Add the spell check menu item.
-	 */
-
+	/* Add the spell check menu item. */
 	buffer = gtk_text_view_get_buffer (view);
 	table = gtk_text_buffer_get_tag_table (buffer);
 
@@ -522,18 +477,13 @@ static void
 chat_text_check_word_spelling_cb (GtkMenuItem     *menuitem, 
 				  GossipChatSpell *chat_spell)
 {
-	GossipChatPriv *priv;
-
-	priv = GET_PRIV (chat_spell->chat);
-
 	gossip_spell_dialog_show (chat_spell->chat,
-				  priv->spell,
 				  chat_spell->start,
 				  chat_spell->end,
 				  chat_spell->word);
 }
 
-GossipChatSpell *
+static GossipChatSpell *
 chat_spell_new (GossipChat  *chat, 
 		const gchar *word,
 		GtkTextIter  start,
@@ -541,16 +491,10 @@ chat_spell_new (GossipChat  *chat,
 {
 	GossipChatSpell *chat_spell;
 
-	g_return_val_if_fail (chat != NULL, NULL);
-	g_return_val_if_fail (word != NULL, NULL);
-
 	chat_spell = g_new0 (GossipChatSpell, 1);
 
 	chat_spell->chat = g_object_ref (chat);
-
-
 	chat_spell->word = g_strdup (word);
-
 	chat_spell->start = start;
 	chat_spell->end = end;
 	
@@ -569,7 +513,10 @@ gboolean
 gossip_chat_get_is_command (const gchar *str)
 {
 	g_return_val_if_fail (str != NULL, FALSE);
-	g_return_val_if_fail (strlen (str) > 0, FALSE);
+
+	if (str[0] != '/') {
+		return FALSE;
+	}
 	
 	if (g_str_has_prefix (str, "/me")) {
 		return TRUE;
@@ -594,14 +541,13 @@ gossip_chat_correct_word (GossipChat  *chat,
 
 	g_return_if_fail (chat != NULL);
 	g_return_if_fail (new_word != NULL);
-	g_return_if_fail (strlen (new_word) > 0);
 
 	buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (chat->input_text_view));
 
 	gtk_text_buffer_delete (buffer, &start, &end);
 	gtk_text_buffer_insert (buffer, &start, 
 				new_word, 
-				g_utf8_strlen (new_word, -1));
+				-1);
 }
 
 const gchar *
@@ -893,12 +839,12 @@ gossip_chat_should_highlight_nick (GossipMessage *message,
 	gchar         *ch;
 	gboolean       ret_val;
 
+	g_return_val_if_fail (GOSSIP_IS_MESSAGE (message), FALSE);
+
 	DEBUG_MSG (("Chat: Highlighting nickname"));
 
 	ret_val = FALSE;
 
-	g_return_val_if_fail (GOSSIP_IS_MESSAGE (message), FALSE);
-
 	msg = gossip_message_get_body (message);
 	if (!msg) {
 		return FALSE;
@@ -932,8 +878,8 @@ gossip_chat_should_highlight_nick (GossipMessage *message,
 		}
 	}
 
-	ch = ch + g_utf8_strlen (cf_to, -1);
-	if (ch >= cf_msg + g_utf8_strlen (cf_msg, -1)) {
+	ch = ch + strlen (cf_to);
+	if (ch >= cf_msg + strlen (cf_msg)) {
 		ret_val = TRUE;
 		goto finished;
 	}
