@@ -70,6 +70,8 @@ typedef struct {
 
 	gulong         contact_signal_handler;
 	gulong         presence_signal_handler;
+
+	guint          show_timeout_id;
 } GossipContactInfoDialog;
 
 static void contact_info_dialog_init                 (void);
@@ -138,7 +140,6 @@ contact_info_dialog_update_presences (GossipContactInfoDialog *dialog)
 	GdkPixbuf   *pixbuf = NULL;
 	const gchar *status = NULL;
 	const gchar *resource = NULL;
-
 	GList       *presences, *l;
 	gint         i = 0;
 	gint         cols = 4;
@@ -422,14 +423,8 @@ contact_info_dialog_get_version_cb (GossipResult       result,
 	}
 
 	if (show_client_info) {
-		GtkSizeGroup *size_group;
-
 		gtk_widget_hide (dialog->client_status_hbox);
 		gtk_widget_show (dialog->client_table);
-
-		size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
-		gtk_size_group_add_widget (size_group, dialog->stub_client_label);
-		g_object_unref (size_group);
 	} else {
 		gtk_widget_hide (dialog->client_vbox);
 	}
@@ -511,6 +506,10 @@ contact_info_dialog_destroy_cb (GtkWidget               *widget,
 					     dialog->presence_signal_handler);
 	}
 
+	if (dialog->show_timeout_id) {
+		g_source_remove (dialog->show_timeout_id);
+	}
+	
 	g_hash_table_remove (contact_info_dialogs, dialog->contact);
 	
 	g_free (dialog);
@@ -524,6 +523,16 @@ contact_info_dialog_response_cb (GtkWidget               *widget,
 	gtk_widget_destroy (dialog->dialog);
 }
 
+static gboolean
+contact_info_dialog_show_timeout_cb (GossipContactInfoDialog *dialog)
+{
+	gtk_widget_show (dialog->dialog);
+
+	dialog->show_timeout_id = 0;
+
+	return FALSE;
+}
+
 void
 gossip_contact_info_dialog_show (GossipContact *contact)
 {
@@ -531,14 +540,14 @@ gossip_contact_info_dialog_show (GossipContact *contact)
 	GossipSession           *session;
 	GossipAccount           *account;
 	GladeXML                *glade;
-	gchar                   *str, *tmp_str;
+	gchar                   *str;
 	GtkSizeGroup            *size_group;
 	guint                    id;
 	GtkWidget               *avatar_image_placeholder;
 	GdkPixbuf               *pixbuf;
 
 	contact_info_dialog_init ();
-	
+
 	dialog = g_hash_table_lookup (contact_info_dialogs, contact);
 	if (dialog) {
 		gtk_window_present (GTK_WINDOW (dialog->dialog));
@@ -617,38 +626,32 @@ gossip_contact_info_dialog_show (GossipContact *contact)
 
 	g_object_unref (size_group);
 	
-	/* set labels */
-	tmp_str = g_strdup_printf (_("Contact Information for %s"), 
-				   gossip_contact_get_name (contact));
-
-	str = g_markup_escape_text (tmp_str, -1);
-	g_free (tmp_str);
-	
+	/* Set labels */
 	gtk_label_set_text (GTK_LABEL (dialog->id_label), 
 			    gossip_contact_get_id (contact));
 
 	gtk_label_set_text (GTK_LABEL (dialog->name_label), 
 			    gossip_contact_get_name (contact));
 
-	/* set details */
+	/* Set details */
 	contact_info_dialog_update_subscription (dialog);
 	contact_info_dialog_update_presences (dialog);
 		
-	/* subscription listener */
+	/* Subscription listener */
 	id = g_signal_connect (session,
 			       "contact-updated",
 			       G_CALLBACK (contact_info_dialog_contact_updated_cb), 
 			       NULL);
 	dialog->contact_signal_handler = id;
 
-	/* presence listener */
+	/* Presence listener */
 	id = g_signal_connect (session,
 			       "contact-presence-updated",
 			       G_CALLBACK (contact_info_dialog_presence_updated_cb),
 			       NULL);
 	dialog->presence_signal_handler = id;
 
-	/* get vcard and version info */
+	/* Get vcard and version info */
 	str = g_strdup_printf ("<i>%s</i>", 
 			       _("Information requested..."));
 	gtk_label_set_markup (GTK_LABEL (dialog->personal_status_label), str);
@@ -671,6 +674,9 @@ gossip_contact_info_dialog_show (GossipContact *contact)
 				    g_object_ref (contact), 
 				    NULL);
 	
-	gtk_widget_show (dialog->dialog);
+	dialog->show_timeout_id =
+		g_timeout_add (500,
+			       (GSourceFunc) contact_info_dialog_show_timeout_cb,
+			       dialog);
 }
 
