@@ -24,6 +24,7 @@
 #include <gtk/gtk.h>
 
 #include <libgossip/gossip-session.h>
+#include <libgossip/gossip-debug.h>
 
 #include "gossip-app.h"
 #include "gossip-cell-renderer-text.h"
@@ -43,10 +44,10 @@
 #include "gossip-notify.h"
 #endif
 
-#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_CONTACT_LIST, GossipContactListPriv))
+#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), \
+		       GOSSIP_TYPE_CONTACT_LIST, GossipContactListPriv))
 
-#define DEBUG_MSG(x)  
-/* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n");  */
+#define DEBUG_DOMAIN "ContactList"
 
 /* Flashing delay for icons (milliseconds). */
 #define FLASH_TIMEOUT 500
@@ -63,6 +64,7 @@
 
 struct _GossipContactListPriv {
 	gboolean             show_offline;
+	gboolean             show_avatars;
 	gboolean             show_active;
 
 	GHashTable          *groups;
@@ -76,12 +78,19 @@ struct _GossipContactListPriv {
 };
 
 typedef struct {
-	gboolean flash_on;
-	guint    flash_timeout_id;
+	gint            event_id;
+	GossipEventType event_type;
+	gboolean        flash_on;
+	guint           flash_timeout_id;
 } FlashData;
 
 typedef struct {
-	gchar       *name;
+	GossipContactList *list;
+	GossipContact     *contact;
+} FlashTimeoutData;
+
+typedef struct {
+	const gchar *name;
 	gboolean     found;
 	GtkTreeIter  iter;
 } FindGroup;
@@ -160,6 +169,11 @@ static void     contact_list_add_contact                 (GossipContactList  *li
 static void     contact_list_remove_contact              (GossipContactList  *list,
 							  GossipContact      *contact);
 static void     contact_list_create_model                (GossipContactList  *list);
+static gboolean contact_list_search_equal_func           (GtkTreeModel       *model,
+							  gint                column,
+							  const gchar        *key,
+							  GtkTreeIter        *iter,
+							  gpointer            search_data);
 static void     contact_list_setup_view                  (GossipContactList  *list);
 static void     contact_list_drag_data_received          (GtkWidget          *widget,
 							  GdkDragContext     *context,
@@ -209,19 +223,19 @@ static void     contact_list_text_cell_data_func         (GtkTreeViewColumn  *tr
 							  GossipContactList  *list);
 static gboolean contact_list_button_press_event_cb       (GossipContactList  *list,
 							  GdkEventButton     *event,
-							  gpointer            unused);
+							  gpointer            user_data);
 static void     contact_list_row_activated_cb            (GossipContactList  *list,
 							  GtkTreePath        *path,
 							  GtkTreeViewColumn  *col,
-							  gpointer            unused);
+							  gpointer            user_data);
 static void     contact_list_row_expand_or_collapse_cb   (GossipContactList  *list,
 							  GtkTreeIter        *iter,
 							  GtkTreePath        *path,
-							  gpointer            unused);
+							  gpointer            user_data);
 static gint     contact_list_sort_func                   (GtkTreeModel       *model,
 							  GtkTreeIter        *iter_a,
 							  GtkTreeIter        *iter_b,
-							  gpointer            unused);
+							  gpointer            user_data);
 static GList *  contact_list_find_contact                (GossipContactList  *list,
 							  GossipContact      *contact);
 static gboolean contact_list_find_contact_foreach        (GtkTreeModel       *model,
@@ -251,13 +265,19 @@ static void     contact_list_item_menu_remove_cb         (gpointer            da
 static void     contact_list_group_menu_rename_cb        (gpointer            data,
 							  guint               action,
 							  GtkWidget          *widget);
+static void     contact_list_free_flash_timeout_data     (FlashTimeoutData   *timeout_data);
+static void     contact_list_flash_free_data             (FlashData          *data);
+static gboolean contact_list_flash_timeout_func          (FlashTimeoutData   *timeout_data);
 static void     contact_list_event_added_cb              (GossipEventManager *manager,
 							  GossipEvent        *event,
 							  GossipContactList  *list);
 static void     contact_list_event_removed_cb            (GossipEventManager *manager,
 							  GossipEvent        *event,
 							  GossipContactList  *list);
-static void     contact_list_flash_free_data             (FlashData          *data);
+static gboolean contact_list_set_show_avatars_foreach    (GtkTreeModel       *model,
+							  GtkTreePath        *path,
+							  GtkTreeIter        *iter,
+							  gpointer            show_avatars);
 
 enum {
         CONTACT_ACTIVATED,
@@ -269,6 +289,7 @@ static guint signals[LAST_SIGNAL];
 enum {
 	COL_PIXBUF_STATUS,
 	COL_PIXBUF_AVATAR,
+	COL_PIXBUF_AVATAR_VISIBLE,
 	COL_NAME,
 	COL_STATUS,
 	COL_CONTACT,
@@ -281,7 +302,8 @@ enum {
 
 enum {
 	PROP_0,
-	PROP_SHOW_OFFLINE
+	PROP_SHOW_OFFLINE,
+	PROP_SHOW_AVATARS
 };
 
 enum {
@@ -418,9 +440,9 @@ gossip_contact_list_class_init (GossipContactListClass *klass)
                               G_SIGNAL_RUN_LAST,
                               0,
                               NULL, NULL,
-                              gossip_marshal_VOID__POINTER,
+                              gossip_marshal_VOID__OBJECT_INT,
                               G_TYPE_NONE,
-                              1, G_TYPE_POINTER);
+                              2, GOSSIP_TYPE_CONTACT, G_TYPE_INT);
 
 	param = g_param_spec_boolean ("show_offline",
 				      "Show Offline",
@@ -433,6 +455,17 @@ gossip_contact_list_class_init (GossipContactListClass *klass)
 					 PROP_SHOW_OFFLINE,
 					 param);
 
+	param = g_param_spec_boolean ("show_avatars",
+				      "Show Avatars",
+				      "Whether contact list should display "
+				      "avatars for contacts",
+				      TRUE,
+				      G_PARAM_READWRITE);
+
+	g_object_class_install_property (object_class,
+					 PROP_SHOW_AVATARS,
+					 param);
+
 	g_type_class_add_private (object_class, sizeof (GossipContactListPriv));
 }
 
@@ -484,7 +517,7 @@ gossip_contact_list_init (GossipContactList *list)
 				       G_N_ELEMENTS (group_menu_items),
 				       group_menu_items,
 				       list);
-        
+	
         /* Signal connection. */
 	g_signal_connect (gossip_app_get_session (),
 			  "connected",
@@ -543,6 +576,7 @@ gossip_contact_list_init (GossipContactList *list)
 static void
 contact_list_finalize (GObject *object)
 {
+	G_OBJECT_CLASS (gossip_contact_list_parent_class)->finalize (object);
 }
 
 static void
@@ -559,6 +593,9 @@ contact_list_get_property (GObject    *object,
 	case PROP_SHOW_OFFLINE:
 		g_value_set_boolean (value, priv->show_offline);
 		break;
+	case PROP_SHOW_AVATARS:
+		g_value_set_boolean (value, priv->show_avatars);
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -580,6 +617,10 @@ contact_list_set_property (GObject      *object,
 		gossip_contact_list_set_show_offline (GOSSIP_CONTACT_LIST (object),
 						      g_value_get_boolean (value));
 		break;
+	case PROP_SHOW_AVATARS:
+		gossip_contact_list_set_show_avatars (GOSSIP_CONTACT_LIST (object),
+						      g_value_get_boolean (value));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -589,7 +630,7 @@ contact_list_set_property (GObject      *object,
 static void
 contact_list_connected_cb (GossipSession *session, GossipContactList *list)
 {
-	DEBUG_MSG (("ContactList: Connected"));
+	gossip_debug (DEBUG_DOMAIN, "Connected");
 
 	g_timeout_add (ACTIVE_USER_WAIT_TO_ENABLE_TIME, 
 		       (GSourceFunc) contact_list_show_active_users_cb,
@@ -619,8 +660,8 @@ contact_list_contact_added_cb (GossipSession     *session,
 
 	priv = GET_PRIV (list);
 
-	DEBUG_MSG (("ContactList: Contact added: %s",
-		   gossip_contact_get_name (contact)));
+	gossip_debug (DEBUG_DOMAIN, "Contact added: %s",
+		      gossip_contact_get_name (contact));
 
 	if (!priv->show_offline && !gossip_contact_is_online (contact)) {
 		return;
@@ -647,16 +688,16 @@ contact_list_contact_updated_cb (GossipSession     *session,
 
 	type = gossip_contact_get_type (contact);
 	if (type != GOSSIP_CONTACT_TYPE_CONTACTLIST) {
-		DEBUG_MSG (("ContactList: Update to none-contact list "
-			   "contact (doing nothing)"));
+		gossip_debug (DEBUG_DOMAIN, "Update to none-contact list "
+			      "contact (doing nothing)");
 		return;
 	}
-
+	
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (list));
 	presence = gossip_contact_get_active_presence (contact);
 
-	DEBUG_MSG (("ContactList: Contact updated: %s",
-		   gossip_contact_get_name (contact)));
+	gossip_debug (DEBUG_DOMAIN, "Contact updated: %s",
+		      gossip_contact_get_name (contact));
 
 	if (!priv->show_offline && !gossip_contact_is_online (contact)) {
 		return;
@@ -688,22 +729,21 @@ contact_list_contact_presence_updated_cb (GossipSession     *session,
 	gboolean               do_remove = FALSE;
 	gboolean               do_set_active = FALSE;
 	gboolean               do_set_refresh = FALSE;
-
 	GossipAccount         *account;
 	gdouble                seconds = 0;
-
 	gboolean               is_composing;
 	const GdkPixbuf       *pixbuf;
  	GdkPixbuf             *pixbuf_composing;
 	GdkPixbuf             *pixbuf_presence;
 
 	priv = GET_PRIV (list);
+
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (list));
 
 	type = gossip_contact_get_type (contact);
 	if (type != GOSSIP_CONTACT_TYPE_CONTACTLIST) {
-		DEBUG_MSG (("ContactList: Presence from none-contact list "
-			   "contact (doing nothing)"));
+		gossip_debug (DEBUG_DOMAIN, "Presence from none-contact list "
+			      "contact (doing nothing)");
 		return;
 	}
 
@@ -742,9 +782,9 @@ contact_list_contact_presence_updated_cb (GossipSession     *session,
 			do_set_refresh = TRUE;
 
 			set_model = TRUE;
-			DEBUG_MSG (("ContactList: Remove item (after timeout)")); 
+			gossip_debug (DEBUG_DOMAIN, "Remove item (after timeout)"); 
 		} else {
-			DEBUG_MSG (("ContactList: Remove item (now)!")); 
+			gossip_debug (DEBUG_DOMAIN, "Remove item (now)!"); 
 			contact_list_remove_contact (list, contact);
 		}
 	}
@@ -760,7 +800,7 @@ contact_list_contact_presence_updated_cb (GossipSession     *session,
 		if (priv->show_active) {
 			do_set_active = TRUE;
 
-			DEBUG_MSG (("ContactList: Set active (contact added)")); 
+			gossip_debug (DEBUG_DOMAIN, "Set active (contact added)");
 		}
 
 	} else {
@@ -786,14 +826,13 @@ contact_list_contact_presence_updated_cb (GossipSession     *session,
 					str = "offline -> online"; 
 				}
 
-				DEBUG_MSG (("ContactList: Set active (contact updated %s)", str)); 
-
+				gossip_debug (DEBUG_DOMAIN, "Set active (contact updated %s)", str);
 			} else {
 				/* Was TRUE for presence updates. */
 				/* do_set_active = FALSE;  */
 				do_set_refresh = TRUE;
 				
-				DEBUG_MSG (("ContactList: Set active (contact updated)")); 
+				gossip_debug (DEBUG_DOMAIN, "Set active (contact updated)");
 			}
 		}
 
@@ -850,8 +889,8 @@ contact_list_contact_removed_cb (GossipSession     *session,
 				 GossipContact     *contact,
 				 GossipContactList *list)
 {
-	DEBUG_MSG (("ContactList: Contact removed: %s",
-		   gossip_contact_get_name (contact)));
+	gossip_debug (DEBUG_DOMAIN, "Contact removed: %s",
+		   gossip_contact_get_name (contact));
 
 	contact_list_remove_contact (list, contact);
 }
@@ -866,10 +905,10 @@ contact_list_contact_composing_cb (GossipSession     *session,
 	GtkTreeModel          *model;
 	GList                 *iters, *l;
 	GdkPixbuf             *pixbuf = NULL;
-
-	DEBUG_MSG (("ContactList: Contact %s typing:'%s'",
-		   composing ? "is" : "is not",
-		   gossip_contact_get_name (contact)));
+	
+	gossip_debug (DEBUG_DOMAIN, "Contact %s typing: '%s'",
+		      composing ? "is" : "is not",
+		      gossip_contact_get_name (contact));
 
 	priv = GET_PRIV (list);
 
@@ -935,7 +974,7 @@ contact_list_contact_set_active (GossipContactList *list,
 		gtk_tree_store_set (GTK_TREE_STORE (model), iter,
 				    COL_IS_ACTIVE, active,
 				    -1);
-		DEBUG_MSG (("ContactList: Set item %s", active ? "active" : "inactive"));
+		gossip_debug (DEBUG_DOMAIN, "Set item %s", active ? "active" : "inactive");
 	
  		if (set_changed) { 
  			path = gtk_tree_model_get_path (model, iter); 
@@ -991,12 +1030,12 @@ contact_list_contact_active_cb (ShowActiveData *data)
 	if (data->remove && 
 	    !priv->show_offline && 
 	    !gossip_contact_is_online (data->contact)) {
-		DEBUG_MSG (("ContactList: Remove item (active timeout)!"));
+		gossip_debug (DEBUG_DOMAIN, "Remove item (active timeout)!");
 		contact_list_remove_contact (data->list,
 					     data->contact);
 	}
 
-	DEBUG_MSG (("ContactList: Setting contact to no longer be active"));
+	gossip_debug (DEBUG_DOMAIN, "Setting contact to no longer be active");
 	contact_list_contact_set_active (data->list, 
 					 data->contact,
 					 FALSE,
@@ -1060,18 +1099,18 @@ contact_list_get_group (GossipContactList *list,
 			gboolean          *created) 
 {
 	GtkTreeModel *model;
-	FindGroup    *fg;
+	FindGroup     fg;
 
-	fg = g_new0 (FindGroup, 1);
+	memset (&fg, 0, sizeof (fg));
 
-	fg->name = g_strdup (name);
+	fg.name = name;
 
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (list));
 	gtk_tree_model_foreach (model, 
 				(GtkTreeModelForeachFunc) contact_list_get_group_foreach, 
-				fg);
+				&fg);
 
-	if (!fg->found) {
+	if (!fg.found) {
 		if (created) {
 			*created = TRUE;
 		}
@@ -1089,11 +1128,8 @@ contact_list_get_group (GossipContactList *list,
 			*created = FALSE;
 		}
 
-		*iter_to_set = fg->iter;
+		*iter_to_set = fg.iter;
 	}
-
-	g_free (fg->name);
-	g_free (fg);
 }
 
 static gboolean
@@ -1144,13 +1180,14 @@ contact_list_add_contact (GossipContactList *list,
 		GdkPixbuf *pixbuf_avatar;
 
 		pixbuf_status = gossip_pixbuf_for_contact (contact);
-		pixbuf_avatar = gossip_pixbuf_avatar_from_contact_scaled (contact,
-									  GTK_ICON_SIZE_DND);
+		pixbuf_avatar = gossip_pixbuf_avatar_from_contact_scaled (
+			contact, 32, 32);
 		
 		gtk_tree_store_append (GTK_TREE_STORE (model), &iter, NULL);
 		gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
 				    COL_PIXBUF_STATUS, pixbuf_status,
 				    COL_PIXBUF_AVATAR, pixbuf_avatar,
+				    COL_PIXBUF_AVATAR_VISIBLE, priv->show_avatars,
 				    COL_NAME, gossip_contact_get_name (contact),
 				    COL_STATUS, gossip_contact_get_status (contact),
 				    COL_CONTACT, contact,
@@ -1181,8 +1218,8 @@ contact_list_add_contact (GossipContactList *list,
 		}
 
 		pixbuf_status = gossip_pixbuf_for_contact (contact);
-		pixbuf_avatar = gossip_pixbuf_avatar_from_contact_scaled (contact, 
-									  GTK_ICON_SIZE_DND);
+		pixbuf_avatar = gossip_pixbuf_avatar_from_contact_scaled (
+			contact, 32, 32);
 		
 		contact_list_get_group (list, name, &iter_group, &created);
 
@@ -1190,6 +1227,7 @@ contact_list_add_contact (GossipContactList *list,
 		gtk_tree_store_set (GTK_TREE_STORE (model), &iter,
 				    COL_PIXBUF_STATUS, pixbuf_status,
 				    COL_PIXBUF_AVATAR, pixbuf_avatar,
+				    COL_PIXBUF_AVATAR_VISIBLE, priv->show_avatars,
 				    COL_NAME, gossip_contact_get_name (contact),
 				    COL_STATUS, gossip_contact_get_status (contact),
 				    COL_CONTACT, contact,
@@ -1215,21 +1253,19 @@ contact_list_add_contact (GossipContactList *list,
 		}
 		
 		if (gossip_contact_group_get_expanded (name)) {
-			g_signal_handlers_block_by_func (GTK_TREE_VIEW (list), 
+			g_signal_handlers_block_by_func (list, 
 							 contact_list_row_expand_or_collapse_cb,
 							 GINT_TO_POINTER (TRUE));
-			gtk_tree_view_expand_row (GTK_TREE_VIEW (list),
-						  path, TRUE);
-			g_signal_handlers_unblock_by_func (GTK_TREE_VIEW (list), 
+			gtk_tree_view_expand_row (GTK_TREE_VIEW (list), path, TRUE);
+			g_signal_handlers_unblock_by_func (list, 
 							   contact_list_row_expand_or_collapse_cb,
 							   GINT_TO_POINTER (TRUE));
 		} else {
-			g_signal_handlers_block_by_func (GTK_TREE_VIEW (list), 
+			g_signal_handlers_block_by_func (list,
 							 contact_list_row_expand_or_collapse_cb,
 							 GINT_TO_POINTER (FALSE));
-			gtk_tree_view_collapse_row (GTK_TREE_VIEW (list),
-						    path);
-			g_signal_handlers_unblock_by_func (GTK_TREE_VIEW (list), 
+			gtk_tree_view_collapse_row (GTK_TREE_VIEW (list), path);
+			g_signal_handlers_unblock_by_func (list,
 							   contact_list_row_expand_or_collapse_cb,
 							   GINT_TO_POINTER (FALSE));
 		}
@@ -1277,16 +1313,18 @@ contact_list_create_model (GossipContactList *list)
 {
 	GtkTreeModel *model;
 
-	model = GTK_TREE_MODEL (gtk_tree_store_new (COL_COUNT,
-						    GDK_TYPE_PIXBUF,     /* Status pixbuf */
-						    GDK_TYPE_PIXBUF,     /* Avatar pixbuf */
-						    G_TYPE_STRING,       /* Name */
-						    G_TYPE_STRING,       /* Status string */
-						    GOSSIP_TYPE_CONTACT, /* Contact type */
-						    G_TYPE_BOOLEAN,      /* Is group */
-						    G_TYPE_BOOLEAN,      /* Is active */
-						    G_TYPE_BOOLEAN,      /* Is online */
-						    G_TYPE_BOOLEAN));    /* Is composing */
+	model = GTK_TREE_MODEL (
+		gtk_tree_store_new (COL_COUNT,
+				    GDK_TYPE_PIXBUF,     /* Status pixbuf */
+				    GDK_TYPE_PIXBUF,     /* Avatar pixbuf */
+				    G_TYPE_BOOLEAN,      /* Avatar pixbuf visible */
+				    G_TYPE_STRING,       /* Name */
+				    G_TYPE_STRING,       /* Status string */
+				    GOSSIP_TYPE_CONTACT, /* Contact type */
+				    G_TYPE_BOOLEAN,      /* Is group */
+				    G_TYPE_BOOLEAN,      /* Is active */
+				    G_TYPE_BOOLEAN,      /* Is online */
+				    G_TYPE_BOOLEAN));    /* Is composing */
 	
 	gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (model),
 					 COL_NAME,
@@ -1300,6 +1338,37 @@ contact_list_create_model (GossipContactList *list)
 	gtk_tree_view_set_model (GTK_TREE_VIEW (list), model);
 }
 
+static gboolean
+contact_list_search_equal_func (GtkTreeModel *model,
+				gint          column,
+				const gchar  *key,
+				GtkTreeIter  *iter,
+				gpointer      search_data)
+{
+	gchar    *name, *name_folded;
+	gchar    *key_folded;
+	gboolean  ret;
+
+	gtk_tree_model_get (model, iter,
+			    COL_NAME, &name,
+			    -1);
+
+	name_folded = g_utf8_casefold (name, -1);
+	key_folded = g_utf8_casefold (key, -1);
+	
+	if (strstr (name_folded, key_folded)) {
+		ret = FALSE;
+	} else {
+		ret = TRUE;
+	}
+
+	g_free (name);
+	g_free (name_folded);
+	g_free (key_folded);
+	
+	return ret;
+}
+	
 static void 
 contact_list_setup_view (GossipContactList *list)
 {
@@ -1307,6 +1376,11 @@ contact_list_setup_view (GossipContactList *list)
 	GtkTreeViewColumn *col;
 	gint               i;
 
+	gtk_tree_view_set_search_equal_func (GTK_TREE_VIEW (list),
+					     contact_list_search_equal_func,
+					     list,
+					     NULL);
+
 	g_object_set (list, 
 		      "headers-visible", FALSE,
 		      "reorderable", TRUE,
@@ -1322,10 +1396,11 @@ contact_list_setup_view (GossipContactList *list)
 		      NULL);
 	gtk_tree_view_column_pack_start (col, cell, FALSE);
 
-	gtk_tree_view_column_set_cell_data_func (col, cell, 
-						 (GtkTreeCellDataFunc)contact_list_pixbuf_cell_data_func,
-						 list, NULL);
-					    
+	gtk_tree_view_column_set_cell_data_func (
+		col, cell, 
+		(GtkTreeCellDataFunc) contact_list_pixbuf_cell_data_func,
+		list, NULL);
+	
 	cell = gossip_cell_renderer_text_new ();
 	g_object_set (cell,
 		      "xpad", (guint) 4, 
@@ -1339,21 +1414,25 @@ contact_list_setup_view (GossipContactList *list)
 	gtk_tree_view_column_add_attribute (col, cell,
 					    "is_group", COL_IS_GROUP);
 
-	gtk_tree_view_column_set_cell_data_func (col, cell, 
-						 (GtkTreeCellDataFunc)contact_list_text_cell_data_func,
-						 list, NULL);
+	gtk_tree_view_column_set_cell_data_func (
+		col, cell, 
+		(GtkTreeCellDataFunc) contact_list_text_cell_data_func,
+		list, NULL);
 
 	cell = gtk_cell_renderer_pixbuf_new ();
 	g_object_set (cell, 
 		      "xpad", (guint) 0,
-		      "ypad", (guint) 1,
+		      "ypad", (guint) 0,
 		      "visible", FALSE,
+		      "width", 32,
+		      "height", 32,
 		      NULL);
 	gtk_tree_view_column_pack_start (col, cell, FALSE);
 
-	gtk_tree_view_column_set_cell_data_func (col, cell, 
-						 (GtkTreeCellDataFunc)contact_list_avatar_cell_data_func,
-						 list, NULL);
+	gtk_tree_view_column_set_cell_data_func (
+		col, cell, 
+		(GtkTreeCellDataFunc) contact_list_avatar_cell_data_func,
+		list, NULL);
 
 	gtk_tree_view_append_column (GTK_TREE_VIEW (list), col);
 
@@ -1434,14 +1513,14 @@ contact_list_drag_data_received (GtkWidget         *widget,
 	gboolean                 drag_del = FALSE;
 
 	id = (const gchar*) selection->data;
-	DEBUG_MSG (("ContactList: Received %s%s drag & drop contact from roster with id:'%s'", 
-		   context->action == GDK_ACTION_MOVE ? "move" : "", 
-		   context->action == GDK_ACTION_COPY ? "copy" : "", 
-		   id));
+	gossip_debug (DEBUG_DOMAIN, "Received %s%s drag & drop contact from roster with id:'%s'",
+		      context->action == GDK_ACTION_MOVE ? "move" : "", 
+		      context->action == GDK_ACTION_COPY ? "copy" : "", 
+		      id);
 
 	contact = gossip_session_find_contact (gossip_app_get_session (), id);
 	if (!contact) {
-		DEBUG_MSG (("ContactList: No contact found associated with drag & drop"));
+		gossip_debug (DEBUG_DOMAIN, "No contact found associated with drag & drop");
 		return;
 	}
 
@@ -1467,9 +1546,9 @@ contact_list_drag_data_received (GtkWidget         *widget,
 
 		gossip_contact_set_groups (contact, NULL);
 	} else {
-		GList       *l, *new_groups;
-		gchar       *name;
-		gboolean     is_group;
+		GList    *l, *new_groups;
+		gchar    *name;
+		gboolean  is_group;
 
 		model = gtk_tree_view_get_model (GTK_TREE_VIEW (widget));
 		name = contact_list_get_parent_group (model, path, &is_group); 
@@ -1483,11 +1562,13 @@ contact_list_drag_data_received (GtkWidget         *widget,
 		/* Get source group information. */
 		priv = GET_PRIV (widget);
 		if (!priv->drag_row) {
+			g_free (name);
 			return;
 		}
 
 		path = gtk_tree_row_reference_get_path (priv->drag_row);
 		if (!path) {
+			g_free (name);
 			return;
 		}
 
@@ -1512,13 +1593,21 @@ contact_list_drag_data_received (GtkWidget         *widget,
 			    strcmp (str, old_group) == 0) {
 				continue;
 			}
+
+			if (str == NULL) {
+				continue;
+			}
 			
 			new_groups = g_list_append (new_groups, g_strdup (str));
 		}
-		
+
 		if (drag_success) {
-			new_groups = g_list_append (new_groups, name);
+			if (name) {
+				new_groups = g_list_append (new_groups, name);
+			}
 			gossip_contact_set_groups (contact, new_groups);
+		} else {
+			g_free (name);
 		}
 	}
 
@@ -1565,10 +1654,12 @@ contact_list_drag_motion (GtkWidget      *widget,
 		return TRUE;
 	}
 
-	g_source_remove_by_user_data (dm);
-	
 	if (dm) {
-		gtk_tree_path_free (dm->path); 
+		gtk_tree_path_free (dm->path);
+		if (dm->timeout_id) {
+			g_source_remove (dm->timeout_id);
+		}
+	
 		g_free (dm); 
 		
 		dm = NULL;
@@ -1580,9 +1671,10 @@ contact_list_drag_motion (GtkWidget      *widget,
 		dm->list = GOSSIP_CONTACT_LIST (widget);
 		dm->path = gtk_tree_path_copy (path);
 		
-		g_timeout_add (1500, 
-			       (GSourceFunc) contact_list_drag_motion_cb, 
-			       dm);
+		dm->timeout_id = g_timeout_add (
+			1500, 
+			(GSourceFunc) contact_list_drag_motion_cb, 
+			dm);
 	}
 
 	return TRUE;
@@ -1591,14 +1683,12 @@ contact_list_drag_motion (GtkWidget      *widget,
 static gboolean
 contact_list_drag_motion_cb (DragMotionData *data)
 {
-        g_return_val_if_fail (data != NULL, FALSE);
-        g_return_val_if_fail (data->list != NULL, FALSE);
-        g_return_val_if_fail (data->path != NULL, FALSE);
-
         gtk_tree_view_expand_row (GTK_TREE_VIEW (data->list), 
 				  data->path, 
 				  FALSE);
 
+	data->timeout_id = 0;
+	
         return FALSE;
 }
 
@@ -1768,17 +1858,19 @@ contact_list_avatar_cell_data_func (GtkTreeViewColumn *tree_column,
 				    GossipContactList *list)
 {
 	GdkPixbuf *pixbuf;
+	gboolean   show_avatar;
 	gboolean   is_group;
 	gboolean   is_active;
 
 	gtk_tree_model_get (model, iter, 
 			    COL_PIXBUF_AVATAR, &pixbuf,
+			    COL_PIXBUF_AVATAR_VISIBLE, &show_avatar,
 			    COL_IS_GROUP, &is_group, 
 			    COL_IS_ACTIVE, &is_active, 
 			    -1);
 
 	g_object_set (cell, 
-		      "visible", !is_group,
+		      "visible", !is_group && show_avatar,
 		      "pixbuf", pixbuf,
 		      NULL); 
 
@@ -1810,7 +1902,7 @@ contact_list_text_cell_data_func (GtkTreeViewColumn *tree_column,
 static gboolean 
 contact_list_button_press_event_cb (GossipContactList *list,
 				    GdkEventButton    *event,
-				    gpointer           unused)
+				    gpointer           user_data)
 {
 	GossipContactListPriv *priv;
 
@@ -1908,28 +2000,33 @@ static void
 contact_list_row_activated_cb (GossipContactList *list,
 			       GtkTreePath       *path,
 			       GtkTreeViewColumn *col,
-			       gpointer           unused)
+			       gpointer           user_data)
 {
 	GossipContactListPriv *priv;
-	GtkTreeModel          *model;
 	GossipContact         *contact;
+	GtkTreeModel          *model;
 	GtkTreeIter            iter;
+	FlashData             *data;
+	gint                   event_id = 0;
 
 	priv = GET_PRIV (list);
 
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (list));
 
 	gtk_tree_model_get_iter (model, &iter, path);
-	gtk_tree_model_get (model, &iter,
-			    COL_CONTACT, &contact,
-			    -1);
+	gtk_tree_model_get (model, &iter, COL_CONTACT, &contact, -1);
 
 	if (!contact) { 
 		/* This is a group. */
 		return;
 	}
 
-	g_signal_emit (list, signals[CONTACT_ACTIVATED], 0, contact);
+	data = g_hash_table_lookup (priv->flash_table, contact);
+	if (data) {
+		event_id = data->event_id;
+	}
+
+	g_signal_emit (list, signals[CONTACT_ACTIVATED], 0, contact, event_id);
 	g_object_unref (contact);
 }
 
@@ -1939,9 +2036,9 @@ contact_list_row_expand_or_collapse_cb (GossipContactList *list,
 					GtkTreePath       *path,
 					gpointer           user_data)
 {
-	GtkTreeModel          *model;
-	gchar                 *name;
-	gboolean               expanded;
+	GtkTreeModel *model;
+	gchar        *name;
+	gboolean      expanded;
 
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (list));
 
@@ -1959,7 +2056,7 @@ static gint
 contact_list_sort_func (GtkTreeModel *model,
 			GtkTreeIter  *iter_a,
 			GtkTreeIter  *iter_b,
-			gpointer      unused)
+			gpointer      user_data)
 {
 	gchar         *name_a, *name_b;
 	GossipContact *contact_a, *contact_b;
@@ -2025,26 +2122,22 @@ contact_list_find_contact (GossipContactList *list,
 			   GossipContact     *contact)
 {
 	GtkTreeModel *model;
-
-	FindContact  *fc;
+	FindContact   fc;
 	GList        *l = NULL;
 
-	fc = g_new0 (FindContact, 1);
+	memset (&fc, 0, sizeof (fc));
 	
-	fc->contact = g_object_ref (contact);
-		
+	fc.contact = contact;
+
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (list));
 	gtk_tree_model_foreach (model, 
 				(GtkTreeModelForeachFunc) contact_list_find_contact_foreach, 
-				fc);
-
-	if (fc->found) {
-		l = fc->iters;
+				&fc);
+	
+	if (fc.found) {
+		l = fc.iters;
 	}
 
-	g_object_unref (fc->contact);
-	g_free (fc);
-
 	return l;
 }
 
@@ -2339,19 +2432,14 @@ contact_list_group_menu_rename_cb (gpointer   data,
 	g_free (group);
 }
 
-typedef struct {
-	GossipContactList *list;
-	GossipContact     *contact;
-} FlashTimeoutData;
-
 static void
-contact_list_free_flash_timeout_data (FlashTimeoutData *t_data)
+contact_list_free_flash_timeout_data (FlashTimeoutData *timeout_data)
 {
-	g_return_if_fail (t_data != NULL);
+	g_return_if_fail (timeout_data != NULL);
 
-	g_object_unref (t_data->contact);
+	g_object_unref (timeout_data->contact);
 
-	g_free (t_data);
+	g_free (timeout_data);
 }
 
 static void
@@ -2367,61 +2455,72 @@ contact_list_flash_free_data (FlashData *data)
 }
 
 static gboolean
-contact_list_flash_timeout_func (FlashTimeoutData *t_data)
+contact_list_flash_timeout_func (FlashTimeoutData *timeout_data)
 {
 	GossipContactList     *list;
 	GossipContactListPriv *priv;
-	FlashData             *data;
-	gboolean               ret_val;
 	GossipContact         *contact;
-	GdkPixbuf             *pixbuf = NULL;
-	GtkTreeModel          *model;
 	GList                 *l, *iters;
+	GtkTreeModel          *model;
+	GdkPixbuf             *pixbuf = NULL;
+	FlashData             *data;
+	gboolean               retval = FALSE;
+	const gchar           *stock_id = NULL;
 	
-	list = t_data->list;
+	list = timeout_data->list;
 	priv = GET_PRIV (list);
 	
-	contact = t_data->contact;
+	contact = timeout_data->contact;
 	
 	iters = contact_list_find_contact (list, contact);
 	if (!iters) {
 		return FALSE;
 	}
 
-	pixbuf = gossip_pixbuf_for_contact (contact);
-	
 	data = g_hash_table_lookup (priv->flash_table, contact);
-	if (!data) {
-		ret_val = FALSE;
-	} else {
+	if (data) {
 		data->flash_on = !data->flash_on;
+		retval = TRUE;
 
 		if (data->flash_on) {
-			pixbuf = gossip_pixbuf_from_stock (GOSSIP_STOCK_MESSAGE,
-							   GTK_ICON_SIZE_MENU);
-		} 
+			switch (data->event_type) {
+			case GOSSIP_EVENT_NEW_MESSAGE: 
+			case GOSSIP_EVENT_SERVER_MESSAGE: 
+				stock_id = GOSSIP_STOCK_MESSAGE;
+				break;
+			case GOSSIP_EVENT_SUBSCRIPTION_REQUEST: 
+				stock_id = GTK_STOCK_DIALOG_QUESTION;
+				break;
+			default:
+				/* Shouldn't happen */
+				stock_id = GTK_STOCK_DIALOG_WARNING;
+				break;
+			}
+		}
+	}
 
-		ret_val = TRUE;
+	if (stock_id) {
+		pixbuf = gossip_pixbuf_from_stock (stock_id, GTK_ICON_SIZE_MENU);
+	} else {
+		pixbuf = gossip_pixbuf_for_contact (contact);
 	}
 
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (list));
 
 	for (l = iters; l; l = l->next) {
-		gtk_tree_store_set (GTK_TREE_STORE (model),
-				    l->data, 
+		gtk_tree_store_set (GTK_TREE_STORE (model), l->data, 
 				    COL_PIXBUF_STATUS, pixbuf,
 				    -1);
 	}
 
 	g_object_unref (pixbuf);
 
-	g_list_foreach (iters, (GFunc)gtk_tree_iter_free, NULL);
+	g_list_foreach (iters, (GFunc) gtk_tree_iter_free, NULL);
 	g_list_free (iters);
 
-	return ret_val;
+	return retval;
 }
 
-
 static void
 contact_list_event_added_cb (GossipEventManager *manager,
 			     GossipEvent        *event,
@@ -2430,36 +2529,52 @@ contact_list_event_added_cb (GossipEventManager *manager,
 	GossipContactListPriv *priv;
 	GossipMessage         *message;
 	GossipContact         *contact;
+	GossipEventType        type;
 	FlashData             *data;
-	FlashTimeoutData      *t_data;
+	FlashTimeoutData      *timeout_data;
 	
 	priv = GET_PRIV (list);
-	
-	if (gossip_event_get_type (event) != GOSSIP_EVENT_NEW_MESSAGE) {
+
+	type = gossip_event_get_type (event);
+
+	switch (type) {
+	case GOSSIP_EVENT_NEW_MESSAGE:
+		message = GOSSIP_MESSAGE (gossip_event_get_data (event));
+		contact = gossip_message_get_sender (message);
+		break;
+
+	case GOSSIP_EVENT_SUBSCRIPTION_REQUEST:
+		contact = GOSSIP_CONTACT (gossip_event_get_data (event));
+		break;
+
+	default:
+		/* Not handled */
 		return;
 	}
 
-	message = GOSSIP_MESSAGE (gossip_event_get_data (event));
-	contact = gossip_message_get_sender (message);
 	data = g_hash_table_lookup (priv->flash_table, contact);
-
 	if (data) {
 		/* Already flashing this item. */
 		return;
 	}
 
-	t_data = g_new0 (FlashTimeoutData, 1);
-	t_data->list    = list;
-	t_data->contact = g_object_ref (contact);
+	timeout_data = g_new0 (FlashTimeoutData, 1);
+
+	timeout_data->list = list;
+	timeout_data->contact = g_object_ref (contact);
 
 	data = g_new0 (FlashData, 1);
+
+	data->event_id = gossip_event_get_id (event);
+	data->event_type = type;
+
 	data->flash_on = TRUE;
 	data->flash_timeout_id = 
 		g_timeout_add_full (G_PRIORITY_DEFAULT, FLASH_TIMEOUT,
 				    (GSourceFunc) contact_list_flash_timeout_func,
-				    t_data, 
+				    timeout_data, 
 				    (GDestroyNotify) contact_list_free_flash_timeout_data);
-	
+
 	g_hash_table_insert (priv->flash_table, g_object_ref (contact), data);
 }
 
@@ -2469,25 +2584,36 @@ contact_list_event_removed_cb (GossipEventManager *manager,
 			       GossipContactList  *list)
 {
 	GossipContactListPriv *priv;
-	FlashData             *data;
 	GossipMessage         *message;
 	GossipContact         *contact;
+	GossipEventType        type;
 	const GdkPixbuf       *pixbuf;
  	GdkPixbuf             *pixbuf_composing;
 	GdkPixbuf             *pixbuf_presence;
 	GtkTreeModel          *model;
 	GList                 *iters, *l;
+	FlashData             *data;
 	gboolean               is_composing;
 
-	if (gossip_event_get_type (event) != GOSSIP_EVENT_NEW_MESSAGE) {
+	priv = GET_PRIV (list);
+
+	type = gossip_event_get_type (event);
+
+	switch (type) {
+	case GOSSIP_EVENT_NEW_MESSAGE:
+		message = GOSSIP_MESSAGE (gossip_event_get_data (event));
+		contact = gossip_message_get_sender (message);
+		break;
+
+	case GOSSIP_EVENT_SUBSCRIPTION_REQUEST:
+		contact = GOSSIP_CONTACT (gossip_event_get_data (event));
+		break;
+
+	default:
+		/* Not handled */
 		return;
 	}
 	
-	priv = GET_PRIV (list);
-
-	message = GOSSIP_MESSAGE (gossip_event_get_data (event));
-	contact = gossip_message_get_sender (message);
-
 	data = g_hash_table_lookup (priv->flash_table, contact);
 	if (!data) {
 		/* Not flashing this contact. */
@@ -2527,10 +2653,25 @@ contact_list_event_removed_cb (GossipEventManager *manager,
 	g_object_unref (pixbuf_presence); 
 	g_object_unref (pixbuf_composing); 
  	
-	g_list_foreach (iters, (GFunc)gtk_tree_iter_free, NULL);
+	g_list_foreach (iters, (GFunc) gtk_tree_iter_free, NULL);
 	g_list_free (iters);
 }
 
+static gboolean    
+contact_list_set_show_avatars_foreach (GtkTreeModel *model,
+				       GtkTreePath  *path,
+				       GtkTreeIter  *iter,
+				       gpointer      show_avatars)
+{
+	gtk_tree_store_set (GTK_TREE_STORE (model), iter,
+			    COL_PIXBUF_AVATAR_VISIBLE, GPOINTER_TO_INT (show_avatars), 
+			    -1);
+
+	gtk_tree_model_row_changed (model, path, iter);  
+
+	return FALSE;
+}
+
 GossipContactList *
 gossip_contact_list_new (void)
 {
@@ -2555,9 +2696,7 @@ gossip_contact_list_get_selected (GossipContactList *list)
                 return NULL;
         }
 
-        gtk_tree_model_get (model, &iter,
-                            COL_CONTACT, &contact,
-			    -1);
+        gtk_tree_model_get (model, &iter, COL_CONTACT, &contact, -1);
 
 	return contact;
 }
@@ -2596,9 +2735,25 @@ gossip_contact_list_get_selected_group (GossipContactList *list)
 gboolean
 gossip_contact_list_get_show_offline (GossipContactList *list)
 {
+	GossipContactListPriv *priv;
+
 	g_return_val_if_fail (GOSSIP_IS_CONTACT_LIST (list), FALSE);
 
-	return list->priv->show_offline;
+	priv = GET_PRIV (list);
+
+	return priv->show_offline;
+}
+
+gboolean
+gossip_contact_list_get_show_avatars (GossipContactList *list)
+{
+	GossipContactListPriv *priv;
+
+	g_return_val_if_fail (GOSSIP_IS_CONTACT_LIST (list), TRUE);
+
+	priv = GET_PRIV (list);
+
+	return priv->show_avatars;
 }
 
 void         
@@ -2635,3 +2790,24 @@ gossip_contact_list_set_show_offline (GossipContactList *list,
 	/* Restore to original setting. */
 	priv->show_active = show_active;
 }
+
+void         
+gossip_contact_list_set_show_avatars (GossipContactList *list,
+				      gboolean           show_avatars)
+{
+	GossipContactListPriv *priv;
+	GtkTreeModel          *model;
+		
+	g_return_if_fail (GOSSIP_IS_CONTACT_LIST (list));
+
+	priv = GET_PRIV (list);
+
+	priv->show_avatars = show_avatars;
+
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (list));
+
+	gtk_tree_model_foreach (model, 
+				(GtkTreeModelForeachFunc) 
+				contact_list_set_show_avatars_foreach, 
+				GINT_TO_POINTER (show_avatars));
+}
