@@ -1,6 +1,6 @@
 /* -*- mode: C; c-file-style: "gnu" -*- */
 /*
- * Copyright (C) 2004 Imendio HB
+ * Copyright (C) 2004 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -23,14 +23,19 @@
 
 #include <dbus/dbus-glib.h>
 
-#define GOSSIP_SERVICE "org.imendio.gossip"
-#define GOSSIP_INTERFACE "org.imendio.Gossip"
-#define GOSSIP_OBJECT "/org/imendio/Gossip"
+#include <libgossip/gossip-session.h>
 
-#define GOSSIP_SET_PRESENCE "set_Presence"
-#define GOSSIP_FORCE_NON_AWAY "ForceNonAway"
+#define GOSSIP_DBUS_ERROR gnome_dbus_error_quark()
 
-gboolean gossip_dbus_init (void);
+GQuark   gossip_dbus_error_quark          (void) G_GNUC_CONST;
 
+/* Session */
+gboolean gossip_dbus_init_for_session     (GossipSession *session,
+					   gboolean       multiple_instances);
+void     gossip_dbus_finalize_for_session (void);
 
-#endif /* __DBUS_H__ */
+/* GNOME Network Manager */
+gboolean gossip_dbus_nm_setup             (void);
+gboolean gossip_dbus_nm_get_state         (gboolean      *connected);
+
+#endif /* __GOSSIP_DBUS_H__ */
