@@ -154,7 +154,7 @@ ft_window_request_cb (GossipProtocol *protocol,
 
 	gossip_event_manager_add (gossip_app_get_event_manager (),
 				  event, 
-				  (GossipEventActivatedFunction)ft_window_event_activated_cb,
+				  (GossipEventActivateFunction) ft_window_event_activated_cb,
 				  G_OBJECT (protocol));
 }
 
@@ -295,8 +295,10 @@ ft_window_vcard_cb (GossipResult  result,
 		start = g_array_new (FALSE, FALSE, sizeof (gint));
 		end = g_array_new (FALSE, FALSE, sizeof (gint));
 		
-		num_matches = gossip_utils_regex_match (GOSSIP_REGEX_ALL, 
-							url, start, end);
+		num_matches = gossip_regex_match (GOSSIP_REGEX_ALL, url, start, end);
+
+		g_array_free (start, TRUE);
+		g_array_free (end, TRUE);
 	}
 
 	if (num_matches > 0) {
