@@ -27,6 +27,7 @@
 #include <glib/gi18n.h>
 
 #include <libgossip/gossip-chatroom-provider.h>
+#include <libgossip/gossip-debug.h>
 #include <libgossip/gossip-message.h>
 
 #include "gossip-app.h"
@@ -44,11 +45,10 @@
 
 #define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_GROUP_CHAT, GossipGroupChatPriv))
 
-#define DEBUG_MSG(x)
-/* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n"); */
-
 #define IS_ENTER(v) (v == GDK_Return || v == GDK_ISO_Enter || v == GDK_KP_Enter)
 
+#define DEBUG_DOMAIN "GroupChat"
+
 struct _GossipGroupChatPriv {
 	GossipContact          *own_contact;
 
@@ -69,6 +69,8 @@ struct _GossipGroupChatPriv {
 	GHashTable             *contacts;
 	GList                  *private_chats;
 
+	guint                   scroll_idle_id;
+
 	gint                    contacts_width;
 	gboolean                contacts_visible;
 };
@@ -292,6 +294,10 @@ group_chat_finalize (GObject *object)
 		g_object_unref (priv->own_contact);
 	}
 
+	if (priv->scroll_idle_id) {
+		g_source_remove (priv->scroll_idle_id);
+	}
+	
  	G_OBJECT_CLASS (gossip_group_chat_parent_class)->finalize (object);
 }
 
@@ -477,12 +483,12 @@ group_chat_drag_data_received (GtkWidget        *widget,
 	priv = GET_PRIV (chat);
 
 	id = (const gchar*) selection->data;
-	g_print ("Received drag & drop contact from roster with id:'%s'\n", id);
+	/*g_print ("Received drag & drop contact from roster with id:'%s'\n", id);*/
 
 	contact = gossip_session_find_contact (gossip_app_get_session (), id);
 	
 	if (!contact) {
-		g_print ("No contact found associated with drag & drop\n");
+		/*g_print ("No contact found associated with drag & drop\n");*/
 		return;
 	}
 
@@ -809,7 +815,7 @@ group_chat_joined_cb (GossipChatroomProvider *provider,
 		return;
 	}
 
-	DEBUG_MSG (("GroupChat: [%d] Joined", id));
+	gossip_debug (DEBUG_DOMAIN, "[%d] Joined", id);
 
 	chatview = GOSSIP_CHAT (chat)->view;
 
@@ -842,7 +848,7 @@ group_chat_new_message_cb (GossipChatroomProvider *provider,
 		return;
 	}
 
-	DEBUG_MSG (("GroupChat: [%d] New message", id));
+	gossip_debug (DEBUG_DOMAIN, "[%d] New message", id);
 
 	invite = gossip_message_get_invite (message);
 	if (invite) {
@@ -856,12 +862,14 @@ group_chat_new_message_cb (GossipChatroomProvider *provider,
 			gossip_chat_view_append_message_from_self (
 				GOSSIP_CHAT (chat)->view,
 				message,
-				priv->own_contact);
+				priv->own_contact,
+				NULL);
 		} else {
 			gossip_chat_view_append_message_from_other (
 				GOSSIP_CHAT (chat)->view,
 				message,
-				priv->own_contact);
+				priv->own_contact,
+				NULL);
 		}
 	}
 	
@@ -891,7 +899,7 @@ group_chat_new_event_cb (GossipChatroomProvider *provider,
 		return;
 	}
 
-	DEBUG_MSG (("GroupChat: [%d] New event:'%s'", id, event));
+	gossip_debug (DEBUG_DOMAIN, "[%d] New event:'%s'", id, event);
 
 	gossip_chat_view_append_event (GOSSIP_CHAT (chat)->view, event);
 }
@@ -913,8 +921,8 @@ group_chat_topic_changed_cb (GossipChatroomProvider *provider,
 		return;
 	}
 
-	DEBUG_MSG (("GroupChat: [%d] Topic changed by:'%s' to:'%s'", 
-		    id, gossip_contact_get_id (who), new_topic));
+	gossip_debug (DEBUG_DOMAIN, "[%d] Topic changed by:'%s' to:'%s'", 
+		      id, gossip_contact_get_id (who), new_topic);
 	
 	gtk_entry_set_text (GTK_ENTRY (priv->topic_entry), new_topic);
 	
@@ -944,8 +952,8 @@ group_chat_contact_joined_cb (GossipChatroomProvider *provider,
 		return;
 	}
 
-	DEBUG_MSG (("GroupChat: [%d] Contact joined:'%s'", 
-		    id, gossip_contact_get_id (contact)));
+	gossip_debug (DEBUG_DOMAIN, "[%d] Contact joined:'%s'", 
+		      id, gossip_contact_get_id (contact));
 
 	pixbuf = gossip_pixbuf_for_contact (contact);
 	
@@ -978,8 +986,8 @@ group_chat_contact_left_cb (GossipChatroomProvider *provider,
 		return;
 	}
 
-	DEBUG_MSG (("GroupChat: [%d] Contact left:'%s'", 
-		    id, gossip_contact_get_id (contact)));
+	gossip_debug (DEBUG_DOMAIN, "[%d] Contact left:'%s'", 
+		      id, gossip_contact_get_id (contact));
 	
 	if (group_chat_contacts_find (chat, contact, &iter)) {
 		GtkTreeModel *model;
@@ -1005,6 +1013,9 @@ group_chat_contact_presence_updated_cb (GossipChatroomProvider *provider,
 		return;
 	}
 
+	gossip_debug (DEBUG_DOMAIN, "[%d] Contact Presence Updated:'%s'", 
+		      id, gossip_contact_get_id (contact));
+
 	if (group_chat_contacts_find (chat, contact, &iter)) {
 		GdkPixbuf    *pixbuf;
 		GtkTreeModel *model;
@@ -1172,7 +1183,7 @@ group_chat_send (GossipGroupChat *chat)
 	if (!handled_command) {
 		gossip_chatroom_provider_send (priv->chatroom_provider, priv->chatroom_id, msg);
 		
-		gossip_app_force_non_away ();
+		gossip_app_set_not_away ();
 	}
 
 	g_free (msg);
@@ -1272,7 +1283,9 @@ group_chat_get_status_pixbuf (GossipChat *chat)
  	priv = GET_PRIV (group_chat); 
 
  	chatroom = gossip_chatroom_provider_find (priv->chatroom_provider, priv->chatroom_id);
-	pixbuf = gossip_pixbuf_for_chatroom_status (chatroom, GTK_ICON_SIZE_MENU);
+	if (chatroom) {
+		pixbuf = gossip_pixbuf_for_chatroom_status (chatroom, GTK_ICON_SIZE_MENU);
+	}
 
 	if (!pixbuf) {
 		pixbuf = gossip_pixbuf_from_stock (GOSSIP_STOCK_GROUP_MESSAGE,
@@ -1359,22 +1372,51 @@ group_chat_is_group_chat (GossipChat *chat)
 
 /* Scroll down after the back-log has been received. */
 static gboolean
-group_chat_scroll_down_func (GossipChat *chat)
+group_chat_scroll_down_idle_func (GossipChat *chat)
 {
+	GossipGroupChatPriv *priv;
+
+	priv = GET_PRIV (chat);
+	
 	gossip_chat_scroll_down (chat);
  	g_object_unref (chat);
 
+	priv->scroll_idle_id = 0;
+
 	return FALSE;
 }
 
+/* Copied from the jabber backend for now since we don't have an abstraction for
+ * "contacts" for group chats. Casefolds the node part (the part before @).
+ */
+static gchar *
+jid_casefold_node (const gchar *str)
+{
+	gchar       *tmp;
+	gchar       *ret;
+	const gchar *at;
+
+	at = strchr (str, '@');
+	if (!at) {
+		return g_strdup (str);
+	}
+
+	tmp = g_utf8_casefold (str, at - str);
+	ret = g_strconcat (tmp, at, NULL);
+	g_free (tmp);
+	
+	return ret;
+}
+
 GossipGroupChat *
 gossip_group_chat_new (GossipChatroomProvider *provider,
 		       GossipChatroomId        id)
 {
 	GossipGroupChat     *chat;
 	GossipGroupChatPriv *priv;
-	GossipContact       *own_contact;
+	gchar               *casefolded_id;
  	gchar               *own_contact_id;
+	GossipContact       *own_contact;
 	GossipChatroom      *chatroom;
 
 	g_return_val_if_fail (GOSSIP_IS_CHATROOM_PROVIDER (provider), NULL);
@@ -1392,10 +1434,14 @@ gossip_group_chat_new (GossipChatroomProvider *provider,
 	chatroom = gossip_chatroom_provider_find (provider, id);
 	g_return_val_if_fail (GOSSIP_IS_CHATROOM (chatroom), NULL);
 
-	own_contact_id = g_strdup_printf ("%s/%s", 
-					  gossip_chatroom_get_id_str (chatroom),
-					  gossip_chatroom_get_nick (chatroom));
+	casefolded_id = jid_casefold_node (gossip_chatroom_get_id_str (chatroom));
 	
+	own_contact_id = g_strdup_printf ("%s/%s", 
+					  casefolded_id,
+					  gossip_chatroom_get_nick (chatroom));
+
+	g_free (casefolded_id);
+
 	own_contact = gossip_contact_new_full (GOSSIP_CONTACT_TYPE_TEMPORARY, 
 					       gossip_chatroom_get_account (chatroom),
 					       own_contact_id,
@@ -1458,7 +1504,8 @@ gossip_group_chat_new (GossipChatroomProvider *provider,
 	gossip_chat_present (GOSSIP_CHAT (chat));
 
  	g_object_ref (chat);
-	g_idle_add ((GSourceFunc) group_chat_scroll_down_func, chat);
+	priv->scroll_idle_id = g_idle_add (
+		(GSourceFunc) group_chat_scroll_down_idle_func, chat);
 	
 	return chat;
 }
