@@ -61,9 +61,6 @@ typedef struct {
 	GossipChatView *chatview_chatrooms;
 
 	gchar          *last_search;
-
-
-	GtkWidget      *button_close;
 } GossipLogWindow;
 
 /* Searching */
@@ -156,8 +153,6 @@ static void            log_window_entry_chatrooms_changed_cb          (GtkWidget
 								       GossipLogWindow  *window);
 static void            log_window_entry_chatrooms_activate_cb         (GtkWidget        *entry,
 								       GossipLogWindow  *window);
-static void            log_window_button_close_clicked_cb             (GtkWidget        *widget,
-								       GossipLogWindow  *window);
 static void            log_window_destroy_cb                          (GtkWidget        *widget,
 								       GossipLogWindow  *window);
 
@@ -301,11 +296,13 @@ log_window_search_changed_cb (GtkTreeSelection *selection,
 		if (gossip_contact_equal (own_contact, sender)) {
 			gossip_chat_view_append_message_from_self (window->chatview_search, 
 								   message,
-								   own_contact);
+								   own_contact,
+								   NULL);
 		} else {
 			gossip_chat_view_append_message_from_other (window->chatview_search, 
 								    message,
-								    sender);
+								    sender,
+								    NULL);
 		}
 	}
 
@@ -759,11 +756,13 @@ log_window_contacts_new_message_cb (GossipContact   *own_contact,
 	if (gossip_contact_equal (own_contact, sender)) {
 		gossip_chat_view_append_message_from_self (window->chatview_contacts, 
 							   message,
-							   own_contact);
+							   own_contact,
+							   NULL);
 	} else {
 		gossip_chat_view_append_message_from_other (window->chatview_contacts, 
 							    message,
-							    sender);
+							    sender,
+							    NULL);
 	}
 
 	/* Scroll to the most recent messages */
@@ -948,11 +947,13 @@ log_window_contacts_get_messages (GossipLogWindow *window,
 		if (gossip_contact_equal (own_contact, sender)) {
 			gossip_chat_view_append_message_from_self (window->chatview_contacts, 
 								   message,
-								   own_contact);
+								   own_contact,
+								   NULL);
 		} else {
 			gossip_chat_view_append_message_from_other (window->chatview_contacts, 
 								    message,
-								    sender);
+								    sender,
+								    NULL);
 		}
 	}
 
@@ -1343,11 +1344,13 @@ log_window_chatrooms_new_message_cb (GossipContact   *own_contact,
 	if (gossip_contact_equal (own_contact, sender)) {
 		gossip_chat_view_append_message_from_self (window->chatview_chatrooms, 
 							   message,
-							   own_contact);
+							   own_contact,
+							   NULL);
 	} else {
 		gossip_chat_view_append_message_from_other (window->chatview_chatrooms, 
 							    message,
-							    sender);
+							    sender,
+							    NULL);
 	}
 
 	/* Scroll to the most recent messages */
@@ -1534,11 +1537,13 @@ log_window_chatrooms_get_messages (GossipLogWindow *window,
 		if (gossip_contact_equal (own_contact, sender)) {
 			gossip_chat_view_append_message_from_self (window->chatview_chatrooms, 
 								   message,
-								   own_contact);
+								   own_contact,
+								   NULL);
 		} else {
 			gossip_chat_view_append_message_from_other (window->chatview_chatrooms, 
 								    message,
-								    sender);
+								    sender,
+								    NULL);
 		}
 	}
 
@@ -1673,13 +1678,6 @@ log_window_entry_chatrooms_activate_cb (GtkWidget       *entry,
 /*
  * Other window callbacks 
  */
-static void
-log_window_button_close_clicked_cb (GtkWidget       *widget,
-				    GossipLogWindow *window)
-{
-	gtk_widget_destroy (window->window);
-}
-
 static void
 log_window_destroy_cb (GtkWidget       *widget, 
 		       GossipLogWindow *window)
@@ -1740,7 +1738,6 @@ gossip_log_window_show (GossipContact  *contact,
 				       "vbox_chatrooms", &window->vbox_chatrooms,
  				       "treeview_chatrooms", &window->treeview_chatrooms, 
 				       "scrolledwindow_chatrooms", &window->scrolledwindow_chatrooms,
-				       "button_close", &window->button_close,
 				       NULL);
 	
 	gossip_glade_connect (glade, 
@@ -1752,9 +1749,10 @@ gossip_log_window_show (GossipContact  *contact,
 			      "entry_contacts", "activate", log_window_entry_contacts_activate_cb,
 			      "entry_chatrooms", "changed", log_window_entry_chatrooms_changed_cb,
 			      "entry_chatrooms", "activate", log_window_entry_chatrooms_activate_cb,
-			      "button_close", "clicked", log_window_button_close_clicked_cb,
 			      NULL);
 
+	g_object_unref (glade);
+
 	g_object_add_weak_pointer (G_OBJECT (window->window),
 				   (gpointer) &window);
 
