@@ -72,11 +72,10 @@ static gboolean            notify_event_remove_foreach        (gpointer
 static void                notify_event_removed_cb            (GossipEventManager *event_manager,
 							       GossipEvent        *event,
 							       gpointer            user_data);
-static void                notify_event_destroy_cb            (NotifyNotification *notify);
 
 enum {
 	NOTIFY_SHOW_MESSAGE,
-	NOTIFY_SHOW_ROSTER,
+	NOTIFY_SHOW_ROSTER
 };
 
 static GHashTable *account_states = NULL;
@@ -85,6 +84,7 @@ static GHashTable *message_notifications = NULL;
 static GHashTable *event_notifications = NULL;
 static GtkWidget  *attach_widget = NULL;
 
+
 static const gchar *
 notify_get_status_from_presence (GossipPresence *presence)
 {
@@ -118,6 +118,13 @@ notify_online_action_cb (NotifyNotification *notify,
 	g_object_unref (contact);
 }
 
+static void
+notify_closed_cb (NotifyNotification *notify,
+		  gpointer            user_data)
+{
+	g_object_unref (notify);
+}
+
 static gboolean
 notify_get_is_busy (void)
 {
@@ -171,6 +178,11 @@ notify_contact_online (GossipContact *contact)
 	notify_notification_set_urgency (notify, NOTIFY_URGENCY_LOW);
 	notify_notification_set_icon_from_pixbuf (notify, pixbuf);
 
+	g_signal_connect (notify,
+			  "closed",
+			  G_CALLBACK (notify_closed_cb),
+			  NULL);
+
 	if (attach_widget) {
 		notify_notification_attach_to_widget (notify, attach_widget);
 	}
@@ -187,7 +199,7 @@ notify_contact_online (GossipContact *contact)
 			   error->message);
 		g_error_free (error);
 	}
-	
+
 	g_free (title);
 
 	if (pixbuf) {
@@ -210,13 +222,13 @@ notify_new_message_default_cb (NotifyNotification *notify,
 		contact = gossip_message_get_sender (message);
 	
 		gossip_event_manager_activate (event_manager, event);
+
+		g_hash_table_remove (event_notifications, notify);
+		g_hash_table_remove (message_notifications, contact);
 	} else {
 		g_warning ("No event found for NotifyNotification: %p", notify);
 	}
 
-	g_hash_table_remove (event_notifications, notify);
-	g_hash_table_remove (message_notifications, contact);
-
 	g_object_unref (event_manager);
 }
 
@@ -235,13 +247,13 @@ notify_new_message_contact_cb (NotifyNotification *notify,
 		contact = gossip_message_get_sender (message);
 
 		gossip_contact_info_dialog_show (contact);
+
+		g_hash_table_remove (event_notifications, notify);
+		g_hash_table_remove (message_notifications, contact);
 	} else {
 		g_warning ("No event found for Notification: %p", notify);
 	}
 
-	g_hash_table_remove (event_notifications, notify);
-	g_hash_table_remove (message_notifications, contact);
-
 	g_object_unref (event_manager);
 }
 
@@ -296,6 +308,11 @@ notify_new_message (GossipEventManager *event_manager,
 	notify_notification_set_icon_from_pixbuf (notify, pixbuf);
 	notify_notification_set_timeout (notify, NOTIFY_MESSAGE_TIME);
 
+	g_signal_connect (notify,
+			  "closed",
+			  G_CALLBACK (notify_closed_cb),
+			  NULL);
+
 	if (attach_widget) {
 		notify_notification_attach_to_widget (notify, attach_widget);
 	}
@@ -303,12 +320,15 @@ notify_new_message (GossipEventManager *event_manager,
 	notify_notification_add_action (notify, "default", _("Default"),
 					(NotifyActionCallback) notify_new_message_default_cb,
 					g_object_ref (event_manager), NULL);
-	notify_notification_add_action (notify, "respond", _("Respond"),
+	notify_notification_add_action (notify, "respond", _("Show"),
 					(NotifyActionCallback) notify_new_message_default_cb,
 					g_object_ref (event_manager), NULL);
-	notify_notification_add_action (notify, "contact", _("Contact Information"),
-					(NotifyActionCallback) notify_new_message_contact_cb,
-					g_object_ref (event_manager), NULL);
+	if (gossip_contact_get_type (contact) == GOSSIP_CONTACT_TYPE_TEMPORARY) {
+		notify_notification_add_action (
+			notify, "contact", _("Contact Information"),
+			(NotifyActionCallback) notify_new_message_contact_cb,
+			g_object_ref (event_manager), NULL);
+	}
 	
 	if (!notify_notification_show (notify, &error)) {
 		g_warning ("Failed to send notification: %s",
@@ -416,7 +436,7 @@ notify_event_added_cb (GossipEventManager *event_manager,
 		/* Find out if there are any other messages waiting,
 		 * if not, show a notification.
 		 */
-		if (! g_hash_table_lookup (message_notifications, contact)) {
+		if (!g_hash_table_lookup (message_notifications, contact)) {
 			notify = notify_new_message (event_manager, message);
 			
 			if (notify) {
@@ -424,7 +444,7 @@ notify_event_added_cb (GossipEventManager *event_manager,
 						     contact,
 						     g_object_ref (event));
 				g_hash_table_insert (event_notifications,   
-						     notify,
+						     g_object_ref (notify),
 						     g_object_ref (event));  
 			}
 		}
@@ -432,14 +452,27 @@ notify_event_added_cb (GossipEventManager *event_manager,
 }
 
 static gboolean
-notify_event_remove_foreach (gpointer key,
+notify_message_remove_foreach (gpointer     key,
 			     GossipEvent *event,
 			     GossipEvent *event_to_compare)
 {
 	if (gossip_event_equal (event, event_to_compare)) {
 		return TRUE;
 	}
+	
+	return FALSE;
+}
 
+static gboolean
+notify_event_remove_foreach (gpointer     key,
+			     GossipEvent *event,
+			     GossipEvent *event_to_compare)
+{
+	if (gossip_event_equal (event, event_to_compare)) {
+		notify_notification_close (key, NULL);
+		return TRUE;
+	}
+	
 	return FALSE;
 }
 
@@ -449,19 +482,13 @@ notify_event_removed_cb (GossipEventManager *event_manager,
 			 gpointer            user_data)
 {
 	g_hash_table_foreach_remove (message_notifications, 
-				     (GHRFunc) notify_event_remove_foreach,
+				     (GHRFunc) notify_message_remove_foreach,
 				     event);
 	g_hash_table_foreach_remove (event_notifications, 
 				     (GHRFunc) notify_event_remove_foreach,
 				     event);
 }
 
-static void
-notify_event_destroy_cb (NotifyNotification *notify)
-{
-	notify_notification_close (notify, NULL);
-}
-
 void
 gossip_notify_set_attach_widget (GtkWidget *new_attach_widget)
 {
@@ -504,7 +531,7 @@ gossip_notify_init (GossipSession      *session,
 
 	event_notifications = g_hash_table_new_full (g_direct_hash,
 						     g_direct_equal,
-						     (GDestroyNotify) notify_event_destroy_cb,
+						     (GDestroyNotify) g_object_unref,
 						     (GDestroyNotify) g_object_unref);
 
 	account_states = g_hash_table_new_full (gossip_account_hash,
