@@ -30,22 +30,25 @@
 #include "gossip-stock.h"
 #include "gossip-ui-utils.h"
 #include "gossip-preferences.h"
+#include "gossip-theme-manager.h"
 #include "gossip-spell.h"
 
 typedef struct {
 	GtkWidget *dialog;
+
 	GtkWidget *notebook;
 
-	GtkWidget *sound_checkbutton;
-	GtkWidget *silent_busy_checkbutton;
-	GtkWidget *silent_away_checkbutton;
-        GtkWidget *separate_windows_checkbutton;
+	GtkWidget *checkbutton_show_avatars;
+	GtkWidget *checkbutton_show_smileys;
+	GtkWidget *combobox_chat_theme;
+        GtkWidget *checkbutton_separate_chat_windows;
 
-	GtkWidget *smileys_checkbutton;
-	GtkWidget *theme_combobox;
+	GtkWidget *checkbutton_sounds_for_messages;
+	GtkWidget *checkbutton_sounds_when_busy;
+	GtkWidget *checkbutton_sounds_when_away;
+	GtkWidget *checkbutton_popups_when_available;
 
-	GtkWidget *spell_checker_vbox;
-	GtkWidget *spell_checker_treeview;
+	GtkWidget *treeview_spell_checker;
 
 	GList     *gconf_ids;
 } GossipPreferences;
@@ -141,32 +144,40 @@ static void
 preferences_setup_widgets (GossipPreferences *preferences)
 {
 	preferences_hookup_toggle_button (preferences,
-					  GCONF_PATH "/sound/play_sounds",
-					  preferences->sound_checkbutton);
+					  GCONF_SOUNDS_FOR_MESSAGES,
+					  preferences->checkbutton_sounds_for_messages);
 	preferences_hookup_toggle_button (preferences,
-					  GCONF_PATH "/sound/silent_away",
-					  preferences->silent_away_checkbutton);
+					  GCONF_SOUNDS_WHEN_AWAY,
+					  preferences->checkbutton_sounds_when_away);
 	preferences_hookup_toggle_button (preferences,
-					  GCONF_PATH "/sound/silent_busy",
-					  preferences->silent_busy_checkbutton);
+					  GCONF_SOUNDS_WHEN_BUSY,
+					  preferences->checkbutton_sounds_when_busy);
+	preferences_hookup_toggle_button (preferences,
+					  GCONF_POPUPS_WHEN_AVAILABLE,
+					  preferences->checkbutton_popups_when_available);
+
 	preferences_hookup_sensitivity (preferences,
-					GCONF_PATH "/sound/play_sounds",
-					preferences->silent_away_checkbutton);
+					GCONF_SOUNDS_FOR_MESSAGES,
+					preferences->checkbutton_sounds_when_away);
 	preferences_hookup_sensitivity (preferences,
-					GCONF_PATH "/sound/play_sounds",
-					preferences->silent_busy_checkbutton);
+					GCONF_SOUNDS_FOR_MESSAGES,
+					preferences->checkbutton_sounds_when_busy);
 
         preferences_hookup_toggle_button (preferences,
-					  GCONF_PATH "/ui/separate_chat_windows",
-					  preferences->separate_windows_checkbutton);
+					  GCONF_UI_SEPARATE_CHAT_WINDOWS,
+					  preferences->checkbutton_separate_chat_windows);
+
+	preferences_hookup_toggle_button (preferences,
+					  GCONF_UI_SHOW_AVATARS,
+					  preferences->checkbutton_show_avatars);
 
         preferences_hookup_toggle_button (preferences,
-					  GCONF_PATH "/conversation/graphical_smileys",
-					  preferences->smileys_checkbutton);
+					  GCONF_CHAT_SHOW_SMILEYS,
+					  preferences->checkbutton_show_smileys);
 
 	preferences_hookup_string_combo (preferences,
-					 GCONF_PATH "/conversation/theme",
-					 preferences->theme_combobox);
+					 GCONF_CHAT_THEME,
+					 preferences->combobox_chat_theme);
 }
 
 static void 
@@ -180,7 +191,7 @@ preferences_languages_setup (GossipPreferences *preferences)
 	GtkCellRenderer   *renderer;
 	guint              col_offset;
 
-	view = GTK_TREE_VIEW (preferences->spell_checker_treeview);
+	view = GTK_TREE_VIEW (preferences->treeview_spell_checker);
 
 	store = gtk_list_store_new (COL_LANG_COUNT,
 				    G_TYPE_BOOLEAN,  /* enabled */
@@ -228,40 +239,31 @@ preferences_languages_add (GossipPreferences *preferences)
 {
 	GtkTreeView  *view;
 	GtkListStore *store;
-	GossipSpell  *spell;
 	GList        *codes, *l;
 
-	view = GTK_TREE_VIEW (preferences->spell_checker_treeview);
+	view = GTK_TREE_VIEW (preferences->treeview_spell_checker);
 	store = GTK_LIST_STORE (gtk_tree_view_get_model (view));
 
-	spell = gossip_spell_new (NULL);
-	codes = gossip_spell_get_language_codes (spell);
-
+	codes = gossip_spell_get_language_codes ();
 	for (l = codes; l; l = l->next) {
 		GtkTreeIter  iter;
 		const gchar *code;
-		const gchar *language;
+		const gchar *name;
 
 		code = l->data; 
-		language = gossip_spell_get_language_name (spell, code);
-		
-		if (!language || strlen (language) < 1) {
+		name = gossip_spell_get_language_name (code);
+		if (!name) {
 			continue;
 		}
 
 		gtk_list_store_append (store, &iter);
 		gtk_list_store_set (store, &iter, 
 				    COL_LANG_CODE, code,
-				    COL_LANG_NAME, language,
+				    COL_LANG_NAME, name,
 				    -1);
 	}
 
-	g_list_foreach (codes, (GFunc)g_free, NULL);
-	g_list_free (codes);
-
-	if (spell) {
-		gossip_spell_unref (spell);
-	}
+	gossip_spell_free_language_codes (codes);
 }
 
 static void
@@ -272,7 +274,7 @@ preferences_languages_save (GossipPreferences *preferences)
 
 	gchar             *languages = NULL;
 
-	view = GTK_TREE_VIEW (preferences->spell_checker_treeview);
+	view = GTK_TREE_VIEW (preferences->treeview_spell_checker);
 	model = gtk_tree_view_get_model (view);
 	
 	gtk_tree_model_foreach (model,
@@ -285,7 +287,7 @@ preferences_languages_save (GossipPreferences *preferences)
 	}
 	
 	gconf_client_set_string (gossip_app_get_gconf_client (),
-				 GCONF_PATH "/conversation/spell_checker_languages", 
+				 GCONF_CHAT_SPELL_CHECKER_LANGUAGES,
 				 languages,
 				 NULL);
 
@@ -339,7 +341,7 @@ preferences_languages_load (GossipPreferences *preferences)
 
 	value = gconf_client_get_string (
 		gossip_app_get_gconf_client (),
-		GCONF_PATH "/conversation/spell_checker_languages", 
+		GCONF_CHAT_SPELL_CHECKER_LANGUAGES,
 		NULL);
 	
 	if (!value) {
@@ -349,7 +351,7 @@ preferences_languages_load (GossipPreferences *preferences)
 	vlanguages = g_strsplit (value, ",", -1);
 	g_free (value);
 
-	view = GTK_TREE_VIEW (preferences->spell_checker_treeview);
+	view = GTK_TREE_VIEW (preferences->treeview_spell_checker);
 	model = gtk_tree_view_get_model (view);
 
 	gtk_tree_model_foreach (model,
@@ -401,7 +403,7 @@ preferences_languages_cell_toggled_cb (GtkCellRendererToggle *cell,
 	GtkTreeIter   iter;
 	gboolean      enabled;
 
-	view = GTK_TREE_VIEW (preferences->spell_checker_treeview);
+	view = GTK_TREE_VIEW (preferences->treeview_spell_checker);
 	model = gtk_tree_view_get_model (view);
 	store = GTK_LIST_STORE (model);
 	
@@ -424,24 +426,23 @@ preferences_themes_setup (GossipPreferences *preferences)
 	GtkComboBox     *combo;
 	GtkListStore    *model;
 	GtkTreeIter      iter;
+	const gchar    **themes;
+	gint             i;
 
-	combo = GTK_COMBO_BOX (preferences->theme_combobox);
+	combo = GTK_COMBO_BOX (preferences->combobox_chat_theme);
 
 	model = gtk_list_store_new (COL_COMBO_COUNT,
 				    G_TYPE_STRING,
 				    G_TYPE_STRING);
 
-	gtk_list_store_append (model, &iter);
-	gtk_list_store_set (model, &iter,
-			    COL_COMBO_VISIBLE_NAME, _("Classic"),
-			    COL_COMBO_NAME, "classic",
-			    -1);
-	
-	gtk_list_store_append (model, &iter);
-	gtk_list_store_set (model, &iter,
-			    COL_COMBO_VISIBLE_NAME, _("Blue"),
-			    COL_COMBO_NAME, "blue",
-			    -1);
+	themes = gossip_theme_manager_get_themes ();
+ 	for (i = 0; themes[i]; i += 2) {
+		gtk_list_store_append (model, &iter);
+		gtk_list_store_set (model, &iter,
+				    COL_COMBO_VISIBLE_NAME, _(themes[i + 1]),
+				    COL_COMBO_NAME, themes[i],
+				    -1);
+	}
 	
 	gtk_combo_box_set_model (combo, GTK_TREE_MODEL (model));
 	g_object_unref (model);
@@ -843,14 +844,15 @@ gossip_preferences_show (void)
 		NULL,
 		"preferences_dialog", &preferences->dialog,
 		"notebook", &preferences->notebook,
-		"sound_checkbutton", &preferences->sound_checkbutton,
-		"silent_busy_checkbutton", &preferences->silent_busy_checkbutton,
-		"silent_away_checkbutton", &preferences->silent_away_checkbutton,
-		"separate_windows_checkbutton", &preferences->separate_windows_checkbutton,
-		"smileys_checkbutton", &preferences->smileys_checkbutton,
-		"theme_combobox", &preferences->theme_combobox,
-		"spell_checker_vbox", &preferences->spell_checker_vbox,
-		"spell_checker_treeview", &preferences->spell_checker_treeview,
+		"checkbutton_show_avatars", &preferences->checkbutton_show_avatars,
+		"checkbutton_show_smileys", &preferences->checkbutton_show_smileys,
+		"combobox_chat_theme", &preferences->combobox_chat_theme,
+		"checkbutton_separate_chat_windows", &preferences->checkbutton_separate_chat_windows,
+		"checkbutton_sounds_for_messages", &preferences->checkbutton_sounds_for_messages,
+		"checkbutton_sounds_when_busy", &preferences->checkbutton_sounds_when_busy,
+		"checkbutton_sounds_when_away", &preferences->checkbutton_sounds_when_away,
+		"checkbutton_popups_when_available", &preferences->checkbutton_popups_when_available,
+		"treeview_spell_checker", &preferences->treeview_spell_checker,
 		NULL);
 
 	gossip_glade_connect (glade, 
@@ -859,6 +861,8 @@ gossip_preferences_show (void)
 			      "preferences_dialog", "response", preferences_response_cb,
 			      NULL);
 
+	g_object_unref (glade);
+
 	g_object_add_weak_pointer (G_OBJECT (preferences->dialog), (gpointer) &preferences);
 	
 	preferences_themes_setup (preferences);
