@@ -51,9 +51,15 @@ typedef struct {
 	GossipPresenceState  flash_state_2;
 
 	guint                flash_timeout_id;
+
+	/* The handle the kind of unnessecary scroll support. */
+	guint                scroll_timeout_id;
+	GossipPresenceState  scroll_state;
+	gchar               *scroll_status;
 } GossipPresenceChooserPriv;
 
 static void     presence_chooser_finalize               (GObject               *object);
+static void     presence_chooser_reset_scroll_timeout   (GossipPresenceChooser *chooser);
 static void     presence_chooser_set_state              (GossipPresenceChooser *chooser,
 							 GossipPresenceState    state,
 							 const gchar           *status,
@@ -85,6 +91,9 @@ static void     presence_chooser_menu_popup             (GossipPresenceChooser *
 static void     presence_chooser_menu_popdown           (GossipPresenceChooser *chooser);
 static void     presence_chooser_toggled_cb             (GtkWidget             *chooser,
 							 gpointer               user_data);
+static gboolean presence_chooser_button_press_event_cb  (GtkWidget             *chooser,
+							 GdkEventButton        *event,
+							 gpointer               user_data);
 static gboolean presence_chooser_scroll_event_cb        (GtkWidget             *chooser,
 							 GdkEventScroll        *event,
 							 gpointer               user_data);
@@ -150,7 +159,6 @@ gossip_presence_chooser_init (GossipPresenceChooser *chooser)
 	priv->label = gtk_label_new (NULL);
 	gtk_widget_show (priv->label);
 	gtk_box_pack_start (GTK_BOX (priv->hbox), priv->label, TRUE, TRUE, 0);
-/*  	gtk_label_set_ellipsize (GTK_LABEL (priv->label), PANGO_ELLIPSIZE_NONE);  */
 	gtk_label_set_ellipsize (GTK_LABEL (priv->label), PANGO_ELLIPSIZE_END);
  	gtk_misc_set_alignment (GTK_MISC (priv->label), 0, 0.5); 
 
@@ -166,6 +174,9 @@ gossip_presence_chooser_init (GossipPresenceChooser *chooser)
         g_signal_connect (chooser, "toggled",
                           G_CALLBACK (presence_chooser_toggled_cb),
                           NULL);
+        g_signal_connect (chooser, "button-press-event",
+                          G_CALLBACK (presence_chooser_button_press_event_cb),
+                          NULL);
         g_signal_connect (chooser, "scroll-event",
                           G_CALLBACK (presence_chooser_scroll_event_cb),
                           NULL);
@@ -180,12 +191,31 @@ presence_chooser_finalize (GObject *object)
 
 	if (priv->flash_timeout_id) {
 		g_source_remove (priv->flash_timeout_id);
-		priv->flash_timeout_id = 0;
+	}
+
+	if (priv->scroll_timeout_id) {
+		g_source_remove (priv->scroll_timeout_id);
 	}
 
 	G_OBJECT_CLASS (gossip_presence_chooser_parent_class)->finalize (object);
 }
 
+static void
+presence_chooser_reset_scroll_timeout (GossipPresenceChooser *chooser)
+{
+	GossipPresenceChooserPriv *priv;
+
+	priv = GET_PRIV (chooser);
+
+	if (priv->scroll_timeout_id) {
+		g_source_remove (priv->scroll_timeout_id);
+		priv->scroll_timeout_id = 0;
+	}
+
+	g_free (priv->scroll_status);
+	priv->scroll_status = NULL;
+}
+
 static void
 presence_chooser_set_state (GossipPresenceChooser *chooser,
 			    GossipPresenceState    state,
@@ -210,6 +240,7 @@ presence_chooser_set_state (GossipPresenceChooser *chooser,
 
 	priv->last_state = state;
 
+	presence_chooser_reset_scroll_timeout (chooser);
 	g_signal_emit (chooser, signals[CHANGED], 0, state, status);
 }
 
@@ -218,16 +249,22 @@ presence_chooser_dialog_response_cb (GtkWidget             *dialog,
 				     gint                   response,
 				     GossipPresenceChooser *chooser)
 {
-	GtkWidget           *entry;
-	GtkWidget           *checkbutton;
-	GossipPresenceState  state;
-	const gchar         *status;
-	gboolean             save;
-
 	if (response == GTK_RESPONSE_OK) {
+		GtkWidget           *entry;
+		GtkWidget           *checkbutton;
+		GtkListStore        *store;
+		GtkTreeIter          iter;
+		GossipPresenceState  state;
+		const gchar         *status;
+		gboolean             save;
+
 		entry = g_object_get_data (G_OBJECT (dialog), "entry");
 		status = gtk_entry_get_text (GTK_ENTRY (entry));
 
+		store = g_object_get_data (G_OBJECT (dialog), "store");
+		gtk_list_store_append (store, &iter);
+		gtk_list_store_set (store, &iter, 0, status, -1);
+
 		state = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (dialog), "state"));
 
 		checkbutton = g_object_get_data (G_OBJECT (dialog), "checkbutton");
@@ -245,17 +282,15 @@ presence_chooser_show_dialog (GossipPresenceChooser *chooser,
 {
         GossipPresenceChooserPriv *priv;
 	static GtkWidget          *dialog;
+	static GtkListStore       *store[3] = { NULL, NULL, NULL };
 	GladeXML                  *glade;
+	GtkWidget                 *parent;
 	GtkWidget                 *image;
+	GtkWidget                 *combo;
 	GtkWidget                 *entry;
 	GtkWidget                 *checkbutton;
-	GtkTreeIter                iter;
-	const gchar               *default_status;
-	GtkEntryCompletion        *completion;
 	GdkPixbuf                 *pixbuf;
-	GtkListStore              *store;
-	GList                     *presets, *l;
-	GtkWidget                 *parent;
+	const gchar               *default_status;
 	gboolean                   visible;
 
         priv = GET_PRIV (chooser);
@@ -269,53 +304,42 @@ presence_chooser_show_dialog (GossipPresenceChooser *chooser,
 				       "status_message_dialog",
 				       NULL,
 				       "status_message_dialog", &dialog,
-				       "status_entry", &entry,
-				       "status_image", &image,
-				       "add_checkbutton", &checkbutton,
+				       "comboentry_status", &combo,
+				       "image_status", &image,
+				       "checkbutton_add", &checkbutton,
 				       NULL);
 
-	g_signal_connect (dialog,
-			  "destroy",
+	g_object_unref (glade);
+
+	g_signal_connect (dialog, "destroy",
 			  G_CALLBACK (gtk_widget_destroyed),
 			  &dialog);
+	g_signal_connect (dialog, "response",
+			  G_CALLBACK (presence_chooser_dialog_response_cb),
+			  chooser);
 
 	pixbuf = gossip_pixbuf_for_presence_state (state);
 	gtk_image_set_from_pixbuf (GTK_IMAGE (image), pixbuf);
 	g_object_unref (pixbuf);
 
-	store = gtk_list_store_new (1, G_TYPE_STRING);
-
-	presets = gossip_status_presets_get (state, -1);
-	for (l = presets; l; l = l->next) {
-		const gchar *status;
-
-		status = l->data;
-		gtk_list_store_append (store, &iter);
-		gtk_list_store_set (store, &iter, 0, status, -1);
+	if (!store[state]) {
+		store[state] = gtk_list_store_new (1, G_TYPE_STRING);
 	}
 
-	g_list_free (presets);
-
 	default_status = gossip_presence_state_get_default_status (state);
 
-	gtk_list_store_append (store, &iter);
-	gtk_list_store_set (store, &iter, 0, default_status, -1);
+	entry = GTK_BIN (combo)->child;
 	gtk_entry_set_text (GTK_ENTRY (entry), default_status);
+	gtk_entry_set_activates_default (GTK_ENTRY (entry), TRUE);
+	gtk_entry_set_width_chars (GTK_ENTRY (entry), 25);
 
-	completion = gtk_entry_completion_new ();
-	gtk_entry_completion_set_model (completion, GTK_TREE_MODEL (store));
-	gtk_entry_completion_set_text_column (completion, 0);
-	gtk_entry_set_completion (GTK_ENTRY (entry), completion);
-	g_object_unref (completion);
-
-	g_object_unref (store);
+	gtk_combo_box_set_model (GTK_COMBO_BOX (combo), GTK_TREE_MODEL (store[state]));
+	gtk_combo_box_entry_set_text_column (GTK_COMBO_BOX_ENTRY (combo), 0);
 
 	/* We make the dialog transient to the app window if it's visible. */
 	parent = gossip_app_get_window ();
 
-	g_object_get (parent,
-		      "visible", &visible,
-		      NULL);
+	g_object_get (parent, "visible", &visible, NULL);
 
 	visible = visible && !(gdk_window_get_state (parent->window) &
 			       GDK_WINDOW_STATE_ICONIFIED);
@@ -325,15 +349,11 @@ presence_chooser_show_dialog (GossipPresenceChooser *chooser,
 					      GTK_WINDOW (parent));
 	}
 
+	g_object_set_data (G_OBJECT (dialog), "store", store[state]);
 	g_object_set_data (G_OBJECT (dialog), "entry", entry);
 	g_object_set_data (G_OBJECT (dialog), "checkbutton", checkbutton);
 	g_object_set_data (G_OBJECT (dialog), "state", GINT_TO_POINTER (state));
 
-	g_signal_connect (dialog,
-			  "response",
-			  G_CALLBACK (presence_chooser_dialog_response_cb),
-			  chooser);
-
 	gtk_widget_show_all (dialog);
 }
 
@@ -358,6 +378,7 @@ presence_chooser_noncustom_activate_cb (GtkWidget             *item,
 	status = g_object_get_data (G_OBJECT (item), "status");
 	state = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (item), "state"));
 
+	presence_chooser_reset_scroll_timeout (chooser);
 	g_signal_emit (chooser, signals[CHANGED], 0, state, status);
 }
 
@@ -584,6 +605,23 @@ presence_chooser_toggled_cb (GtkWidget *chooser,
 	}
 }
 
+static gboolean
+presence_chooser_button_press_event_cb (GtkWidget      *chooser,
+					GdkEventButton *event,
+					gpointer        user_data)
+{
+	if (event->button != 1 || event->type != GDK_BUTTON_PRESS) {
+		return FALSE;
+	}
+
+	if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (chooser))) {
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (chooser), TRUE);
+			return TRUE;
+		}
+
+	return FALSE;
+}
+
 typedef struct {
 	GossipPresenceState  state;
 	const gchar         *status;
@@ -650,6 +688,25 @@ presence_chooser_get_presets (GossipPresenceChooser *chooser)
 	return list;
 }
 
+static gboolean
+presence_chooser_scroll_timeout_cb (GossipPresenceChooser *chooser)
+{
+	GossipPresenceChooserPriv *priv;
+
+	priv = GET_PRIV (chooser);
+	
+	g_signal_emit (chooser, signals[CHANGED], 0,
+		       priv->scroll_state,
+		       priv->scroll_status);
+	
+	priv->scroll_timeout_id = 0;
+
+	g_free (priv->scroll_status);
+	priv->scroll_status = NULL;
+
+	return FALSE;
+}
+
 static gboolean
 presence_chooser_scroll_event_cb (GtkWidget      *chooser,
 				  GdkEventScroll *event,
@@ -702,16 +759,28 @@ presence_chooser_scroll_event_cb (GtkWidget      *chooser,
 
 		sas = NULL;
 	}
-			
-	if (sas) {
-		g_signal_emit (chooser, signals[CHANGED], 0,
-			       sas->state, sas->status);
 
+	if (sas) {
+		presence_chooser_reset_scroll_timeout (GOSSIP_PRESENCE_CHOOSER (chooser));
+
+		priv->scroll_status = g_strdup (sas->status);
+		priv->scroll_state = sas->state;
+		
+		priv->scroll_timeout_id =
+			g_timeout_add (500,
+				       (GSourceFunc) presence_chooser_scroll_timeout_cb,
+				       chooser);
+
+		gossip_presence_chooser_set_status (GOSSIP_PRESENCE_CHOOSER (chooser),
+						    sas->status);
+		gossip_presence_chooser_set_state (GOSSIP_PRESENCE_CHOOSER (chooser),
+						   sas->state);
 	}
 	else if (!match) {
 		/* If we didn't get any match at all, it means the last state
 		 * was a custom one. Just switch to the first one.
 		 */ 
+		presence_chooser_reset_scroll_timeout (GOSSIP_PRESENCE_CHOOSER (chooser));
 		g_signal_emit (chooser, signals[CHANGED], 0,
 			       GOSSIP_PRESENCE_STATE_AVAILABLE,
 			       _("Available"));
