@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2002-2004 Imendio AB
+ * Copyright (C) 2002-2006 Imendio AB
  * Copyright (C) 2003-2004 Geert-Jan Van den Bogaerde <geertjan@gnome.org>
  *
  * This program is free software; you can redistribute it and/or
@@ -28,10 +28,10 @@
 #include <glib/gi18n.h>
 #include <libgnomeui/gnome-href.h>
 
+#include <libgossip/gossip-debug.h>
 #include <libgossip/gossip-message.h>
 
 #include "gossip-app.h"
-#include "gossip-avatar-image.h"
 #include "gossip-chat-window.h"
 #include "gossip-chat-view.h"
 #include "gossip-chat.h"
@@ -42,10 +42,10 @@
 #include "gossip-stock.h"
 #include "gossip-ui-utils.h"
 
-#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GOSSIP_TYPE_PRIVATE_CHAT, GossipPrivateChatPriv))
+#define GET_PRIV(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), \
+		       GOSSIP_TYPE_PRIVATE_CHAT, GossipPrivateChatPriv))
 
-#define DEBUG_MSG(x)  
-/* #define DEBUG_MSG(args) g_printerr args ; g_printerr ("\n");  */
+#define DEBUG_DOMAIN "PrivateChat"
 
 #define IS_ENTER(v) (v == GDK_Return || v == GDK_ISO_Enter || v == GDK_KP_Enter)
 
@@ -55,19 +55,20 @@ struct _GossipPrivateChatPriv {
         GossipContact    *contact;
         GossipContact    *own_contact;
 	gchar            *name;
-	
+
+	GdkPixbuf        *own_avatar;
+	GdkPixbuf        *other_avatar;
+
 	gchar            *locked_resource;
 	gchar            *roster_resource;
 
         guint             composing_stop_timeout_id;
-	
+	guint             scroll_idle_id;
+
 	gboolean          is_online;
 
         GtkWidget        *widget;
 	GtkWidget	 *text_view_sw;
-
-	GtkWidget        *avatar_image;
-	GtkWidget        *avatar_image_placeholder;
 };
 
 static void           gossip_private_chat_class_init            (GossipPrivateChatClass *klass);
@@ -76,11 +77,6 @@ static void           private_chat_finalize                     (GObject
 static void           private_chat_create_ui                    (GossipPrivateChat      *chat);
 static void           private_chat_send                         (GossipPrivateChat      *chat,
 								 const gchar            *msg);
-static void           private_chat_update_avatar                (GossipPrivateChat      *chat);
-static void           private_chat_show_avatars_key_changed_cb  (GConfClient            *client,
-								 guint                   cnxn_id,
-								 GConfEntry             *entry,
-								 GossipPrivateChat      *chat);
 static void           private_chat_composing_start              (GossipPrivateChat      *chat);
 static void           private_chat_composing_stop               (GossipPrivateChat      *chat);
 static void           private_chat_composing_remove_timeout     (GossipPrivateChat      *chat);
@@ -94,9 +90,13 @@ static void           private_chat_contact_updated              (gpointer
 static void           private_chat_contact_added                (gpointer                not_used,
 								 GossipContact          *contact,
 								 GossipPrivateChat      *chat);
-static void           private_chat_connected_cb                 (GossipSession          *session,
+static void           private_chat_protocol_connected_cb        (GossipSession          *session,
+								 GossipAccount          *account,
+								 GossipProtocol         *protocol,
 								 GossipPrivateChat      *chat);
-static void           private_chat_disconnected_cb              (GossipSession          *session,
+static void           private_chat_protocol_disconnected_cb     (GossipSession          *session,
+								 GossipAccount          *account,
+								 GossipProtocol         *protocol,
 								 GossipPrivateChat      *chat);
 static void           private_chat_composing_cb                 (GossipSession          *session,
 								 GossipContact          *contact,
@@ -113,25 +113,31 @@ static gboolean       private_chat_text_view_focus_in_event_cb  (GtkWidget
 static gboolean       private_chat_delete_event_cb              (GtkWidget              *widget,
 								 GdkEvent               *event,
 								 GossipPrivateChat      *chat);
+static void           private_chat_own_avatar_notify_cb         (GossipContact          *contact,
+								 GParamSpec             *pspec,
+								 GossipPrivateChat      *chat);
+static void           private_chat_other_avatar_notify_cb       (GossipContact          *contact,
+								 GParamSpec             *pspec,
+								 GossipPrivateChat      *chat);
 static const gchar *  private_chat_get_name                     (GossipChat             *chat);
 static gchar *        private_chat_get_tooltip                  (GossipChat             *chat);
 static GdkPixbuf *    private_chat_get_status_pixbuf            (GossipChat             *chat);
 static GtkWidget *    private_chat_get_widget                   (GossipChat             *chat);
 static GossipContact *private_chat_get_contact                  (GossipChat             *chat);
 static GossipContact *private_chat_get_own_contact              (GossipChat             *chat);
-
- static GObjectClass *parent_class  = NULL;
+static GdkPixbuf *    private_chat_pad_to_size                  (GdkPixbuf              *pixbuf, 
+								 gint                    width,
+								 gint                    height,
+								 gint                    extra_padding_right);
 
 G_DEFINE_TYPE (GossipPrivateChat, gossip_private_chat, GOSSIP_TYPE_CHAT);
 
 static void
 gossip_private_chat_class_init (GossipPrivateChatClass *klass)
 {
-        GObjectClass *object_class = G_OBJECT_CLASS (klass);
+        GObjectClass    *object_class = G_OBJECT_CLASS (klass);
 	GossipChatClass *chat_class = GOSSIP_CHAT_CLASS (klass);
 
-        parent_class = g_type_class_peek_parent (klass);
-
         object_class->finalize = private_chat_finalize;
 	
 	chat_class->get_name          = private_chat_get_name;
@@ -155,16 +161,16 @@ gossip_private_chat_init (GossipPrivateChat *chat)
 
         private_chat_create_ui (chat);
 
-	DEBUG_MSG (("PrivateChat: Connecting"));
+	gossip_debug (DEBUG_DOMAIN, "Connecting");
 
 	g_signal_connect_object (gossip_app_get_session (),
-				 "connected",
-				 G_CALLBACK (private_chat_connected_cb),
+				 "protocol-connected",
+				 G_CALLBACK (private_chat_protocol_connected_cb),
 				 chat, 0);
 
 	g_signal_connect_object (gossip_app_get_session (),
-				 "disconnected",
-				 G_CALLBACK (private_chat_disconnected_cb),
+				 "protocol-disconnected",
+				 G_CALLBACK (private_chat_protocol_disconnected_cb),
 				 chat, 0);
 	
 	g_signal_connect_object (gossip_app_get_session (),
@@ -182,7 +188,7 @@ private_chat_finalize (GObject *object)
 	chat = GOSSIP_PRIVATE_CHAT (object);
 	priv = GET_PRIV (chat);
 
-	if (priv->own_contact) {
+	if (priv->contact) {
 		g_object_unref (priv->contact);
 	}
 
@@ -190,9 +196,25 @@ private_chat_finalize (GObject *object)
 		g_object_unref (priv->own_contact);
 	}
 
+	if (priv->own_avatar) {
+		g_object_unref (priv->own_avatar);
+	}
+
+	if (priv->other_avatar) {
+		g_object_unref (priv->other_avatar);
+	}
+
+	if (priv->scroll_idle_id) {
+		g_source_remove (priv->scroll_idle_id);
+	}
+
+	g_free (priv->name);
+	g_free (priv->locked_resource);
+	g_free (priv->roster_resource);
+	
 	private_chat_composing_remove_timeout (chat);
 	
-        G_OBJECT_CLASS (parent_class)->finalize (object);
+        G_OBJECT_CLASS (gossip_private_chat_parent_class)->finalize (object);
 }
 
 static void
@@ -210,16 +232,8 @@ private_chat_create_ui (GossipPrivateChat *chat)
                                       "chat_widget", &priv->widget,
                                       "chat_view_sw", &priv->text_view_sw,
 				      "input_text_view_sw", &input_text_view_sw,
-				      "avatar_image_placeholder", &priv->avatar_image_placeholder,
                                       NULL);
 
-	priv->avatar_image = gossip_avatar_image_new (NULL);
-	gtk_container_add (GTK_CONTAINER (priv->avatar_image_placeholder),
-			   priv->avatar_image);
-
-	gtk_widget_show (priv->avatar_image);
-	gtk_widget_hide (priv->avatar_image_placeholder);
-
 	gtk_container_add (GTK_CONTAINER (priv->text_view_sw),
 			   GTK_WIDGET (GOSSIP_CHAT (chat)->view));
 	gtk_widget_show (GTK_WIDGET (GOSSIP_CHAT (chat)->view));
@@ -229,7 +243,6 @@ private_chat_create_ui (GossipPrivateChat *chat)
 	gtk_widget_show (GOSSIP_CHAT (chat)->input_text_view);
 
 	g_object_ref (priv->widget);
-
 	g_object_set_data (G_OBJECT (priv->widget), "chat", chat);
 
 	g_signal_connect (GOSSIP_CHAT (chat)->input_text_view,
@@ -290,7 +303,7 @@ private_chat_update_locked_resource (GossipPrivateChat *chat)
 	
 	if (priv->roster_resource &&
 	    g_ascii_strcasecmp (priv->roster_resource, roster_resource) == 0) {
-		DEBUG_MSG (("PrivateChat: Roster unchanged"));
+		gossip_debug (DEBUG_DOMAIN, "Roster unchanged");
 
 		if (!priv->locked_resource) {
 			priv->locked_resource = g_strdup (roster_resource);
@@ -299,7 +312,7 @@ private_chat_update_locked_resource (GossipPrivateChat *chat)
 		return;
 	}
 	
-	DEBUG_MSG (("PrivateChat: New roster resource: %s", roster_resource));
+	gossip_debug (DEBUG_DOMAIN, "New roster resource: %s", roster_resource);
 	
 	g_free (priv->roster_resource);
 	priv->roster_resource = g_strdup (roster_resource);
@@ -326,7 +339,7 @@ private_chat_send (GossipPrivateChat *chat,
                 return;
         }
 
-	gossip_app_force_non_away ();
+	gossip_app_set_not_away ();
 
         private_chat_composing_remove_timeout (chat);
 
@@ -346,58 +359,14 @@ private_chat_send (GossipPrivateChat *chat,
 
 	gossip_chat_view_append_message_from_self (GOSSIP_CHAT (chat)->view, 
 						   message, 
-						   priv->own_contact);
+						   priv->own_contact,
+						   priv->own_avatar);
 
 	gossip_session_send_message (gossip_app_get_session (), message);
 	
 	g_object_unref (message);
 }
 
-static void
-private_chat_update_avatar (GossipPrivateChat *chat)
-{
-	GossipPrivateChatPriv *priv;
-	GConfClient           *gconf_client;
-	GdkPixbuf             *pixbuf = NULL;
-	gboolean               show_avatars;
-
-	priv = GET_PRIV (chat);
-
-	gconf_client = gossip_app_get_gconf_client ();
-
-	show_avatars = gconf_client_get_bool (gconf_client,
-					      GCONF_UI_SHOW_AVATARS,
-					      NULL);
-
-	if (show_avatars) {
-		pixbuf = gossip_pixbuf_avatar_from_contact (priv->contact);
-		if (pixbuf) {
-			gossip_avatar_image_set_pixbuf (GOSSIP_AVATAR_IMAGE (priv->avatar_image), 
-							pixbuf);
-			g_object_unref (pixbuf);
-		}
-	}
-
-	if (pixbuf) {
-		/* FIXME: We are not happy with the positioning of
-		 * this for the time being it will be hidden.
-		 */
-		gtk_widget_hide (priv->avatar_image_placeholder);
-/* 		gtk_widget_show (priv->avatar_image_placeholder); */
-	} else {
-		gtk_widget_hide (priv->avatar_image_placeholder);
-	}
-}
-
-static void
-private_chat_show_avatars_key_changed_cb (GConfClient       *client,
-					  guint              cnxn_id,
-					  GConfEntry        *entry,
-					  GossipPrivateChat *chat)
-{
-	private_chat_update_avatar (chat);
-}
-
 static void
 private_chat_composing_start (GossipPrivateChat *chat)
 {
@@ -465,17 +434,14 @@ private_chat_contact_presence_updated (gpointer           not_used,
 {
 	GossipPrivateChatPriv *priv;
 
-	g_return_if_fail (GOSSIP_IS_PRIVATE_CHAT (chat));
-	g_return_if_fail (contact != NULL);
-	
 	priv = GET_PRIV (chat);
 
 	if (!gossip_contact_equal (contact, priv->contact)) {
 		return;
 	}
 
-	DEBUG_MSG (("PrivateChat: Presence update for contact:'%s'", 
-		   gossip_contact_get_id (contact)));
+	gossip_debug (DEBUG_DOMAIN, "Presence update for contact: %s",
+		      gossip_contact_get_id (contact));
 
 	g_signal_emit_by_name (chat, "status-changed");
 
@@ -509,8 +475,6 @@ private_chat_contact_presence_updated (gpointer           not_used,
 
 		priv->is_online = TRUE;
 	}
-
-	private_chat_update_avatar (chat);
 }
 
 static void
@@ -520,9 +484,6 @@ private_chat_contact_updated (gpointer           not_used,
 {
 	GossipPrivateChatPriv *priv;
 
-	g_return_if_fail (GOSSIP_IS_PRIVATE_CHAT (chat));
-	g_return_if_fail (contact != NULL);
-	
 	priv = GET_PRIV (chat);
 
 	if (!gossip_contact_equal (contact, priv->contact)) {
@@ -534,8 +495,6 @@ private_chat_contact_updated (gpointer           not_used,
 		priv->name = g_strdup (gossip_contact_get_name (contact));
 		g_signal_emit_by_name (chat, "name-changed", priv->name);
 	}
-
-	private_chat_update_avatar (chat);
 }
 
 static void
@@ -545,48 +504,68 @@ private_chat_contact_added (gpointer           not_user,
 {
 	GossipPrivateChatPriv *priv;
 
-	g_return_if_fail (GOSSIP_IS_PRIVATE_CHAT (chat));
-	g_return_if_fail (contact != NULL);
-
 	priv = GET_PRIV (chat);
 	
 	if (!gossip_contact_equal (contact, priv->contact)) {
 		return;
 	}
-	
+
+	g_signal_handlers_disconnect_by_func (priv->contact,
+					      private_chat_other_avatar_notify_cb,
+					      chat);
+		
 	g_object_unref (priv->contact);
 	priv->contact = g_object_ref (contact);
+
+	g_signal_connect (priv->contact,
+			  "notify::avatar",
+			  G_CALLBACK (private_chat_other_avatar_notify_cb),
+			  chat);
 }
 
 static void
-private_chat_connected_cb (GossipSession     *session, 
-			   GossipPrivateChat *chat)
+private_chat_protocol_connected_cb (GossipSession     *session,
+				    GossipAccount     *account,
+				    GossipProtocol    *protocol,
+				    GossipPrivateChat *chat)
 {
 	GossipPrivateChatPriv *priv;
-
-	g_return_if_fail (GOSSIP_IS_PRIVATE_CHAT (chat));
+	GossipAccount         *this_account;
 
 	priv = GET_PRIV (chat);
 
+	this_account = gossip_contact_get_account (priv->own_contact);
+	if (!gossip_account_equal (this_account, account)) {
+		return;
+	}
+
 	gtk_widget_set_sensitive (GOSSIP_CHAT (chat)->input_text_view, TRUE);
 
 	gossip_chat_view_append_event (GOSSIP_CHAT (chat)->view, _("Connected"));
 }
 
 static void
-private_chat_disconnected_cb (GossipSession     *session, 
-			      GossipPrivateChat *chat)
+private_chat_protocol_disconnected_cb (GossipSession     *session,
+				       GossipAccount     *account,
+				       GossipProtocol    *protocol,
+				       GossipPrivateChat *chat)
 {
 	GossipPrivateChatPriv *priv;
-
-	g_return_if_fail (GOSSIP_IS_PRIVATE_CHAT (chat));
+	GossipAccount         *this_account;
 
 	priv = GET_PRIV (chat);
 
+	this_account = gossip_contact_get_account (priv->own_contact);
+	if (!gossip_account_equal (this_account, account)) {
+		return;
+	}
+
 	gtk_widget_set_sensitive (GOSSIP_CHAT (chat)->input_text_view, FALSE);
 
 	gossip_chat_view_append_event (GOSSIP_CHAT (chat)->view, _("Disconnected"));
 
+	g_signal_emit_by_name (chat, "status-changed");
+
 	private_chat_composing_remove_timeout (chat);
 }
 
@@ -599,15 +578,13 @@ private_chat_composing_cb (GossipSession *session,
 	GossipPrivateChat     *p_chat;
 	GossipPrivateChatPriv *priv;
 
-	g_return_if_fail (GOSSIP_IS_PRIVATE_CHAT (chat));
-
 	p_chat = GOSSIP_PRIVATE_CHAT (chat);
 	priv = GET_PRIV (p_chat);
 
 	if (gossip_contact_equal (contact, priv->contact)) {
-		DEBUG_MSG (("PrivateChat: Contact:'%s' %s typing",
-			   gossip_contact_get_name (contact),
-			   composing ? "is" : "is not"));
+		gossip_debug (DEBUG_DOMAIN, "Contact: '%s' %s typing",
+			      gossip_contact_get_name (contact),
+			      composing ? "is" : "is not");
 
 		g_signal_emit_by_name (chat, "composing", composing);
 	}
@@ -649,6 +626,10 @@ private_chat_input_key_press_event_cb (GtkWidget         *widget,
 	
 	priv = GET_PRIV (chat);
 
+	if (event->keyval == GDK_Tab && !(event->state & GDK_CONTROL_MASK)) {
+		return TRUE;
+	}
+	
 	/* Catch enter but not ctrl/shift-enter */
 	if (IS_ENTER (event->keyval) && !(event->state & GDK_SHIFT_MASK)) {
 		GtkTextView *view;
@@ -725,16 +706,60 @@ private_chat_delete_event_cb (GtkWidget         *widget,
 	return TRUE;
 }
 
+static void
+private_chat_own_avatar_notify_cb (GossipContact     *contact,
+				   GParamSpec        *pspec,
+				   GossipPrivateChat *chat)
+{
+	GossipPrivateChatPriv *priv;
+	GdkPixbuf             *pixbuf;
+
+	priv = GET_PRIV (chat);
+	
+	if (priv->own_avatar) {
+		g_object_unref (priv->own_avatar);
+	}
+	
+	pixbuf = gossip_pixbuf_avatar_from_contact_scaled (priv->own_contact, 32, 32);
+	if (pixbuf) {
+		priv->own_avatar = private_chat_pad_to_size (pixbuf, 32, 32, 6);
+		g_object_unref (pixbuf);
+	} else {
+		priv->own_avatar = NULL;
+	}
+}
+
+static void
+private_chat_other_avatar_notify_cb (GossipContact     *contact,
+				     GParamSpec        *pspec,
+				     GossipPrivateChat *chat)
+{
+	GossipPrivateChatPriv *priv;
+	GdkPixbuf             *pixbuf;
+
+	priv = GET_PRIV (chat);
+
+	if (priv->other_avatar) {
+		g_object_unref (priv->other_avatar);
+	}
+	
+	pixbuf = gossip_pixbuf_avatar_from_contact_scaled (priv->contact, 32, 32);
+	if (pixbuf) {
+		priv->other_avatar = private_chat_pad_to_size (pixbuf, 32, 32, 6);
+		g_object_unref (pixbuf);
+	} else {
+		priv->other_avatar = NULL;
+	}
+}
+
 static const gchar *
 private_chat_get_name (GossipChat *chat)
 {
-	GossipPrivateChat     *p_chat;
 	GossipPrivateChatPriv *priv;
 
 	g_return_val_if_fail (GOSSIP_IS_PRIVATE_CHAT (chat), NULL);
 
-	p_chat = GOSSIP_PRIVATE_CHAT (chat);
-	priv = GET_PRIV (p_chat);
+	priv = GET_PRIV (chat);
 
 	return priv->name;
 }
@@ -742,15 +767,13 @@ private_chat_get_name (GossipChat *chat)
 static gchar *
 private_chat_get_tooltip (GossipChat *chat)
 {
-	GossipPrivateChat     *p_chat;
 	GossipPrivateChatPriv *priv;
 	GossipContact         *contact;
 	const gchar           *status;
 
 	g_return_val_if_fail (GOSSIP_IS_PRIVATE_CHAT (chat), NULL);
 
-	p_chat = GOSSIP_PRIVATE_CHAT (chat);
-	priv = GET_PRIV (p_chat);
+	priv = GET_PRIV (chat);
 
 	contact = gossip_chat_get_contact (chat);
 	status = gossip_contact_get_status (contact);
@@ -763,14 +786,12 @@ private_chat_get_tooltip (GossipChat *chat)
 GdkPixbuf *
 private_chat_get_status_pixbuf (GossipChat *chat)
 {
-	GossipPrivateChat     *p_chat;
 	GossipPrivateChatPriv *priv;
 	GossipContact         *contact;
 
 	g_return_val_if_fail (GOSSIP_IS_PRIVATE_CHAT (chat), NULL);
 
-	p_chat = GOSSIP_PRIVATE_CHAT (chat);
-	priv = GET_PRIV (p_chat);
+	priv = GET_PRIV (chat);
 
 	contact = gossip_chat_get_contact (chat);
 
@@ -780,13 +801,11 @@ private_chat_get_status_pixbuf (GossipChat *chat)
 static GossipContact *
 private_chat_get_contact (GossipChat *chat)
 {
-	GossipPrivateChat     *p_chat;
 	GossipPrivateChatPriv *priv;
 
 	g_return_val_if_fail (GOSSIP_IS_PRIVATE_CHAT (chat), NULL);
 
-	p_chat = GOSSIP_PRIVATE_CHAT (chat);
-	priv = GET_PRIV (p_chat);
+	priv = GET_PRIV (chat);
 
 	return priv->contact;
 }
@@ -794,13 +813,11 @@ private_chat_get_contact (GossipChat *chat)
 static GossipContact *
 private_chat_get_own_contact (GossipChat *chat)
 {
-	GossipPrivateChat     *p_chat;
 	GossipPrivateChatPriv *priv;
 
 	g_return_val_if_fail (GOSSIP_IS_PRIVATE_CHAT (chat), NULL);
 
-	p_chat = GOSSIP_PRIVATE_CHAT (chat);
-	priv = GET_PRIV (p_chat);
+	priv = GET_PRIV (chat);
 
 	return priv->own_contact;
 }
@@ -808,24 +825,40 @@ private_chat_get_own_contact (GossipChat *chat)
 static GtkWidget *
 private_chat_get_widget (GossipChat *chat)
 {
-	GossipPrivateChat     *p_chat;
 	GossipPrivateChatPriv *priv;
 
-	g_return_val_if_fail (GOSSIP_IS_PRIVATE_CHAT (chat), NULL);
-
-	p_chat = GOSSIP_PRIVATE_CHAT (chat);
-	priv = GET_PRIV (p_chat);
+	priv = GET_PRIV (chat);
 
 	return priv->widget;
 }
 
+/* Scroll down after the back-log has been received. */
+static gboolean
+private_chat_scroll_down_idle_func (GossipChat *chat)
+{
+	GossipPrivateChatPriv *priv;
+
+	priv = GET_PRIV (chat);
+
+	gossip_chat_scroll_down (chat);
+ 	g_object_unref (chat);
+	
+	priv->scroll_idle_id = 0;
+	
+	return FALSE;
+}
+
 GossipPrivateChat *
 gossip_private_chat_new (GossipContact *own_contact,
 			 GossipContact *contact)
 {
 	GossipPrivateChat     *chat;
 	GossipPrivateChatPriv *priv;
-	GConfClient           *gconf_client;
+	GossipChatView        *view;
+	GossipContact         *sender;
+	GossipMessage         *message;
+	GList                 *messages, *l;
+	gint                   num_messages, i;
 
 	g_return_val_if_fail (GOSSIP_IS_CONTACT (own_contact), NULL);
 	g_return_val_if_fail (GOSSIP_IS_CONTACT (contact), NULL);
@@ -839,15 +872,59 @@ gossip_private_chat_new (GossipContact *own_contact,
 
 	priv->name = g_strdup (gossip_contact_get_name (contact));
 
-	/* Set up avatar */
-	gconf_client = gossip_app_get_gconf_client ();
+	g_signal_connect (priv->own_contact,
+			  "notify::avatar",
+			  G_CALLBACK (private_chat_own_avatar_notify_cb),
+			  chat);
+	
+	g_signal_connect (priv->contact,
+			  "notify::avatar",
+			  G_CALLBACK (private_chat_other_avatar_notify_cb),
+			  chat);
 
-	gconf_client_notify_add (gconf_client,
-				 GCONF_UI_SHOW_AVATARS,
-				 (GConfClientNotifyFunc) private_chat_show_avatars_key_changed_cb,
-				 chat, NULL, NULL);
+	private_chat_own_avatar_notify_cb (priv->own_contact, NULL, chat);
+	private_chat_other_avatar_notify_cb (priv->contact, NULL, chat);
+	
+	view = GOSSIP_CHAT (chat)->view;
 
-	private_chat_update_avatar (chat);
+	/* Turn off scrolling temporarily */
+	gossip_chat_view_scroll (view, FALSE);
+
+	/* Add messages from last conversation */
+	messages = gossip_log_get_last_for_contact (priv->contact);
+	num_messages  = g_list_length (messages);
+
+	for (l = messages, i = 0; l; l = l->next, i++) {
+		message = l->data;
+
+		if (num_messages - i > 10) {
+			continue;
+		}
+
+		sender = gossip_message_get_sender (message);
+		if (gossip_contact_equal (priv->own_contact, sender)) {
+			gossip_chat_view_append_message_from_self (view, 
+								   message,
+								   priv->own_contact,
+								   priv->own_avatar);
+		} else {
+			gossip_chat_view_append_message_from_other (view, 
+								    message,
+								    sender,
+								    priv->other_avatar);
+		}
+	}
+
+ 	g_list_foreach (messages, (GFunc) g_object_unref, NULL);
+	g_list_free (messages);
+
+	/* Turn back on scrolling */
+	gossip_chat_view_scroll (view, TRUE);
+
+	/* Scroll to the most recent messages */
+	g_object_ref (chat);
+	priv->scroll_idle_id = g_idle_add (
+		(GSourceFunc) private_chat_scroll_down_idle_func, chat);
 
 	/* Set up signals */
 	g_signal_connect_object (gossip_app_get_session (),
@@ -865,11 +942,7 @@ gossip_private_chat_new (GossipContact *own_contact,
 				 G_CALLBACK (private_chat_contact_added),
 				 chat, 0);
 
-	if (gossip_contact_is_online (priv->contact)) {
-		priv->is_online = TRUE;
-	} else {
-		priv->is_online = FALSE;
-	}
+	priv->is_online = gossip_contact_is_online (priv->contact);
 
 	return chat;
 }
@@ -916,8 +989,8 @@ gossip_private_chat_append_message (GossipPrivateChat *chat,
 
 	priv = GET_PRIV (chat);
 	
-	DEBUG_MSG (("PrivateChat: Appending message ('%s')",
-		   gossip_contact_get_name (gossip_message_get_sender (message))));
+	gossip_debug (DEBUG_DOMAIN, "Appending message ('%s')",
+		   gossip_contact_get_name (gossip_message_get_sender (message)));
 
 	sender = gossip_message_get_sender (message);
 	if (!gossip_contact_equal (priv->contact, sender)) {
@@ -958,7 +1031,8 @@ gossip_private_chat_append_message (GossipPrivateChat *chat,
 	} else {
 		gossip_chat_view_append_message_from_other (GOSSIP_CHAT (chat)->view,
 							    message,
-							    priv->own_contact);
+							    priv->own_contact,
+							    priv->other_avatar);
 	}
 
 	if (gossip_chat_should_play_sound (GOSSIP_CHAT (chat))) {
@@ -967,3 +1041,43 @@ gossip_private_chat_append_message (GossipPrivateChat *chat,
 
 	g_signal_emit_by_name (chat, "new-message", message);
 }
+
+/* Pads a pixbuf to the specified size, by centering it in a larger transparent
+ * pixbuf. Returns a new ref.
+ */
+static GdkPixbuf *
+private_chat_pad_to_size (GdkPixbuf *pixbuf, 
+			  gint       width,
+			  gint       height,
+			  gint       extra_padding_right)
+{
+	gint       src_width, src_height;
+	GdkPixbuf *padded;
+	gint       x_offset, y_offset;
+
+	src_width = gdk_pixbuf_get_width (pixbuf);
+	src_height = gdk_pixbuf_get_height (pixbuf);
+	
+	x_offset = (width - src_width) / 2;
+	y_offset = (height - src_height) / 2;
+
+	padded = gdk_pixbuf_new (gdk_pixbuf_get_colorspace (pixbuf),
+				 TRUE, /* alpha */
+				 gdk_pixbuf_get_bits_per_sample (pixbuf),
+				 width + extra_padding_right,
+				 height);
+
+	gdk_pixbuf_fill (padded, 0);
+
+	gdk_pixbuf_copy_area (pixbuf,
+			      0, /* source coords */
+			      0,
+			      src_width,
+			      src_height,
+			      padded,
+			      x_offset, /* dest coords */
+			      y_offset); 
+
+	return padded;
+}
+
