@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2002-2005 Imendio AB
+ * Copyright (C) 2002-2006 Imendio AB
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -18,6 +18,11 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/* Part of this file is copied from GtkSourceView (gtksourceiter.c):
+ *  Copyright (C) 2000 - 2005 Paolo Maggi 
+ *  Copyright (C) 2002, 2003 Jeroen Zwartepoorte
+ */
+
 #include <config.h>
 #include <string.h>
 #include <time.h>
@@ -839,7 +844,6 @@ gossip_pixbuf_from_avatar_scaled (const guchar *avatar,
 
 	data.width = width;
 	data.height = height;
-
 	data.preserve_aspect_ratio = TRUE;
 
 	loader = gdk_pixbuf_loader_new ();
@@ -867,11 +871,11 @@ gossip_pixbuf_from_avatar_scaled (const guchar *avatar,
 
 GdkPixbuf *
 gossip_pixbuf_avatar_from_contact_scaled (GossipContact *contact, 
-					  GtkIconSize    size)
+					  gint           width,
+					  gint           height)
 {
 	const guchar *avatar;
 	gsize         len;
-	gint          width, height;
 
 	g_return_val_if_fail (GOSSIP_IS_CONTACT (contact), NULL);
 
@@ -879,11 +883,7 @@ gossip_pixbuf_avatar_from_contact_scaled (GossipContact *contact,
 	if (!avatar) {
 		return NULL;
 	}
-
-	if (!gtk_icon_size_lookup (size, &width, &height)) {
-		height = width = 48;
-	}
-
+	
 	return gossip_pixbuf_from_avatar_scaled (avatar, len, width, height);
 }
 
@@ -908,3 +908,419 @@ gossip_pixbuf_avatar_from_vcard_scaled (GossipVCard *vcard,
 
 	return gossip_pixbuf_from_avatar_scaled (avatar, len, width, height);
 }
+
+/* Stolen from GtkSourceView, hence the weird intendation. Please keep it like
+ * that to make it easier to apply changes from the original code.
+ */
+#define GTK_TEXT_UNKNOWN_CHAR 0xFFFC
+
+/* this function acts like g_utf8_offset_to_pointer() except that if it finds a
+ * decomposable character it consumes the decomposition length from the given
+ * offset.  So it's useful when the offset was calculated for the normalized
+ * version of str, but we need a pointer to str itself. */
+static const gchar *
+pointer_from_offset_skipping_decomp (const gchar *str, gint offset)
+{
+	gchar *casefold, *normal;
+	const gchar *p, *q;
+
+	p = str;
+	while (offset > 0)
+	{
+		q = g_utf8_next_char (p);
+		casefold = g_utf8_casefold (p, q - p);
+		normal = g_utf8_normalize (casefold, -1, G_NORMALIZE_NFD);
+		offset -= g_utf8_strlen (normal, -1);
+		g_free (casefold);
+		g_free (normal);
+		p = q;
+	}
+	return p;
+}
+
+static const gchar *
+g_utf8_strcasestr (const gchar *haystack, const gchar *needle)
+{
+	gsize needle_len;
+	gsize haystack_len;
+	const gchar *ret = NULL;
+	gchar *p;
+	gchar *casefold;
+	gchar *caseless_haystack;
+	gint i;
+
+	g_return_val_if_fail (haystack != NULL, NULL);
+	g_return_val_if_fail (needle != NULL, NULL);
+
+	casefold = g_utf8_casefold (haystack, -1);
+	caseless_haystack = g_utf8_normalize (casefold, -1, G_NORMALIZE_NFD);
+	g_free (casefold);
+
+	needle_len = g_utf8_strlen (needle, -1);
+	haystack_len = g_utf8_strlen (caseless_haystack, -1);
+
+	if (needle_len == 0)
+	{
+		ret = (gchar *)haystack;
+		goto finally_1;
+	}
+
+	if (haystack_len < needle_len)
+	{
+		ret = NULL;
+		goto finally_1;
+	}
+
+	p = (gchar*)caseless_haystack;
+	needle_len = strlen (needle);
+	i = 0;
+
+	while (*p)
+	{
+		if ((strncmp (p, needle, needle_len) == 0))
+		{
+			ret = pointer_from_offset_skipping_decomp (haystack, i);
+			goto finally_1;
+		}
+
+		p = g_utf8_next_char (p);
+		i++;
+	}
+
+finally_1:
+	g_free (caseless_haystack);
+
+	return ret;
+}
+
+static gboolean
+g_utf8_caselessnmatch (const char *s1, const char *s2,
+		       gssize n1, gssize n2)
+{
+	gchar *casefold;
+	gchar *normalized_s1;
+	gchar *normalized_s2;
+	gint len_s1;
+	gint len_s2;
+	gboolean ret = FALSE;
+
+	g_return_val_if_fail (s1 != NULL, FALSE);
+	g_return_val_if_fail (s2 != NULL, FALSE);
+	g_return_val_if_fail (n1 > 0, FALSE);
+	g_return_val_if_fail (n2 > 0, FALSE);
+
+	casefold = g_utf8_casefold (s1, n1);
+	normalized_s1 = g_utf8_normalize (casefold, -1, G_NORMALIZE_NFD);
+	g_free (casefold);
+
+	casefold = g_utf8_casefold (s2, n2);
+	normalized_s2 = g_utf8_normalize (casefold, -1, G_NORMALIZE_NFD);
+	g_free (casefold);
+
+	len_s1 = strlen (normalized_s1);
+	len_s2 = strlen (normalized_s2);
+
+	if (len_s1 < len_s2)
+		goto finally_2;
+
+	ret = (strncmp (normalized_s1, normalized_s2, len_s2) == 0);
+
+finally_2:
+	g_free (normalized_s1);
+	g_free (normalized_s2);	
+
+	return ret;
+}
+
+static void
+forward_chars_with_skipping (GtkTextIter *iter,
+			     gint         count,
+			     gboolean     skip_invisible,
+			     gboolean     skip_nontext,
+			     gboolean     skip_decomp)
+{
+	gint i;
+
+	g_return_if_fail (count >= 0);
+
+	i = count;
+
+	while (i > 0)
+	{
+		gboolean ignored = FALSE;
+
+		/* minimal workaround to avoid the infinite loop of bug #168247.
+		 * It doesn't fix the problemjust the symptom...
+		 */
+		if (gtk_text_iter_is_end (iter))
+			return;
+
+		if (skip_nontext && gtk_text_iter_get_char (iter) == GTK_TEXT_UNKNOWN_CHAR)
+			ignored = TRUE;
+
+		if (!ignored && skip_invisible &&
+		    /* _gtk_text_btree_char_is_invisible (iter)*/ FALSE)
+			ignored = TRUE;
+
+		if (!ignored && skip_decomp)
+		{
+			/* being UTF8 correct sucks; this accounts for extra
+			   offsets coming from canonical decompositions of
+			   UTF8 characters (e.g. accented characters) which 
+			   g_utf8_normalize() performs */
+			gchar *normal;
+			gchar buffer[6];
+			gint buffer_len;
+
+			buffer_len = g_unichar_to_utf8 (gtk_text_iter_get_char (iter), buffer);
+			normal = g_utf8_normalize (buffer, buffer_len, G_NORMALIZE_NFD);
+			i -= (g_utf8_strlen (normal, -1) - 1);
+			g_free (normal);
+		}
+
+		gtk_text_iter_forward_char (iter);
+
+		if (!ignored)
+			--i;
+	}
+}
+
+static gboolean
+lines_match (const GtkTextIter *start,
+	     const gchar      **lines,
+	     gboolean           visible_only,
+	     gboolean           slice,
+	     GtkTextIter       *match_start,
+	     GtkTextIter       *match_end)
+{
+	GtkTextIter next;
+	gchar *line_text;
+	const gchar *found;
+	gint offset;
+
+	if (*lines == NULL || **lines == '\0')
+	{
+		if (match_start)
+			*match_start = *start;
+		if (match_end)
+			*match_end = *start;
+		return TRUE;
+	}
+
+	next = *start;
+	gtk_text_iter_forward_line (&next);
+
+	/* No more text in buffer, but *lines is nonempty */
+	if (gtk_text_iter_equal (start, &next))
+		return FALSE;
+
+	if (slice)
+	{
+		if (visible_only)
+			line_text = gtk_text_iter_get_visible_slice (start, &next);
+		else
+			line_text = gtk_text_iter_get_slice (start, &next);
+	}
+	else
+	{
+		if (visible_only)
+			line_text = gtk_text_iter_get_visible_text (start, &next);
+		else
+			line_text = gtk_text_iter_get_text (start, &next);
+	}
+
+	if (match_start) /* if this is the first line we're matching */
+	{
+		found = g_utf8_strcasestr (line_text, *lines);
+	}
+	else
+	{
+		/* If it's not the first line, we have to match from the
+		 * start of the line.
+		 */
+		if (g_utf8_caselessnmatch (line_text, *lines, strlen (line_text),
+					   strlen (*lines)))
+			found = line_text;
+		else
+			found = NULL;
+	}
+
+	if (found == NULL)
+	{
+		g_free (line_text);
+		return FALSE;
+	}
+
+	/* Get offset to start of search string */
+	offset = g_utf8_strlen (line_text, found - line_text);
+
+	next = *start;
+
+	/* If match start needs to be returned, set it to the
+	 * start of the search string.
+	 */
+	forward_chars_with_skipping (&next, offset, visible_only, !slice, FALSE);
+	if (match_start)
+	{
+		*match_start = next;
+	}
+
+	/* Go to end of search string */
+	forward_chars_with_skipping (&next, g_utf8_strlen (*lines, -1), visible_only, !slice, TRUE);
+
+	g_free (line_text);
+
+	++lines;
+
+	if (match_end)
+		*match_end = next;
+
+	/* pass NULL for match_start, since we don't need to find the
+	 * start again.
+	 */
+	return lines_match (&next, lines, visible_only, slice, NULL, match_end);
+}
+
+/* strsplit () that retains the delimiter as part of the string. */
+static gchar **
+strbreakup (const char *string,
+	    const char *delimiter,
+	    gint        max_tokens)
+{
+	GSList *string_list = NULL, *slist;
+	gchar **str_array, *s, *casefold, *new_string;
+	guint i, n = 1;
+
+	g_return_val_if_fail (string != NULL, NULL);
+	g_return_val_if_fail (delimiter != NULL, NULL);
+
+	if (max_tokens < 1)
+		max_tokens = G_MAXINT;
+
+	s = strstr (string, delimiter);
+	if (s)
+	{
+		guint delimiter_len = strlen (delimiter);
+
+		do
+		{
+			guint len;
+
+			len = s - string + delimiter_len;
+			new_string = g_new (gchar, len + 1);
+			strncpy (new_string, string, len);
+			new_string[len] = 0;
+			casefold = g_utf8_casefold (new_string, -1);
+			g_free (new_string);
+			new_string = g_utf8_normalize (casefold, -1, G_NORMALIZE_NFD);
+			g_free (casefold);
+			string_list = g_slist_prepend (string_list, new_string);
+			n++;
+			string = s + delimiter_len;
+			s = strstr (string, delimiter);
+		} while (--max_tokens && s);
+	}
+
+	if (*string)
+	{
+		n++;
+		casefold = g_utf8_casefold (string, -1);
+		new_string = g_utf8_normalize (casefold, -1, G_NORMALIZE_NFD);
+		g_free (casefold);
+		string_list = g_slist_prepend (string_list, new_string);
+	}
+
+	str_array = g_new (gchar*, n);
+
+	i = n - 1;
+
+	str_array[i--] = NULL;
+	for (slist = string_list; slist; slist = slist->next)
+		str_array[i--] = slist->data;
+
+	g_slist_free (string_list);
+
+	return str_array;
+}
+
+gboolean
+gossip_text_iter_forward_search (const GtkTextIter   *iter,
+				 const gchar         *str,
+				 GtkTextIter         *match_start,
+				 GtkTextIter         *match_end,
+				 const GtkTextIter   *limit)
+{
+	gchar **lines = NULL;
+	GtkTextIter match;
+	gboolean retval = FALSE;
+	GtkTextIter search;
+	gboolean visible_only;
+	gboolean slice;
+
+	g_return_val_if_fail (iter != NULL, FALSE);
+	g_return_val_if_fail (str != NULL, FALSE);
+
+	if (limit && gtk_text_iter_compare (iter, limit) >= 0)
+		return FALSE;
+
+	if (*str == '\0') {
+		/* If we can move one char, return the empty string there */
+		match = *iter;
+
+		if (gtk_text_iter_forward_char (&match)) {
+			if (limit && gtk_text_iter_equal (&match, limit)) {
+				return FALSE;
+			}
+
+			if (match_start) {
+				*match_start = match;
+			}
+			if (match_end) {
+				*match_end = match;
+			}
+			return TRUE;
+		} else {
+			return FALSE;
+		}
+	}
+	
+	visible_only = TRUE;
+	slice = FALSE;
+
+	/* locate all lines */
+	lines = strbreakup (str, "\n", -1);
+
+	search = *iter;
+
+	do {
+		/* This loop has an inefficient worst-case, where
+		 * gtk_text_iter_get_text () is called repeatedly on
+		 * a single line.
+		 */
+		GtkTextIter end;
+
+		if (limit && gtk_text_iter_compare (&search, limit) >= 0) {
+			break;
+		}
+
+		if (lines_match (&search, (const gchar**)lines,
+				 visible_only, slice, &match, &end)) {
+			if (limit == NULL ||
+			    (limit && gtk_text_iter_compare (&end, limit) <= 0)) {
+				retval = TRUE;
+
+				if (match_start) {
+					*match_start = match;
+				}
+				if (match_end) {
+					*match_end = end;
+				}
+			}
+			break;
+		}
+	} while (gtk_text_iter_forward_line (&search));
+
+	g_strfreev ((gchar**)lines);
+
+	return retval;
+}
